; CPack install script designed for a nmake build

;--------------------------------
; You must define these values

  !define VERSION "0.11.1"
  !define PATCH  "1"
  !define INST_DIR "C:/Local/dev/gitlab-runner/builds/a913a013/0/s3a/VISR/build_py36/_CPack_Packages/win64/NSIS/VISR-0.11.1-python3.6-Windows"

;--------------------------------
;Variables

  Var MUI_TEMP
  Var STARTMENU_FOLDER
  Var SV_ALLUSERS
  Var START_MENU
  Var DO_NOT_ADD_TO_PATH
  Var ADD_TO_PATH_ALL_USERS
  Var ADD_TO_PATH_CURRENT_USER
  Var INSTALL_DESKTOP
  Var IS_DEFAULT_INSTALLDIR
;--------------------------------
;Include Modern UI

  !include "MUI.nsh"

  ;Default installation folder
  InstallDir "$PROGRAMFILES64\VISR-0.11.1"

;--------------------------------
;General

  ;Name and file
  Name "VISR-0.11.1"
  OutFile "C:/Local/dev/gitlab-runner/builds/a913a013/0/s3a/VISR/build_py36/_CPack_Packages/win64/NSIS/VISR-0.11.1-python3.6-Windows.exe"

  ;Set compression
  SetCompressor lzma

  ;Require administrator access
  RequestExecutionLevel admin



  !include Sections.nsh

;--- Component support macros: ---
; The code for the add/remove functionality is from:
;   http://nsis.sourceforge.net/Add/Remove_Functionality
; It has been modified slightly and extended to provide
; inter-component dependencies.
Var AR_SecFlags
Var AR_RegFlags
Var api_documentation_pdf_selected
Var api_documentation_pdf_was_installed
Var base_selected
Var base_was_installed
Var development_files_selected
Var development_files_was_installed
Var doxygen_documentation_html_selected
Var doxygen_documentation_html_was_installed
Var loudspeaker_configs_selected
Var loudspeaker_configs_was_installed
Var python_externals_selected
Var python_externals_was_installed
Var python_package_bst_selected
Var python_package_bst_was_installed
Var python_package_metadapter_selected
Var python_package_metadapter_was_installed
Var python_templates_selected
Var python_templates_was_installed
Var shared_libraries_selected
Var shared_libraries_was_installed
Var standalone_applications_selected
Var standalone_applications_was_installed
Var thirdparty_libraries_selected
Var thirdparty_libraries_was_installed
Var user_documentation_pdf_selected
Var user_documentation_pdf_was_installed


; Loads the "selected" flag for the section named SecName into the
; variable VarName.
!macro LoadSectionSelectedIntoVar SecName VarName
 SectionGetFlags ${${SecName}} $${VarName}
 IntOp $${VarName} $${VarName} & ${SF_SELECTED}  ;Turn off all other bits
!macroend

; Loads the value of a variable... can we get around this?
!macro LoadVar VarName
  IntOp $R0 0 + $${VarName}
!macroend

; Sets the value of a variable
!macro StoreVar VarName IntValue
  IntOp $${VarName} 0 + ${IntValue}
!macroend

!macro InitSection SecName
  ;  This macro reads component installed flag from the registry and
  ;changes checked state of the section on the components page.
  ;Input: section index constant name specified in Section command.

  ClearErrors
  ;Reading component status from registry
  ReadRegDWORD $AR_RegFlags HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\VISR-0.11.1\Components\${SecName}" "Installed"
  IfErrors "default_${SecName}"
    ;Status will stay default if registry value not found
    ;(component was never installed)
  IntOp $AR_RegFlags $AR_RegFlags & ${SF_SELECTED} ;Turn off all other bits
  SectionGetFlags ${${SecName}} $AR_SecFlags  ;Reading default section flags
  IntOp $AR_SecFlags $AR_SecFlags & 0xFFFE  ;Turn lowest (enabled) bit off
  IntOp $AR_SecFlags $AR_RegFlags | $AR_SecFlags      ;Change lowest bit

  ; Note whether this component was installed before
  !insertmacro StoreVar ${SecName}_was_installed $AR_RegFlags
  IntOp $R0 $AR_RegFlags & $AR_RegFlags

  ;Writing modified flags
  SectionSetFlags ${${SecName}} $AR_SecFlags

 "default_${SecName}:"
 !insertmacro LoadSectionSelectedIntoVar ${SecName} ${SecName}_selected
!macroend

!macro FinishSection SecName
  ;  This macro reads section flag set by user and removes the section
  ;if it is not selected.
  ;Then it writes component installed flag to registry
  ;Input: section index constant name specified in Section command.

  SectionGetFlags ${${SecName}} $AR_SecFlags  ;Reading section flags
  ;Checking lowest bit:
  IntOp $AR_SecFlags $AR_SecFlags & ${SF_SELECTED}
  IntCmp $AR_SecFlags 1 "leave_${SecName}"
    ;Section is not selected:
    ;Calling Section uninstall macro and writing zero installed flag
    !insertmacro "Remove_${${SecName}}"
    WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\VISR-0.11.1\Components\${SecName}" \
  "Installed" 0
    Goto "exit_${SecName}"

 "leave_${SecName}:"
    ;Section is selected:
    WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\VISR-0.11.1\Components\${SecName}" \
  "Installed" 1

 "exit_${SecName}:"
!macroend

!macro RemoveSection_CPack SecName
  ;  This macro is used to call section's Remove_... macro
  ;from the uninstaller.
  ;Input: section index constant name specified in Section command.

  !insertmacro "Remove_${${SecName}}"
!macroend

; Determine whether the selection of SecName changed
!macro MaybeSelectionChanged SecName
  !insertmacro LoadVar ${SecName}_selected
  SectionGetFlags ${${SecName}} $R1
  IntOp $R1 $R1 & ${SF_SELECTED} ;Turn off all other bits

  ; See if the status has changed:
  IntCmp $R0 $R1 "${SecName}_unchanged"
  !insertmacro LoadSectionSelectedIntoVar ${SecName} ${SecName}_selected

  IntCmp $R1 ${SF_SELECTED} "${SecName}_was_selected"
  !insertmacro "Deselect_required_by_${SecName}"
  goto "${SecName}_unchanged"

  "${SecName}_was_selected:"
  !insertmacro "Select_${SecName}_depends"

  "${SecName}_unchanged:"
!macroend
;--- End of Add/Remove macros ---

;--------------------------------
;Interface Settings

  !define MUI_HEADERIMAGE
  !define MUI_ABORTWARNING

;----------------------------------------
; based upon a script of "Written by KiCHiK 2003-01-18 05:57:02"
;----------------------------------------
!verbose 3
!include "WinMessages.NSH"
!verbose 4
;====================================================
; get_NT_environment
;     Returns: the selected environment
;     Output : head of the stack
;====================================================
!macro select_NT_profile UN
Function ${UN}select_NT_profile
   StrCmp $ADD_TO_PATH_ALL_USERS "1" 0 environment_single
      DetailPrint "Selected environment for all users"
      Push "all"
      Return
   environment_single:
      DetailPrint "Selected environment for current user only."
      Push "current"
      Return
FunctionEnd
!macroend
!insertmacro select_NT_profile ""
!insertmacro select_NT_profile "un."
;----------------------------------------------------
!define NT_current_env 'HKCU "Environment"'
!define NT_all_env     'HKLM "SYSTEM\CurrentControlSet\Control\Session Manager\Environment"'

!ifndef WriteEnvStr_RegKey
  !ifdef ALL_USERS
    !define WriteEnvStr_RegKey \
       'HKLM "SYSTEM\CurrentControlSet\Control\Session Manager\Environment"'
  !else
    !define WriteEnvStr_RegKey 'HKCU "Environment"'
  !endif
!endif

; AddToPath - Adds the given dir to the search path.
;        Input - head of the stack
;        Note - Win9x systems requires reboot

Function AddToPath
  Exch $0
  Push $1
  Push $2
  Push $3

  # don't add if the path doesn't exist
  IfFileExists "$0\*.*" "" AddToPath_done

  ReadEnvStr $1 PATH
  ; if the path is too long for a NSIS variable NSIS will return a 0
  ; length string.  If we find that, then warn and skip any path
  ; modification as it will trash the existing path.
  StrLen $2 $1
  IntCmp $2 0 CheckPathLength_ShowPathWarning CheckPathLength_Done CheckPathLength_Done
    CheckPathLength_ShowPathWarning:
    Messagebox MB_OK|MB_ICONEXCLAMATION "Warning! PATH too long installer unable to modify PATH!"
    Goto AddToPath_done
  CheckPathLength_Done:
  Push "$1;"
  Push "$0;"
  Call StrStr
  Pop $2
  StrCmp $2 "" "" AddToPath_done
  Push "$1;"
  Push "$0\;"
  Call StrStr
  Pop $2
  StrCmp $2 "" "" AddToPath_done
  GetFullPathName /SHORT $3 $0
  Push "$1;"
  Push "$3;"
  Call StrStr
  Pop $2
  StrCmp $2 "" "" AddToPath_done
  Push "$1;"
  Push "$3\;"
  Call StrStr
  Pop $2
  StrCmp $2 "" "" AddToPath_done

  Call IsNT
  Pop $1
  StrCmp $1 1 AddToPath_NT
    ; Not on NT
    StrCpy $1 $WINDIR 2
    FileOpen $1 "$1\autoexec.bat" a
    FileSeek $1 -1 END
    FileReadByte $1 $2
    IntCmp $2 26 0 +2 +2 # DOS EOF
      FileSeek $1 -1 END # write over EOF
    FileWrite $1 "$\r$\nSET PATH=%PATH%;$3$\r$\n"
    FileClose $1
    SetRebootFlag true
    Goto AddToPath_done

  AddToPath_NT:
    StrCmp $ADD_TO_PATH_ALL_USERS "1" ReadAllKey
      ReadRegStr $1 ${NT_current_env} "PATH"
      Goto DoTrim
    ReadAllKey:
      ReadRegStr $1 ${NT_all_env} "PATH"
    DoTrim:
    StrCmp $1 "" AddToPath_NTdoIt
      Push $1
      Call Trim
      Pop $1
      StrCpy $0 "$1;$0"
    AddToPath_NTdoIt:
      StrCmp $ADD_TO_PATH_ALL_USERS "1" WriteAllKey
        WriteRegExpandStr ${NT_current_env} "PATH" $0
        Goto DoSend
      WriteAllKey:
        WriteRegExpandStr ${NT_all_env} "PATH" $0
      DoSend:
      SendMessage ${HWND_BROADCAST} ${WM_WININICHANGE} 0 "STR:Environment" /TIMEOUT=5000

  AddToPath_done:
    Pop $3
    Pop $2
    Pop $1
    Pop $0
FunctionEnd


; RemoveFromPath - Remove a given dir from the path
;     Input: head of the stack

Function un.RemoveFromPath
  Exch $0
  Push $1
  Push $2
  Push $3
  Push $4
  Push $5
  Push $6

  IntFmt $6 "%c" 26 # DOS EOF

  Call un.IsNT
  Pop $1
  StrCmp $1 1 unRemoveFromPath_NT
    ; Not on NT
    StrCpy $1 $WINDIR 2
    FileOpen $1 "$1\autoexec.bat" r
    GetTempFileName $4
    FileOpen $2 $4 w
    GetFullPathName /SHORT $0 $0
    StrCpy $0 "SET PATH=%PATH%;$0"
    Goto unRemoveFromPath_dosLoop

    unRemoveFromPath_dosLoop:
      FileRead $1 $3
      StrCpy $5 $3 1 -1 # read last char
      StrCmp $5 $6 0 +2 # if DOS EOF
        StrCpy $3 $3 -1 # remove DOS EOF so we can compare
      StrCmp $3 "$0$\r$\n" unRemoveFromPath_dosLoopRemoveLine
      StrCmp $3 "$0$\n" unRemoveFromPath_dosLoopRemoveLine
      StrCmp $3 "$0" unRemoveFromPath_dosLoopRemoveLine
      StrCmp $3 "" unRemoveFromPath_dosLoopEnd
      FileWrite $2 $3
      Goto unRemoveFromPath_dosLoop
      unRemoveFromPath_dosLoopRemoveLine:
        SetRebootFlag true
        Goto unRemoveFromPath_dosLoop

    unRemoveFromPath_dosLoopEnd:
      FileClose $2
      FileClose $1
      StrCpy $1 $WINDIR 2
      Delete "$1\autoexec.bat"
      CopyFiles /SILENT $4 "$1\autoexec.bat"
      Delete $4
      Goto unRemoveFromPath_done

  unRemoveFromPath_NT:
    StrCmp $ADD_TO_PATH_ALL_USERS "1" unReadAllKey
      ReadRegStr $1 ${NT_current_env} "PATH"
      Goto unDoTrim
    unReadAllKey:
      ReadRegStr $1 ${NT_all_env} "PATH"
    unDoTrim:
    StrCpy $5 $1 1 -1 # copy last char
    StrCmp $5 ";" +2 # if last char != ;
      StrCpy $1 "$1;" # append ;
    Push $1
    Push "$0;"
    Call un.StrStr ; Find `$0;` in $1
    Pop $2 ; pos of our dir
    StrCmp $2 "" unRemoveFromPath_done
      ; else, it is in path
      # $0 - path to add
      # $1 - path var
      StrLen $3 "$0;"
      StrLen $4 $2
      StrCpy $5 $1 -$4 # $5 is now the part before the path to remove
      StrCpy $6 $2 "" $3 # $6 is now the part after the path to remove
      StrCpy $3 $5$6

      StrCpy $5 $3 1 -1 # copy last char
      StrCmp $5 ";" 0 +2 # if last char == ;
        StrCpy $3 $3 -1 # remove last char

      StrCmp $ADD_TO_PATH_ALL_USERS "1" unWriteAllKey
        WriteRegExpandStr ${NT_current_env} "PATH" $3
        Goto unDoSend
      unWriteAllKey:
        WriteRegExpandStr ${NT_all_env} "PATH" $3
      unDoSend:
      SendMessage ${HWND_BROADCAST} ${WM_WININICHANGE} 0 "STR:Environment" /TIMEOUT=5000

  unRemoveFromPath_done:
    Pop $6
    Pop $5
    Pop $4
    Pop $3
    Pop $2
    Pop $1
    Pop $0
FunctionEnd

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Uninstall sutff
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

###########################################
#            Utility Functions            #
###########################################

;====================================================
; IsNT - Returns 1 if the current system is NT, 0
;        otherwise.
;     Output: head of the stack
;====================================================
; IsNT
; no input
; output, top of the stack = 1 if NT or 0 if not
;
; Usage:
;   Call IsNT
;   Pop $R0
;  ($R0 at this point is 1 or 0)

!macro IsNT un
Function ${un}IsNT
  Push $0
  ReadRegStr $0 HKLM "SOFTWARE\Microsoft\Windows NT\CurrentVersion" CurrentVersion
  StrCmp $0 "" 0 IsNT_yes
  ; we are not NT.
  Pop $0
  Push 0
  Return

  IsNT_yes:
    ; NT!!!
    Pop $0
    Push 1
FunctionEnd
!macroend
!insertmacro IsNT ""
!insertmacro IsNT "un."

; StrStr
; input, top of stack = string to search for
;        top of stack-1 = string to search in
; output, top of stack (replaces with the portion of the string remaining)
; modifies no other variables.
;
; Usage:
;   Push "this is a long ass string"
;   Push "ass"
;   Call StrStr
;   Pop $R0
;  ($R0 at this point is "ass string")

!macro StrStr un
Function ${un}StrStr
Exch $R1 ; st=haystack,old$R1, $R1=needle
  Exch    ; st=old$R1,haystack
  Exch $R2 ; st=old$R1,old$R2, $R2=haystack
  Push $R3
  Push $R4
  Push $R5
  StrLen $R3 $R1
  StrCpy $R4 0
  ; $R1=needle
  ; $R2=haystack
  ; $R3=len(needle)
  ; $R4=cnt
  ; $R5=tmp
  loop:
    StrCpy $R5 $R2 $R3 $R4
    StrCmp $R5 $R1 done
    StrCmp $R5 "" done
    IntOp $R4 $R4 + 1
    Goto loop
done:
  StrCpy $R1 $R2 "" $R4
  Pop $R5
  Pop $R4
  Pop $R3
  Pop $R2
  Exch $R1
FunctionEnd
!macroend
!insertmacro StrStr ""
!insertmacro StrStr "un."

Function Trim ; Added by Pelaca
	Exch $R1
	Push $R2
Loop:
	StrCpy $R2 "$R1" 1 -1
	StrCmp "$R2" " " RTrim
	StrCmp "$R2" "$\n" RTrim
	StrCmp "$R2" "$\r" RTrim
	StrCmp "$R2" ";" RTrim
	GoTo Done
RTrim:
	StrCpy $R1 "$R1" -1
	Goto Loop
Done:
	Pop $R2
	Exch $R1
FunctionEnd

Function ConditionalAddToRegisty
  Pop $0
  Pop $1
  StrCmp "$0" "" ConditionalAddToRegisty_EmptyString
    WriteRegStr SHCTX "Software\Microsoft\Windows\CurrentVersion\Uninstall\VISR-0.11.1" \
    "$1" "$0"
    ;MessageBox MB_OK "Set Registry: '$1' to '$0'"
    DetailPrint "Set install registry entry: '$1' to '$0'"
  ConditionalAddToRegisty_EmptyString:
FunctionEnd

;--------------------------------

!ifdef CPACK_USES_DOWNLOAD
Function DownloadFile
    IfFileExists $INSTDIR\* +2
    CreateDirectory $INSTDIR
    Pop $0

    ; Skip if already downloaded
    IfFileExists $INSTDIR\$0 0 +2
    Return

    StrCpy $1 ""

  try_again:
    NSISdl::download "$1/$0" "$INSTDIR\$0"

    Pop $1
    StrCmp $1 "success" success
    StrCmp $1 "Cancelled" cancel
    MessageBox MB_OK "Download failed: $1"
  cancel:
    Return
  success:
FunctionEnd
!endif

;--------------------------------
; Installation types
InstType "Default"
InstType "Developer"
InstType "Full"
InstType "Python developer"


;--------------------------------
; Component sections
SectionGroup "Documentation" documentation
Section "API documentation (PDF)" api_documentation_pdf
  SectionIn 2 3
  SetOutPath "$INSTDIR"
  File /r "${INST_DIR}\api_documentation_pdf\*.*"
SectionEnd
Section /o "Doxygen code documentation" doxygen_documentation_html
  SectionIn 3
  SetOutPath "$INSTDIR"
  File /r "${INST_DIR}\doxygen_documentation_html\*.*"
SectionEnd
Section "User documentation (PDF)" user_documentation_pdf
  SectionIn 1 2 3
  SetOutPath "$INSTDIR"
  File /r "${INST_DIR}\user_documentation_pdf\*.*"
SectionEnd
SectionGroupEnd
SectionGroup "Python packages" python_packages
Section "Binaural Synthesis Toolkit" python_package_bst
  SectionIn 1 4 3
  SetOutPath "$INSTDIR"
  File /r "${INST_DIR}\python_package_bst\*.*"
SectionEnd
Section "Metadapter" python_package_metadapter
  SectionIn 1 4 3
  SetOutPath "$INSTDIR"
  File /r "${INST_DIR}\python_package_metadapter\*.*"
SectionEnd
SectionGroupEnd
Section "-Base" base
  SectionIn RO
  SetOutPath "$INSTDIR"
  File /r "${INST_DIR}\base\*.*"
SectionEnd
Section "Development files" development_files
  SectionIn 2 3
  SetOutPath "$INSTDIR"
  File /r "${INST_DIR}\development_files\*.*"
SectionEnd
Section "Loudspeaker configuration files" loudspeaker_configs
  SectionIn 1 2 3
  SetOutPath "$INSTDIR"
  File /r "${INST_DIR}\loudspeaker_configs\*.*"
SectionEnd
Section "Python Externals" python_externals
  SectionIn 1 4 3
  SetOutPath "$INSTDIR"
  File /r "${INST_DIR}\python_externals\*.*"
SectionEnd
Section "Python Templates" python_templates
  SectionIn 4 3
  SetOutPath "$INSTDIR"
  File /r "${INST_DIR}\python_templates\*.*"
SectionEnd
Section "Shared Libraries" shared_libraries
  SectionIn RO
  SetOutPath "$INSTDIR"
  File /r "${INST_DIR}\shared_libraries\*.*"
SectionEnd
Section "Standalone applications" standalone_applications
  SectionIn 1 3
  SetOutPath "$INSTDIR"
  File /r "${INST_DIR}\standalone_applications\*.*"
SectionEnd
Section "-Third-party libraries" thirdparty_libraries
  SectionIn RO
  SetOutPath "$INSTDIR"
  File /r "${INST_DIR}\thirdparty_libraries\*.*"
SectionEnd
!macro Remove_${api_documentation_pdf}
  IntCmp $api_documentation_pdf_was_installed 0 noremove_api_documentation_pdf
  Delete "$INSTDIR\doc"
  Delete "$INSTDIR\doc\apidoc-0.11.1.pdf"
  RMDir "$INSTDIR\doc"
  noremove_api_documentation_pdf:
!macroend
!macro Select_api_documentation_pdf_depends
!macroend
!macro Deselect_required_by_api_documentation_pdf
!macroend
!macro Remove_${doxygen_documentation_html}
  IntCmp $doxygen_documentation_html_was_installed 0 noremove_doxygen_documentation_html
  Delete "$INSTDIR\doc"
  Delete "$INSTDIR\doc\html"
  Delete "$INSTDIR\doc\html\_all_r_a_d_8cpp.html"
  Delete "$INSTDIR\doc\html\_all_r_a_d_8h.html"
  Delete "$INSTDIR\doc\html\_all_r_a_d_8h_source.html"
  Delete "$INSTDIR\doc\html\_c_a_p_8cpp.html"
  Delete "$INSTDIR\doc\html\_c_a_p_8h.html"
  Delete "$INSTDIR\doc\html\_c_a_p_8h_source.html"
  Delete "$INSTDIR\doc\html\_loudspeaker_array_8cpp.html"
  Delete "$INSTDIR\doc\html\_loudspeaker_array_8h.html"
  Delete "$INSTDIR\doc\html\_loudspeaker_array_8h_source.html"
  Delete "$INSTDIR\doc\html\_v_b_a_p_8h.html"
  Delete "$INSTDIR\doc\html\_v_b_a_p_8h_source.html"
  Delete "$INSTDIR\doc\html\_x_y_z_8h.html"
  Delete "$INSTDIR\doc\html\_x_y_z_8h_source.html"
  Delete "$INSTDIR\doc\html\add_8hpp.html"
  Delete "$INSTDIR\doc\html\add_8hpp_source.html"
  Delete "$INSTDIR\doc\html\aligned__array_8hpp.html"
  Delete "$INSTDIR\doc\html\aligned__array_8hpp_source.html"
  Delete "$INSTDIR\doc\html\alignment_8hpp.html"
  Delete "$INSTDIR\doc\html\alignment_8hpp.js"
  Delete "$INSTDIR\doc\html\alignment_8hpp_source.html"
  Delete "$INSTDIR\doc\html\annotated.html"
  Delete "$INSTDIR\doc\html\annotated_dup.js"
  Delete "$INSTDIR\doc\html\apps_2baseline__renderer_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2baseline__renderer_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2baseline__renderer_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2feedthrough_2signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2feedthrough_2signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2feedthrough_2signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2gain__matrix_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2gain__matrix_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2gain__matrix_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2init__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2init__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2init__filter__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2init__filter__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2init__filter__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2network__audio__streamer_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2network__audio__streamer_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2network__audio__streamer_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2network__audio__streamer_2signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2network__audio__streamer_2signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2network__audio__streamer_2signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2python__runner_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2python__runner_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2python__runner_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2scene__decoder_2signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2scene__decoder_2signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2scene__decoder_2signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2visr__renderer_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2visr__renderer_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2visr__renderer_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\argument__list_8cpp.html"
  Delete "$INSTDIR\doc\html\argument__list_8hpp.html"
  Delete "$INSTDIR\doc\html\argument__list_8hpp_source.html"
  Delete "$INSTDIR\doc\html\array__configuration_8cpp.html"
  Delete "$INSTDIR\doc\html\array__configuration_8hpp.html"
  Delete "$INSTDIR\doc\html\array__configuration_8hpp_source.html"
  Delete "$INSTDIR\doc\html\atomic__component_8hpp.html"
  Delete "$INSTDIR\doc\html\atomic__component_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__connection_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__connection_8cpp.js"
  Delete "$INSTDIR\doc\html\audio__connection__descriptor_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__connection__descriptor_8hpp.js"
  Delete "$INSTDIR\doc\html\audio__connection__descriptor_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__connection__map_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__connection__map_8cpp.js"
  Delete "$INSTDIR\doc\html\audio__connection__map_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__connection__map_8hpp.js"
  Delete "$INSTDIR\doc\html\audio__connection__map_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__input_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__input_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__input_8hpp.js"
  Delete "$INSTDIR\doc\html\audio__input_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__interface_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__interface_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__interface__configuration_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__interface__configuration_8cpp.js"
  Delete "$INSTDIR\doc\html\audio__interface__factory_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__interface__factory_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__network__encoder_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__network__encoder_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__network__encoder_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__object__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__object__parser_8cpp.js"
  Delete "$INSTDIR\doc\html\audio__output_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__output_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__output_8hpp.js"
  Delete "$INSTDIR\doc\html\audio__output_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__port_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__port_8cpp.js"
  Delete "$INSTDIR\doc\html\audio__port__base_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__port__base_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__port__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__port__base__implementation_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__port__base__implementation_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__port__base__implementation_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__sample__type_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__sample__type_8hpp.js"
  Delete "$INSTDIR\doc\html\audio__sample__type_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__signal__flow__checking_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__signal__flow__checking_8cpp.js"
  Delete "$INSTDIR\doc\html\audiointerfaces_8cpp.html"
  Delete "$INSTDIR\doc\html\audiointerfaces_8cpp.js"
  Delete "$INSTDIR\doc\html\baseline__renderer_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\baseline__renderer_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\baseline__renderer__usage_8dox.html"
  Delete "$INSTDIR\doc\html\baseline_renderer_usage.html"
  Delete "$INSTDIR\doc\html\basic__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\basic__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\basic__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\basic__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\bc_s.png"
  Delete "$INSTDIR\doc\html\bdwn.png"
  Delete "$INSTDIR\doc\html\biquad__coefficient_8hpp.html"
  Delete "$INSTDIR\doc\html\biquad__coefficient_8hpp_source.html"
  Delete "$INSTDIR\doc\html\biquad__iir__filter_8hpp.html"
  Delete "$INSTDIR\doc\html\biquad__iir__filter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\biquad__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\biquad__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\biquad__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\build_8dox.html"
  Delete "$INSTDIR\doc\html\build_instructions.html"
  Delete "$INSTDIR\doc\html\build_instructions_max_msp_externals.html"
  Delete "$INSTDIR\doc\html\bunch__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\bunch__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\bunch__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\cap__gain__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\cap__gain__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\cartesian__spherical__conversion_8hpp.html"
  Delete "$INSTDIR\doc\html\cartesian__spherical__conversion_8hpp.js"
  Delete "$INSTDIR\doc\html\cartesian__spherical__conversion_8hpp_source.html"
  Delete "$INSTDIR\doc\html\channel__list_8hpp.html"
  Delete "$INSTDIR\doc\html\channel__list_8hpp.js"
  Delete "$INSTDIR\doc\html\channel__list_8hpp_source.html"
  Delete "$INSTDIR\doc\html\channel__object_8hpp.html"
  Delete "$INSTDIR\doc\html\channel__object_8hpp_source.html"
  Delete "$INSTDIR\doc\html\channel__object__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\channel__object__parser_8cpp.js"
  Delete "$INSTDIR\doc\html\channel__object__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\channel__object__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\channel__object__routing__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\channel__object__routing__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\circular__buffer_8cpp.html"
  Delete "$INSTDIR\doc\html\circular__buffer_8hpp.html"
  Delete "$INSTDIR\doc\html\circular__buffer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\class__registrar_8hpp.html"
  Delete "$INSTDIR\doc\html\class__registrar_8hpp_source.html"
  Delete "$INSTDIR\doc\html\classes.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_atomic_component-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_atomic_component.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_atomic_component.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_atomic_component.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_t-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_t.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_t.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_t.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_t-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_t.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_t.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_t.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_port_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_port_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_port_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_port_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_channel_list-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_channel_list.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_channel_list.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_channel_range-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_channel_range.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_channel_range.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_input-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_input.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_input.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_input.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_output-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_output.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_output.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_output.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_factory-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_factory.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_factory.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_component-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_component.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_component.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_component.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_composite_component-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_composite_component.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_composite_component.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_composite_component.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_config_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_config_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_config_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_config_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_factory-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_factory.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_factory.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_factory_1_1_registrar-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_factory_1_1_registrar.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_factory_1_1_registrar.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_port_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_port_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_port_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_port_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_input-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_input.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_input.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_input.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_output-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_output.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_output.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_output.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_port_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_port_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_port_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_signal_flow_context-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_signal_flow_context.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_signal_flow_context.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_status_message-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_status_message.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_status_message.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_typed_parameter_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_typed_parameter_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_typed_parameter_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_typed_parameter_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_audio_network_encoder-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_audio_network_encoder.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_audio_network_encoder.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_audio_network_encoder.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_options.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_signal_flow-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_signal_flow.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_signal_flow.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_signal_flow.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1baseline__renderer_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1baseline__renderer_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1baseline__renderer_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1baseline__renderer_1_1_options.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1feedthrough_1_1_feedthrough-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1feedthrough_1_1_feedthrough.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1feedthrough_1_1_feedthrough.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1feedthrough_1_1_feedthrough.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1gain__matrix_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1gain__matrix_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1gain__matrix_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1matrix__convolver_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1matrix__convolver_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1matrix__convolver_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1matrix__convolver_1_1_options.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1pythonflowrunner_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1pythonflowrunner_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1pythonflowrunner_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1pythonflowrunner_1_1_options.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1scene__decoder_1_1_signal_flow-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1scene__decoder_1_1_signal_flow.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1scene__decoder_1_1_signal_flow.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1scene__decoder_1_1_signal_flow.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apputilities_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apputilities_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apputilities_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apputilities_1_1_options.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_1_1_configuration-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_1_1_configuration.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_1_1_configuration.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_factory-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_factory.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_factory.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_factory_1_1_registrar-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_factory_1_1_registrar.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_factory_1_1_registrar.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_jack_interface-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_jack_interface.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_jack_interface.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_jack_interface.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_jack_interface_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_jack_interface_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_jack_interface_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_config_1_1_sample_format-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_config_1_1_sample_format.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_config_1_1_sample_format.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1test_1_1_feedthrough-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1test_1_1_feedthrough.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1test_1_1_feedthrough.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1test_1_1_feedthrough.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_aligned_array-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_aligned_array.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_aligned_array.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_vector-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_vector.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_vector.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_vector.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_lagrange_coefficient_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_lagrange_coefficient_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_lagrange_coefficient_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_audio_port_base_implementation-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_audio_port_base_implementation.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_audio_port_base_implementation.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_audio_port_base_implementation.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_component_implementation-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_component_implementation.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_component_implementation.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_component_implementation.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_composite_component_implementation-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_composite_component_implementation.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_composite_component_implementation.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_composite_component_implementation.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_parameter_port_base_implementation-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_parameter_port_base_implementation.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_parameter_port_base_implementation.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_parameter_port_base_implementation.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_port_base_implementation-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_port_base_implementation.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_port_base_implementation.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_port_base_implementation.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1test_1_1_test_atom-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1test_1_1_test_atom.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1test_1_1_test_atom.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1test_1_1_test_atom.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_argument_list-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_argument_list.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_argument_list.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_class_registrar-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_class_registrar.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_class_registrar.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_delay_vector-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_delay_vector.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_delay_vector.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_delay_vector.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_external_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_external_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_external_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_external_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_external_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_external_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_external_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_gain_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_gain_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_gain_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_gain_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_pml_initialiser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_pml_initialiser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_pml_initialiser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_signal_flow_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_signal_flow_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_signal_flow_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_matrix_convolver-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_matrix_convolver.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_matrix_convolver.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_matrix_convolver.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_options.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_options.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_visr_renderer-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_visr_renderer.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_visr_renderer.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_visr_renderer.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1feedthrough_1_1_signal_flow-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1feedthrough_1_1_signal_flow.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1feedthrough_1_1_signal_flow.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1feedthrough_1_1_signal_flow.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1gain__matrix_1_1_signal_flow-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1gain__matrix_1_1_signal_flow.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1gain__matrix_1_1_signal_flow.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1gain__matrix_1_1_signal_flow.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1late__reverb__filter__calculator_1_1_signal_flow-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1late__reverb__filter__calculator_1_1_signal_flow.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1late__reverb__filter__calculator_1_1_signal_flow.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1late__reverb__filter__calculator_1_1_signal_flow.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1mexsupport_1_1_mex_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mexsupport_1_1_mex_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mexsupport_1_1_mex_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_factory-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_factory.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_1_1const__iterator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_1_1const__iterator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_1_1const__iterator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_1_1iterator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_1_1iterator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_1_1iterator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_1_1_discrete_reflection-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_1_1_discrete_reflection.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_1_1_discrete_reflection.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_1_1_late_reverb-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_1_1_late_reverb.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_1_1_late_reverb.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_all_r_a_d-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_all_r_a_d.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_all_r_a_d.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_c_a_p-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_c_a_p.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_c_a_p.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_loudspeaker_array-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_loudspeaker_array.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_loudspeaker_array.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_v_b_a_p-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_v_b_a_p.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_v_b_a_p.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_x_y_z-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_x_y_z.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_x_y_z.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_array_configuration-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_array_configuration.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_array_configuration.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_biquad_parameter_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_biquad_parameter_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_biquad_parameter_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_biquad_parameter_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_input-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_input.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_input.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_input.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_output-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_output.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_output.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_output.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_empty_parameter_config-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_empty_parameter_config.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_empty_parameter_config.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_empty_parameter_config.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_filter_routing_list_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_filter_routing_list_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_filter_routing_list_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_filter_routing_list_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_indexed_value_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_indexed_value_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_indexed_value_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_indexed_value_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter_config-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter_config.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter_config.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter_config.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_listener_position-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_listener_position.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_listener_position.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_listener_position.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter_config-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter_config.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter_config.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter_config.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_object_vector-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_object_vector.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_object_vector.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_object_vector.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_scalar_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_scalar_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_scalar_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_scalar_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_signal_routing_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_signal_routing_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_signal_routing_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_signal_routing_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_signal_routing_parameter_1_1_compare_entries-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_signal_routing_parameter_1_1_compare_entries.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_signal_routing_parameter_1_1_compare_entries.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_list_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_list_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_list_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_list_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_string_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_string_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_string_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_string_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter_config-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter_config.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter_config.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter_config.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter_config-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter_config.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter_config.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter_config.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_atomic_component_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_atomic_component_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_atomic_component_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_atomic_component_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_communication_protocol_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_communication_protocol_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_communication_protocol_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_communication_protocol_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_composite_component_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_composite_component_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_composite_component_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_composite_component_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_base_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_base_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_base_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_base_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_config_base_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_config_base_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_config_base_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_config_base_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythoncomponents_1_1_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythoncomponents_1_1_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythoncomponents_1_1_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythoncomponents_1_1_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythoncomponents_1_1_wrapper_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythoncomponents_1_1_wrapper_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythoncomponents_1_1_wrapper_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_gil_ensure_guard-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_gil_ensure_guard.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_gil_ensure_guard.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_gil_ensure_guard_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_gil_ensure_guard_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_gil_ensure_guard_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_initialisation_guard-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_initialisation_guard.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_initialisation_guard.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_initialisation_guard_1_1_internal-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_initialisation_guard_1_1_internal.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_initialisation_guard_1_1_internal.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient_list-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient_list.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient_list.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_circular_buffer-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_circular_buffer.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_circular_buffer.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_circular_buffer_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_circular_buffer_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_circular_buffer_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_core_convolver_uniform-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_core_convolver_uniform.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_core_convolver_uniform.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_crossfading_convolver_uniform-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_crossfading_convolver_uniform.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_crossfading_convolver_uniform.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_f_i_r-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_f_i_r.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_f_i_r.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fft_wrapper_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fft_wrapper_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fft_wrapper_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fft_wrapper_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fft_wrapper_factory-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fft_wrapper_factory.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ffts_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ffts_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ffts_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ffts_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ffts_wrapper_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ffts_wrapper_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ffts_wrapper_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_filter_routing_list-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_filter_routing_list.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_filter_routing_list.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_filter_routing_list.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_filter_routing_list_1_1_compare_entries-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_filter_routing_list_1_1_compare_entries.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_filter_routing_list_1_1_compare_entries.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_float_sequence-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_float_sequence.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_float_sequence.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fractional_delay_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fractional_delay_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fractional_delay_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fractional_delay_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fractional_delay_factory-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fractional_delay_factory.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_gain_fader-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_gain_fader.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_gain_fader.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_gain_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_gain_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_gain_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_index_sequence-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_index_sequence.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_index_sequence.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolating_convolver_uniform-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolating_convolver_uniform.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolating_convolver_uniform.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter_set-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter_set.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter_set.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter_set.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ipp_fft_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ipp_fft_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ipp_fft_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ipp_fft_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ipp_fft_wrapper_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ipp_fft_wrapper_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ipp_fft_wrapper_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_kiss_fft_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_kiss_fft_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_kiss_fft_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_kiss_fft_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_kiss_fft_wrapper_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_kiss_fft_wrapper_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_kiss_fft_wrapper_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_lagrange_interpolator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_lagrange_interpolator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_lagrange_interpolator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_lagrange_interpolator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_multichannel_convolver_uniform-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_multichannel_convolver_uniform.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_multichannel_convolver_uniform.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_multichannel_delay_line-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_multichannel_delay_line.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_multichannel_delay_line.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_object_channel_allocator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_object_channel_allocator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_object_channel_allocator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_base_1_1_type_id_translator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_base_1_1_type_id_translator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_base_1_1_type_id_translator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_list-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_list.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_list.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_sparse_gain_routing_list-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_sparse_gain_routing_list.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_sparse_gain_routing_list.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_sparse_gain_routing_list.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_sparse_gain_routing_list_1_1_compare_entries-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_sparse_gain_routing_list_1_1_compare_entries.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_sparse_gain_routing_list_1_1_compare_entries.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_add-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_add.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_add.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_add.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_biquad_iir_filter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_biquad_iir_filter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_biquad_iir_filter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_biquad_iir_filter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_c_a_p_gain_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_c_a_p_gain_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_c_a_p_gain_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_c_a_p_gain_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_channel_object_routing_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_channel_object_routing_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_channel_object_routing_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_channel_object_routing_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_crossfading_fir_filter_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_crossfading_fir_filter_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_crossfading_fir_filter_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_crossfading_fir_filter_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_vector-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_vector.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_vector.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_vector.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_diffusion_gain_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_diffusion_gain_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_diffusion_gain_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_diffusion_gain_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_fir_filter_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_fir_filter_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_fir_filter_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_fir_filter_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_vector-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_vector.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_vector.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_vector.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_hoa_all_rad_gain_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_hoa_all_rad_gain_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_hoa_all_rad_gain_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_hoa_all_rad_gain_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_interpolating_fir_filter_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_interpolating_fir_filter_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_interpolating_fir_filter_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_interpolating_fir_filter_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_listener_compensation-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_listener_compensation.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_listener_compensation.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_listener_compensation.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_null_source-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_null_source.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_null_source.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_null_source.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_object_gain_eq_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_object_gain_eq_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_object_gain_eq_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_object_gain_eq_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_panning_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_panning_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_panning_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_panning_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_position_decoder-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_position_decoder.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_position_decoder.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_position_decoder.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scalar_osc_decoder-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scalar_osc_decoder.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scalar_osc_decoder.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scalar_osc_decoder.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_decoder-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_decoder.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_decoder.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_decoder.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_encoder-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_encoder.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_encoder.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_encoder.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_signal_routing-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_signal_routing.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_signal_routing.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_signal_routing.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_sparse_gain_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_sparse_gain_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_sparse_gain_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_sparse_gain_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_inverse_transform-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_inverse_transform.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_inverse_transform.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_inverse_transform.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_transform-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_transform.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_transform.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_transform.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_receiver-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_receiver.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_receiver.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_receiver.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_sender-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_sender.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_sender.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_sender.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_filter_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_filter_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_filter_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_filter_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_object_renderer-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_object_renderer.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_object_renderer.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_object_renderer.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_parameter_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_parameter_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_parameter_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_parameter_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_channel-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_channel.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_channel.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_connection_map-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_connection_map.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_connection_map.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_signal_flow-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_signal_flow.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_signal_flow.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_signal_flow.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_signal_pool-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_signal_pool.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_signal_pool.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_communication_area-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_communication_area.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_communication_area.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_graph-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_graph.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_graph.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_graph_1_1_connected_ports-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_graph_1_1_connected_ports.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_graph_1_1_connected_ports.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_map.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_map.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_port_lookup-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_port_lookup.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_port_lookup.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_scheduling_graph-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_scheduling_graph.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_scheduling_graph.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_signal_routing_internal-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_signal_routing_internal.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_signal_routing_internal.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_signal_routing_internal.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_baseline_renderer-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_baseline_renderer.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_baseline_renderer.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_baseline_renderer.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_bunch_renderer-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_bunch_renderer.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_bunch_renderer.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_bunch_renderer.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_core_renderer-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_core_renderer.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_core_renderer.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_core_renderer.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_delay_vector-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_delay_vector.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_delay_vector.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_delay_vector.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_gain_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_gain_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_gain_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_gain_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_time_frequency_feedthrough-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_time_frequency_feedthrough.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_time_frequency_feedthrough.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_time_frequency_feedthrough.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflowspython_1_1_visr_renderer-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflowspython_1_1_visr_renderer.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflowspython_1_1_visr_renderer.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflowspython_1_1_visr_renderer.png"
  Delete "$INSTDIR\doc\html\closed.png"
  Delete "$INSTDIR\doc\html\communication__area_8cpp.html"
  Delete "$INSTDIR\doc\html\communication__area_8hpp.html"
  Delete "$INSTDIR\doc\html\communication__area_8hpp_source.html"
  Delete "$INSTDIR\doc\html\communication__protocol_8cpp.html"
  Delete "$INSTDIR\doc\html\communication__protocol_8cpp.js"
  Delete "$INSTDIR\doc\html\communication__protocol__base_8cpp.html"
  Delete "$INSTDIR\doc\html\communication__protocol__base_8hpp.html"
  Delete "$INSTDIR\doc\html\communication__protocol__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\communication__protocol__factory_8hpp.html"
  Delete "$INSTDIR\doc\html\communication__protocol__factory_8hpp_source.html"
  Delete "$INSTDIR\doc\html\communication__protocol__type_8hpp.html"
  Delete "$INSTDIR\doc\html\communication__protocol__type_8hpp.js"
  Delete "$INSTDIR\doc\html\communication__protocol__type_8hpp_source.html"
  Delete "$INSTDIR\doc\html\compile__time__hash__fnv1_8hpp.html"
  Delete "$INSTDIR\doc\html\compile__time__hash__fnv1_8hpp.js"
  Delete "$INSTDIR\doc\html\compile__time__hash__fnv1_8hpp_source.html"
  Delete "$INSTDIR\doc\html\complex__multiply_8cpp.html"
  Delete "$INSTDIR\doc\html\complex__multiply_8cpp.js"
  Delete "$INSTDIR\doc\html\component_8hpp.html"
  Delete "$INSTDIR\doc\html\component_8hpp_source.html"
  Delete "$INSTDIR\doc\html\component__implementation_8cpp.html"
  Delete "$INSTDIR\doc\html\component__implementation_8cpp.js"
  Delete "$INSTDIR\doc\html\component__implementation_8hpp.html"
  Delete "$INSTDIR\doc\html\component__implementation_8hpp_source.html"
  Delete "$INSTDIR\doc\html\component__names_8cpp.html"
  Delete "$INSTDIR\doc\html\component__names_8cpp.js"
  Delete "$INSTDIR\doc\html\compose__message__string_8hpp.html"
  Delete "$INSTDIR\doc\html\compose__message__string_8hpp_source.html"
  Delete "$INSTDIR\doc\html\composite__component_8hpp.html"
  Delete "$INSTDIR\doc\html\composite__component_8hpp_source.html"
  Delete "$INSTDIR\doc\html\composite__component__implementation_8cpp.html"
  Delete "$INSTDIR\doc\html\composite__component__implementation_8hpp.html"
  Delete "$INSTDIR\doc\html\composite__component__implementation_8hpp_source.html"
  Delete "$INSTDIR\doc\html\constants_8hpp.html"
  Delete "$INSTDIR\doc\html\constants_8hpp.js"
  Delete "$INSTDIR\doc\html\constants_8hpp_source.html"
  Delete "$INSTDIR\doc\html\core__convolver__uniform_8hpp.html"
  Delete "$INSTDIR\doc\html\core__convolver__uniform_8hpp_source.html"
  Delete "$INSTDIR\doc\html\crossfading__convolver__uniform_8hpp.html"
  Delete "$INSTDIR\doc\html\crossfading__convolver__uniform_8hpp_source.html"
  Delete "$INSTDIR\doc\html\crossfading__fir__filter__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\crossfading__fir__filter__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\crossfading__fir__filter__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\db__linear__conversion_8hpp.html"
  Delete "$INSTDIR\doc\html\db__linear__conversion_8hpp.js"
  Delete "$INSTDIR\doc\html\db__linear__conversion_8hpp_source.html"
  Delete "$INSTDIR\doc\html\decode___n8___p40__t-design__t8___p40_8txt_8cpp.html"
  Delete "$INSTDIR\doc\html\defs_8h.html"
  Delete "$INSTDIR\doc\html\defs_8h.js"
  Delete "$INSTDIR\doc\html\defs_8h_source.html"
  Delete "$INSTDIR\doc\html\degree__radian__conversion_8hpp.html"
  Delete "$INSTDIR\doc\html\degree__radian__conversion_8hpp.js"
  Delete "$INSTDIR\doc\html\degree__radian__conversion_8hpp_source.html"
  Delete "$INSTDIR\doc\html\delay__line_8cpp.html"
  Delete "$INSTDIR\doc\html\delay__line_8cpp.js"
  Delete "$INSTDIR\doc\html\denormalised__number__handling_8cpp.html"
  Delete "$INSTDIR\doc\html\denormalised__number__handling_8hpp.html"
  Delete "$INSTDIR\doc\html\denormalised__number__handling_8hpp_source.html"
  Delete "$INSTDIR\doc\html\deprecated.html"
  Delete "$INSTDIR\doc\html\diffuse__source_8hpp.html"
  Delete "$INSTDIR\doc\html\diffuse__source_8hpp_source.html"
  Delete "$INSTDIR\doc\html\diffuse__source__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\diffuse__source__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\diffuse__source__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\diffusion__gain__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\diffusion__gain__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\dir_014c6c332b259aea005167b130f4b4a9.html"
  Delete "$INSTDIR\doc\html\dir_014c6c332b259aea005167b130f4b4a9.js"
  Delete "$INSTDIR\doc\html\dir_02ecb9b96232f0e6802f6b197f581ed9.html"
  Delete "$INSTDIR\doc\html\dir_02ecb9b96232f0e6802f6b197f581ed9.js"
  Delete "$INSTDIR\doc\html\dir_0645e67c857f3ed5fe452946d333ec84.html"
  Delete "$INSTDIR\doc\html\dir_0645e67c857f3ed5fe452946d333ec84.js"
  Delete "$INSTDIR\doc\html\dir_0b263b942c0be7e5862088d0b3304562.html"
  Delete "$INSTDIR\doc\html\dir_0b263b942c0be7e5862088d0b3304562.js"
  Delete "$INSTDIR\doc\html\dir_0d54c8da732fd7ef3ebc9bd8970f5f73.html"
  Delete "$INSTDIR\doc\html\dir_0d54c8da732fd7ef3ebc9bd8970f5f73.js"
  Delete "$INSTDIR\doc\html\dir_0f00094951dfe5f0dac235ea92ea9e02.html"
  Delete "$INSTDIR\doc\html\dir_0f00094951dfe5f0dac235ea92ea9e02.js"
  Delete "$INSTDIR\doc\html\dir_0f4e3eebf44c879e7b7bbd92038c452e.html"
  Delete "$INSTDIR\doc\html\dir_0f4e3eebf44c879e7b7bbd92038c452e.js"
  Delete "$INSTDIR\doc\html\dir_1040b460d8b674aa121d87fe413d7826.html"
  Delete "$INSTDIR\doc\html\dir_1040b460d8b674aa121d87fe413d7826.js"
  Delete "$INSTDIR\doc\html\dir_11c82a1c6d49869166f7ddedda96e670.html"
  Delete "$INSTDIR\doc\html\dir_11c82a1c6d49869166f7ddedda96e670.js"
  Delete "$INSTDIR\doc\html\dir_13e2d675882fbd9c5da8ed54f0a9f3ec.html"
  Delete "$INSTDIR\doc\html\dir_13e2d675882fbd9c5da8ed54f0a9f3ec.js"
  Delete "$INSTDIR\doc\html\dir_16df2321f6a74705e2f8557f30970449.html"
  Delete "$INSTDIR\doc\html\dir_16df2321f6a74705e2f8557f30970449.js"
  Delete "$INSTDIR\doc\html\dir_189e8538bc549a9aeb0e4f2c948817e9.html"
  Delete "$INSTDIR\doc\html\dir_189e8538bc549a9aeb0e4f2c948817e9.js"
  Delete "$INSTDIR\doc\html\dir_1d1c3bb45d911e5755cc628c0f9a04d1.html"
  Delete "$INSTDIR\doc\html\dir_1d1c3bb45d911e5755cc628c0f9a04d1.js"
  Delete "$INSTDIR\doc\html\dir_203e0c315d1beb6785357a49fa3f979d.html"
  Delete "$INSTDIR\doc\html\dir_203e0c315d1beb6785357a49fa3f979d.js"
  Delete "$INSTDIR\doc\html\dir_260fe735f3cb79f89ea46b4e9fca3d19.html"
  Delete "$INSTDIR\doc\html\dir_260fe735f3cb79f89ea46b4e9fca3d19.js"
  Delete "$INSTDIR\doc\html\dir_2b4d203186da8b3994156779da527cab.html"
  Delete "$INSTDIR\doc\html\dir_2b4d203186da8b3994156779da527cab.js"
  Delete "$INSTDIR\doc\html\dir_2c465f66ec8d3aa6317d374a4bd24cdb.html"
  Delete "$INSTDIR\doc\html\dir_2c465f66ec8d3aa6317d374a4bd24cdb.js"
  Delete "$INSTDIR\doc\html\dir_2e00633f6c8c861df92e83ef05b46ad6.html"
  Delete "$INSTDIR\doc\html\dir_2e00633f6c8c861df92e83ef05b46ad6.js"
  Delete "$INSTDIR\doc\html\dir_39135db674f1851f170545dbae5cf6fe.html"
  Delete "$INSTDIR\doc\html\dir_39135db674f1851f170545dbae5cf6fe.js"
  Delete "$INSTDIR\doc\html\dir_395a501f50135c7b2f9ffa49479fda50.html"
  Delete "$INSTDIR\doc\html\dir_395a501f50135c7b2f9ffa49479fda50.js"
  Delete "$INSTDIR\doc\html\dir_3be19a81f182e7332ca41dfde2c67825.html"
  Delete "$INSTDIR\doc\html\dir_3be19a81f182e7332ca41dfde2c67825.js"
  Delete "$INSTDIR\doc\html\dir_3d62a41c1001155905fae299516ac865.html"
  Delete "$INSTDIR\doc\html\dir_3d62a41c1001155905fae299516ac865.js"
  Delete "$INSTDIR\doc\html\dir_41fe5a68be84f836d7091e1273488cbe.html"
  Delete "$INSTDIR\doc\html\dir_41fe5a68be84f836d7091e1273488cbe.js"
  Delete "$INSTDIR\doc\html\dir_4495670b9be8bc659cd773f191b4e213.html"
  Delete "$INSTDIR\doc\html\dir_4495670b9be8bc659cd773f191b4e213.js"
  Delete "$INSTDIR\doc\html\dir_47fd8888ef5b1409ed82e01ef4c0bb3d.html"
  Delete "$INSTDIR\doc\html\dir_47fd8888ef5b1409ed82e01ef4c0bb3d.js"
  Delete "$INSTDIR\doc\html\dir_493a5415a20e2e5f2c6e45c090b9b342.html"
  Delete "$INSTDIR\doc\html\dir_493a5415a20e2e5f2c6e45c090b9b342.js"
  Delete "$INSTDIR\doc\html\dir_49415fe834e80fa493f89c8e73a06250.html"
  Delete "$INSTDIR\doc\html\dir_49415fe834e80fa493f89c8e73a06250.js"
  Delete "$INSTDIR\doc\html\dir_4bede4f65f26296b0614ac618d002cd0.html"
  Delete "$INSTDIR\doc\html\dir_4bede4f65f26296b0614ac618d002cd0.js"
  Delete "$INSTDIR\doc\html\dir_4f859de48b9bbd587a4999ae58b80ad7.html"
  Delete "$INSTDIR\doc\html\dir_4f859de48b9bbd587a4999ae58b80ad7.js"
  Delete "$INSTDIR\doc\html\dir_4fce6a84a1f36302d1b3a007a424b188.html"
  Delete "$INSTDIR\doc\html\dir_4fce6a84a1f36302d1b3a007a424b188.js"
  Delete "$INSTDIR\doc\html\dir_5103cbbdad57e3a7b8feeefa49fd7b0e.html"
  Delete "$INSTDIR\doc\html\dir_5103cbbdad57e3a7b8feeefa49fd7b0e.js"
  Delete "$INSTDIR\doc\html\dir_53adf0b982dc8545998aae3f283a5a58.html"
  Delete "$INSTDIR\doc\html\dir_53adf0b982dc8545998aae3f283a5a58.js"
  Delete "$INSTDIR\doc\html\dir_556b2188208462a0df1d460b6e1fb55d.html"
  Delete "$INSTDIR\doc\html\dir_556b2188208462a0df1d460b6e1fb55d.js"
  Delete "$INSTDIR\doc\html\dir_5c0d64f70903e893b1efe571a4b8de29.html"
  Delete "$INSTDIR\doc\html\dir_5c0d64f70903e893b1efe571a4b8de29.js"
  Delete "$INSTDIR\doc\html\dir_68267d1309a1af8e8297ef4c3efbcdba.html"
  Delete "$INSTDIR\doc\html\dir_68267d1309a1af8e8297ef4c3efbcdba.js"
  Delete "$INSTDIR\doc\html\dir_68fc8eca6a76365009b89f36d4deca27.html"
  Delete "$INSTDIR\doc\html\dir_68fc8eca6a76365009b89f36d4deca27.js"
  Delete "$INSTDIR\doc\html\dir_6ab670b730245e0968deb2276ab39377.html"
  Delete "$INSTDIR\doc\html\dir_6ab670b730245e0968deb2276ab39377.js"
  Delete "$INSTDIR\doc\html\dir_6c57063d49b019795782b93903c47c0f.html"
  Delete "$INSTDIR\doc\html\dir_6c57063d49b019795782b93903c47c0f.js"
  Delete "$INSTDIR\doc\html\dir_729a4516a537cf6e64e41697e523b001.html"
  Delete "$INSTDIR\doc\html\dir_729a4516a537cf6e64e41697e523b001.js"
  Delete "$INSTDIR\doc\html\dir_740094eb819e8a2c8c4709fffd68c5ee.html"
  Delete "$INSTDIR\doc\html\dir_740094eb819e8a2c8c4709fffd68c5ee.js"
  Delete "$INSTDIR\doc\html\dir_77e0c9fe4173c72b875ae9562407de2c.html"
  Delete "$INSTDIR\doc\html\dir_77e0c9fe4173c72b875ae9562407de2c.js"
  Delete "$INSTDIR\doc\html\dir_7c408682270efa343e1328cf24c8bddb.html"
  Delete "$INSTDIR\doc\html\dir_7c408682270efa343e1328cf24c8bddb.js"
  Delete "$INSTDIR\doc\html\dir_86a697919b01f146390e616c449e50ab.html"
  Delete "$INSTDIR\doc\html\dir_86a697919b01f146390e616c449e50ab.js"
  Delete "$INSTDIR\doc\html\dir_92c2de66b023d858b4a99ee08753a706.html"
  Delete "$INSTDIR\doc\html\dir_92c2de66b023d858b4a99ee08753a706.js"
  Delete "$INSTDIR\doc\html\dir_9894b53404392327e63a436630cc83b5.html"
  Delete "$INSTDIR\doc\html\dir_9894b53404392327e63a436630cc83b5.js"
  Delete "$INSTDIR\doc\html\dir_9ee58dd100ef9e6e22b6913e7160f612.html"
  Delete "$INSTDIR\doc\html\dir_9ee58dd100ef9e6e22b6913e7160f612.js"
  Delete "$INSTDIR\doc\html\dir_a004b8f2a10531e25389c8daac04d9d7.html"
  Delete "$INSTDIR\doc\html\dir_a004b8f2a10531e25389c8daac04d9d7.js"
  Delete "$INSTDIR\doc\html\dir_a693e0399f729a4c656e47adac53de21.html"
  Delete "$INSTDIR\doc\html\dir_a693e0399f729a4c656e47adac53de21.js"
  Delete "$INSTDIR\doc\html\dir_a8056c6c1f7c0ece9dad68b7dd289f70.html"
  Delete "$INSTDIR\doc\html\dir_a8056c6c1f7c0ece9dad68b7dd289f70.js"
  Delete "$INSTDIR\doc\html\dir_a8e04a0aec16c24c77026d19e386080d.html"
  Delete "$INSTDIR\doc\html\dir_a8e04a0aec16c24c77026d19e386080d.js"
  Delete "$INSTDIR\doc\html\dir_aaafa20671a50e39df67208137461614.html"
  Delete "$INSTDIR\doc\html\dir_aaafa20671a50e39df67208137461614.js"
  Delete "$INSTDIR\doc\html\dir_ab446da4b10677d231516a215c2593b0.html"
  Delete "$INSTDIR\doc\html\dir_ab446da4b10677d231516a215c2593b0.js"
  Delete "$INSTDIR\doc\html\dir_ac01287d47bed959ac3eb860f37c424a.html"
  Delete "$INSTDIR\doc\html\dir_ac01287d47bed959ac3eb860f37c424a.js"
  Delete "$INSTDIR\doc\html\dir_ac7e44afd0a44393c47248e9b958c834.html"
  Delete "$INSTDIR\doc\html\dir_ac7e44afd0a44393c47248e9b958c834.js"
  Delete "$INSTDIR\doc\html\dir_ace829b89b39dc7471dba565463fab26.html"
  Delete "$INSTDIR\doc\html\dir_ace829b89b39dc7471dba565463fab26.js"
  Delete "$INSTDIR\doc\html\dir_ae9967c8309e42d02eaed7906b4d761a.html"
  Delete "$INSTDIR\doc\html\dir_ae9967c8309e42d02eaed7906b4d761a.js"
  Delete "$INSTDIR\doc\html\dir_c04d921e285ee1e7047dc07228956a5d.html"
  Delete "$INSTDIR\doc\html\dir_c04d921e285ee1e7047dc07228956a5d.js"
  Delete "$INSTDIR\doc\html\dir_c6a8403601c24848571ea2a73ccd640b.html"
  Delete "$INSTDIR\doc\html\dir_c6a8403601c24848571ea2a73ccd640b.js"
  Delete "$INSTDIR\doc\html\dir_d3f050b33e8ab44f7f8481b8e486c613.html"
  Delete "$INSTDIR\doc\html\dir_d3f050b33e8ab44f7f8481b8e486c613.js"
  Delete "$INSTDIR\doc\html\dir_dcb615ec72ec05db5d42ec52739b625b.html"
  Delete "$INSTDIR\doc\html\dir_dcb615ec72ec05db5d42ec52739b625b.js"
  Delete "$INSTDIR\doc\html\dir_de74e5d28a99eeb7db40a22eaf2b6e84.html"
  Delete "$INSTDIR\doc\html\dir_de74e5d28a99eeb7db40a22eaf2b6e84.js"
  Delete "$INSTDIR\doc\html\dir_dee10c86eae1af048e484ab6918c0951.html"
  Delete "$INSTDIR\doc\html\dir_dee10c86eae1af048e484ab6918c0951.js"
  Delete "$INSTDIR\doc\html\dir_e02dec0bef18f00458efcf941bdcd802.html"
  Delete "$INSTDIR\doc\html\dir_e02dec0bef18f00458efcf941bdcd802.js"
  Delete "$INSTDIR\doc\html\dir_e0f72a4a7b4df723ad02ea9cf6df5894.html"
  Delete "$INSTDIR\doc\html\dir_e0f72a4a7b4df723ad02ea9cf6df5894.js"
  Delete "$INSTDIR\doc\html\dir_e3092f42fdaf7ee5568d923f7bdfbfe5.html"
  Delete "$INSTDIR\doc\html\dir_e3092f42fdaf7ee5568d923f7bdfbfe5.js"
  Delete "$INSTDIR\doc\html\dir_f044611984161f2608b01c838efbf751.html"
  Delete "$INSTDIR\doc\html\dir_f044611984161f2608b01c838efbf751.js"
  Delete "$INSTDIR\doc\html\dir_f311637798a211c3495f8027b948e033.html"
  Delete "$INSTDIR\doc\html\dir_f311637798a211c3495f8027b948e033.js"
  Delete "$INSTDIR\doc\html\dir_f742e70b55103b98462bb00a3b94819f.html"
  Delete "$INSTDIR\doc\html\dir_f742e70b55103b98462bb00a3b94819f.js"
  Delete "$INSTDIR\doc\html\dir_fdc2158e5d491370a0e22ceeff6a2471.html"
  Delete "$INSTDIR\doc\html\dir_fdc2158e5d491370a0e22ceeff6a2471.js"
  Delete "$INSTDIR\doc\html\doc.png"
  Delete "$INSTDIR\doc\html\double__buffering__protocol_8hpp.html"
  Delete "$INSTDIR\doc\html\double__buffering__protocol_8hpp_source.html"
  Delete "$INSTDIR\doc\html\doxygen.css"
  Delete "$INSTDIR\doc\html\doxygen.png"
  Delete "$INSTDIR\doc\html\dynsections.js"
  Delete "$INSTDIR\doc\html\efl_8cpp.html"
  Delete "$INSTDIR\doc\html\efl_8cpp.js"
  Delete "$INSTDIR\doc\html\empty__parameter__config_8hpp.html"
  Delete "$INSTDIR\doc\html\empty__parameter__config_8hpp_source.html"
  Delete "$INSTDIR\doc\html\error__codes_8cpp.html"
  Delete "$INSTDIR\doc\html\error__codes_8cpp.js"
  Delete "$INSTDIR\doc\html\error__codes_8hpp.html"
  Delete "$INSTDIR\doc\html\error__codes_8hpp.js"
  Delete "$INSTDIR\doc\html\error__codes_8hpp_source.html"
  Delete "$INSTDIR\doc\html\export__symbol_8hpp.html"
  Delete "$INSTDIR\doc\html\export__symbol_8hpp.js"
  Delete "$INSTDIR\doc\html\export__symbol_8hpp_source.html"
  Delete "$INSTDIR\doc\html\external__base_8cpp.html"
  Delete "$INSTDIR\doc\html\external__base_8hpp.html"
  Delete "$INSTDIR\doc\html\external__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\external__wrapper_8hpp.html"
  Delete "$INSTDIR\doc\html\external__wrapper_8hpp_source.html"
  Delete "$INSTDIR\doc\html\feedthrough_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\feedthrough_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\feedthrough_8cpp.html"
  Delete "$INSTDIR\doc\html\feedthrough_8cpp.js"
  Delete "$INSTDIR\doc\html\fft__wrapper__base_8hpp.html"
  Delete "$INSTDIR\doc\html\fft__wrapper__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\fft__wrapper__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\fft__wrapper__factory_8cpp.js"
  Delete "$INSTDIR\doc\html\fft__wrapper__factory_8hpp.html"
  Delete "$INSTDIR\doc\html\fft__wrapper__factory_8hpp_source.html"
  Delete "$INSTDIR\doc\html\ffts__wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\ffts__wrapper_8hpp.html"
  Delete "$INSTDIR\doc\html\ffts__wrapper_8hpp_source.html"
  Delete "$INSTDIR\doc\html\files.html"
  Delete "$INSTDIR\doc\html\files_dup.js"
  Delete "$INSTDIR\doc\html\filter__routing_8hpp.html"
  Delete "$INSTDIR\doc\html\filter__routing_8hpp_source.html"
  Delete "$INSTDIR\doc\html\filter__routing__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\filter__routing__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\fir_8cpp.html"
  Delete "$INSTDIR\doc\html\fir_8hpp.html"
  Delete "$INSTDIR\doc\html\fir_8hpp.js"
  Delete "$INSTDIR\doc\html\fir_8hpp_source.html"
  Delete "$INSTDIR\doc\html\fir__filter__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\fir__filter__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\fir__filter__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\float__sequence_8cpp.html"
  Delete "$INSTDIR\doc\html\float__sequence_8hpp.html"
  Delete "$INSTDIR\doc\html\float__sequence_8hpp_source.html"
  Delete "$INSTDIR\doc\html\folderclosed.png"
  Delete "$INSTDIR\doc\html\folderopen.png"
  Delete "$INSTDIR\doc\html\fractional__delay__base_8cpp.html"
  Delete "$INSTDIR\doc\html\fractional__delay__base_8hpp.html"
  Delete "$INSTDIR\doc\html\fractional__delay__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\fractional__delay__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\fractional__delay__factory_8hpp.html"
  Delete "$INSTDIR\doc\html\fractional__delay__factory_8hpp_source.html"
  Delete "$INSTDIR\doc\html\fractional__delay__interpolator__base_8hpp.html"
  Delete "$INSTDIR\doc\html\fractional__delay__interpolator__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\functions.html"
  Delete "$INSTDIR\doc\html\functions_0x7e.html"
  Delete "$INSTDIR\doc\html\functions_b.html"
  Delete "$INSTDIR\doc\html\functions_c.html"
  Delete "$INSTDIR\doc\html\functions_d.html"
  Delete "$INSTDIR\doc\html\functions_dup.js"
  Delete "$INSTDIR\doc\html\functions_e.html"
  Delete "$INSTDIR\doc\html\functions_enum.html"
  Delete "$INSTDIR\doc\html\functions_eval.html"
  Delete "$INSTDIR\doc\html\functions_f.html"
  Delete "$INSTDIR\doc\html\functions_func.html"
  Delete "$INSTDIR\doc\html\functions_func.js"
  Delete "$INSTDIR\doc\html\functions_func_0x7e.html"
  Delete "$INSTDIR\doc\html\functions_func_b.html"
  Delete "$INSTDIR\doc\html\functions_func_c.html"
  Delete "$INSTDIR\doc\html\functions_func_d.html"
  Delete "$INSTDIR\doc\html\functions_func_e.html"
  Delete "$INSTDIR\doc\html\functions_func_f.html"
  Delete "$INSTDIR\doc\html\functions_func_g.html"
  Delete "$INSTDIR\doc\html\functions_func_h.html"
  Delete "$INSTDIR\doc\html\functions_func_i.html"
  Delete "$INSTDIR\doc\html\functions_func_j.html"
  Delete "$INSTDIR\doc\html\functions_func_k.html"
  Delete "$INSTDIR\doc\html\functions_func_l.html"
  Delete "$INSTDIR\doc\html\functions_func_m.html"
  Delete "$INSTDIR\doc\html\functions_func_n.html"
  Delete "$INSTDIR\doc\html\functions_func_o.html"
  Delete "$INSTDIR\doc\html\functions_func_p.html"
  Delete "$INSTDIR\doc\html\functions_func_q.html"
  Delete "$INSTDIR\doc\html\functions_func_r.html"
  Delete "$INSTDIR\doc\html\functions_func_s.html"
  Delete "$INSTDIR\doc\html\functions_func_t.html"
  Delete "$INSTDIR\doc\html\functions_func_u.html"
  Delete "$INSTDIR\doc\html\functions_func_v.html"
  Delete "$INSTDIR\doc\html\functions_func_w.html"
  Delete "$INSTDIR\doc\html\functions_func_x.html"
  Delete "$INSTDIR\doc\html\functions_func_y.html"
  Delete "$INSTDIR\doc\html\functions_func_z.html"
  Delete "$INSTDIR\doc\html\functions_g.html"
  Delete "$INSTDIR\doc\html\functions_h.html"
  Delete "$INSTDIR\doc\html\functions_i.html"
  Delete "$INSTDIR\doc\html\functions_j.html"
  Delete "$INSTDIR\doc\html\functions_k.html"
  Delete "$INSTDIR\doc\html\functions_l.html"
  Delete "$INSTDIR\doc\html\functions_m.html"
  Delete "$INSTDIR\doc\html\functions_n.html"
  Delete "$INSTDIR\doc\html\functions_o.html"
  Delete "$INSTDIR\doc\html\functions_p.html"
  Delete "$INSTDIR\doc\html\functions_q.html"
  Delete "$INSTDIR\doc\html\functions_r.html"
  Delete "$INSTDIR\doc\html\functions_rela.html"
  Delete "$INSTDIR\doc\html\functions_s.html"
  Delete "$INSTDIR\doc\html\functions_t.html"
  Delete "$INSTDIR\doc\html\functions_type.html"
  Delete "$INSTDIR\doc\html\functions_u.html"
  Delete "$INSTDIR\doc\html\functions_v.html"
  Delete "$INSTDIR\doc\html\functions_vars.html"
  Delete "$INSTDIR\doc\html\functions_w.html"
  Delete "$INSTDIR\doc\html\functions_x.html"
  Delete "$INSTDIR\doc\html\functions_y.html"
  Delete "$INSTDIR\doc\html\functions_z.html"
  Delete "$INSTDIR\doc\html\gain__fader_8cpp.html"
  Delete "$INSTDIR\doc\html\gain__fader_8hpp.html"
  Delete "$INSTDIR\doc\html\gain__fader_8hpp_source.html"
  Delete "$INSTDIR\doc\html\gain__matrix_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\gain__matrix_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\gain__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\gain__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\gil__ensure__guard_8cpp.html"
  Delete "$INSTDIR\doc\html\gil__ensure__guard_8hpp.html"
  Delete "$INSTDIR\doc\html\gil__ensure__guard_8hpp_source.html"
  Delete "$INSTDIR\doc\html\globals.html"
  Delete "$INSTDIR\doc\html\globals_defs.html"
  Delete "$INSTDIR\doc\html\globals_func.html"
  Delete "$INSTDIR\doc\html\globals_type.html"
  Delete "$INSTDIR\doc\html\hierarchy.html"
  Delete "$INSTDIR\doc\html\hierarchy.js"
  Delete "$INSTDIR\doc\html\hoa__allrad__gain__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\hoa__allrad__gain__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\hoa__source_8hpp.html"
  Delete "$INSTDIR\doc\html\hoa__source_8hpp_source.html"
  Delete "$INSTDIR\doc\html\hoa__source__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\hoa__source__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\hoa__source__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\impl_2audio__connection__descriptor_8cpp.html"
  Delete "$INSTDIR\doc\html\impl_2audio__connection__descriptor_8cpp.js"
  Delete "$INSTDIR\doc\html\index.html"
  Delete "$INSTDIR\doc\html\index__sequence_8cpp.html"
  Delete "$INSTDIR\doc\html\index__sequence_8hpp.html"
  Delete "$INSTDIR\doc\html\index__sequence_8hpp_source.html"
  Delete "$INSTDIR\doc\html\indexed__value__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\indexed__value__parameter_8hpp.js"
  Delete "$INSTDIR\doc\html\indexed__value__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\initialisation__guard_8cpp.html"
  Delete "$INSTDIR\doc\html\initialisation__guard_8hpp.html"
  Delete "$INSTDIR\doc\html\initialisation__guard_8hpp_source.html"
  Delete "$INSTDIR\doc\html\initialise__parameter__library_8cpp.html"
  Delete "$INSTDIR\doc\html\initialise__parameter__library_8cpp.js"
  Delete "$INSTDIR\doc\html\initialise__parameter__library_8hpp.html"
  Delete "$INSTDIR\doc\html\initialise__parameter__library_8hpp.js"
  Delete "$INSTDIR\doc\html\initialise__parameter__library_8hpp_source.html"
  Delete "$INSTDIR\doc\html\installation_8dox.html"
  Delete "$INSTDIR\doc\html\installation_instructions.html"
  Delete "$INSTDIR\doc\html\instantiation_8cpp.html"
  Delete "$INSTDIR\doc\html\instantiation_8cpp.js"
  Delete "$INSTDIR\doc\html\integrity__checking_8hpp.html"
  Delete "$INSTDIR\doc\html\integrity__checking_8hpp.js"
  Delete "$INSTDIR\doc\html\integrity__checking_8hpp_source.html"
  Delete "$INSTDIR\doc\html\interpolating__convolver_8cpp.html"
  Delete "$INSTDIR\doc\html\interpolating__convolver_8cpp.js"
  Delete "$INSTDIR\doc\html\interpolating__convolver__uniform_8hpp.html"
  Delete "$INSTDIR\doc\html\interpolating__convolver__uniform_8hpp_source.html"
  Delete "$INSTDIR\doc\html\interpolating__fir__filter__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\interpolating__fir__filter__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\interpolating__fir__filter__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\ipp__fft__wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\ipp__fft__wrapper_8hpp.html"
  Delete "$INSTDIR\doc\html\ipp__fft__wrapper_8hpp_source.html"
  Delete "$INSTDIR\doc\html\jack__interface_8hpp.html"
  Delete "$INSTDIR\doc\html\jack__interface_8hpp_source.html"
  Delete "$INSTDIR\doc\html\jack_audio_interface_configuration.html"
  Delete "$INSTDIR\doc\html\jackconf_8dox.html"
  Delete "$INSTDIR\doc\html\jackexample.jpeg"
  Delete "$INSTDIR\doc\html\jacksimpleexample.jpeg"
  Delete "$INSTDIR\doc\html\jquery.js"
  Delete "$INSTDIR\doc\html\kiss__fft__wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\kiss__fft__wrapper_8cpp.js"
  Delete "$INSTDIR\doc\html\kiss__fft__wrapper_8hpp.html"
  Delete "$INSTDIR\doc\html\kiss__fft__wrapper_8hpp_source.html"
  Delete "$INSTDIR\doc\html\kiss__fft__wrapper__double_8cpp.html"
  Delete "$INSTDIR\doc\html\kiss__fft__wrapper__float_8cpp.html"
  Delete "$INSTDIR\doc\html\lagrange__coefficient__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\lagrange__coefficient__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\lagrange__coefficient__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\lagrange__interpolator_8hpp.html"
  Delete "$INSTDIR\doc\html\lagrange__interpolator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\late__reverb__filter__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\late__reverb__filter__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\late__reverb__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\late__reverb__parameter_8hpp.js"
  Delete "$INSTDIR\doc\html\late__reverb__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libapputilities_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libapputilities_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libapputilities_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\libapputilities_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\libapputilities_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libaudiointerfaces_2audio__interface_8cpp.html"
  Delete "$INSTDIR\doc\html\libaudiointerfaces_2audio__interface__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\libaudiointerfaces_2audio__interface__factory_8cpp.js"
  Delete "$INSTDIR\doc\html\libaudiointerfaces_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libaudiointerfaces_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libaudiointerfaces_2jack__interface_8cpp.html"
  Delete "$INSTDIR\doc\html\libaudiointerfaces_2portaudio__interface_8cpp.html"
  Delete "$INSTDIR\doc\html\libefl_2basic__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\libefl_2basic__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\libefl_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libefl_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libefl_2test_2lagrange__interpolator_8cpp.html"
  Delete "$INSTDIR\doc\html\libefl_2test_2lagrange__interpolator_8cpp.js"
  Delete "$INSTDIR\doc\html\libefl_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\libefl_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\libobjectmodel_2channel__object_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2diffuse__source_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2hoa__source_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2object_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2object__type_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2object__type_8cpp.js"
  Delete "$INSTDIR\doc\html\libobjectmodel_2object__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2object__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2object__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2plane__wave_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2point__source_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2point__source__extent_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2point__source__with__diffuseness_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2point__source__with__reverb_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2test_2point__source__with__reverb_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2test_2point__source__with__reverb_8cpp.js"
  Delete "$INSTDIR\doc\html\libobjectmodel_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\libobjectmodel_8dox.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_mainpage.html"
  Delete "$INSTDIR\doc\html\libpanning_2_x_y_z_8cpp.html"
  Delete "$INSTDIR\doc\html\libpanning_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libpanning_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libpanning_2test_2loudspeaker__array_8cpp.html"
  Delete "$INSTDIR\doc\html\libpanning_2test_2loudspeaker__array_8cpp.js"
  Delete "$INSTDIR\doc\html\libpanning_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\libpanning_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\libpanning_2test_2vbap_8cpp.html"
  Delete "$INSTDIR\doc\html\libpanning_2test_2vbap_8cpp.js"
  Delete "$INSTDIR\doc\html\libpanning_2vbap_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2double__buffering__protocol_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2empty__parameter__config_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libpml_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libpml_2filter__routing__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2indexed__value__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2interpolation__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2interpolation__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\libpml_2interpolation__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libpml_2listener__position_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2listener__position_8cpp.js"
  Delete "$INSTDIR\doc\html\libpml_2matrix__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2message__queue__protocol_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2object__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2object__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\libpml_2object__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libpml_2scalar__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2shared__data__protocol_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2signal__routing__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2sparse__gain__routing__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2string__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2test_2filter__routing__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2test_2filter__routing__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\libpml_2test_2matrix__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2test_2matrix__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\libpml_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\libpml_2test_2vector__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2test_2vector__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\libpml_2vector__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpythoncomponents_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libpythoncomponents_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libpythoncomponents_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\libpythoncomponents_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\libpythonsupport_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libpythonsupport_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librbbl_2biquad__coefficient_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2core__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2crossfading__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librbbl_2filter__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2gain__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2gain__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librbbl_2interpolating__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2interpolation__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2interpolation__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2interpolation__parameter_8hpp.js"
  Delete "$INSTDIR\doc\html\librbbl_2interpolation__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librbbl_2lagrange__interpolator_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2multichannel__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2object__channel__allocator_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2sparse__gain__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2test_2biquad__coefficient_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2test_2biquad__coefficient_8cpp.js"
  Delete "$INSTDIR\doc\html\librbbl_2test_2multichannel__convolver_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2test_2multichannel__convolver_8cpp.js"
  Delete "$INSTDIR\doc\html\librbbl_2test_2object__channel__allocator_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2test_2object__channel__allocator_8cpp.js"
  Delete "$INSTDIR\doc\html\librbbl_2test_2sparse__gain__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2test_2sparse__gain__routing_8cpp.js"
  Delete "$INSTDIR\doc\html\librbbl_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2add_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2biquad__iir__filter_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2cap__gain__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2channel__object__routing__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2crossfading__fir__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2crossfading__fir__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2delay__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2delay__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2delay__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\librcl_2delay__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\librcl_2delay__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librcl_2delay__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2delay__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2delay__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\librcl_2delay__vector_8hpp.js"
  Delete "$INSTDIR\doc\html\librcl_2delay__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librcl_2diffusion__gain__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\librcl_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librcl_2fir__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2fir__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2gain__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\librcl_2gain__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librcl_2gain__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2hoa__allrad__gain__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2interpolating__fir__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2interpolating__fir__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2listener__compensation_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2listener__compensation_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2null__source_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2object__gain__eq__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2panning__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2panning__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2position__decoder_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2scalar__osc__decoder_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2scene__decoder_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2scene__encoder_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2signal__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2sparse__gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2sparse__gain__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2test_2biquad__iir__filter_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2test_2biquad__iir__filter_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2test_2hoa__allrad__gain__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2test_2hoa__allrad__gain__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2test_2scene__decoder_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2test_2scene__decoder_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2test_2signal__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2test_2signal__routing_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2time__frequency__inverse__transform_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2time__frequency__transform_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2udp__receiver_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2udp__sender_8cpp.html"
  Delete "$INSTDIR\doc\html\libreverbobject_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libreverbobject_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libreverbobject_2late__reverb__filter__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\libreverbobject_2late__reverb__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libreverbobject_2reverb__object__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\libreverbobject_2reverb__parameter__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\libreverbobject_2reverb__parameter__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\librrl_2audio__signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\librrl_2audio__signal__flow_8cpp.js"
  Delete "$INSTDIR\doc\html\librrl_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\librrl_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librrl_2integrity__checking_8cpp.html"
  Delete "$INSTDIR\doc\html\librrl_2integrity__checking_8cpp.js"
  Delete "$INSTDIR\doc\html\librrl_2test_2parameter__connection_8cpp.html"
  Delete "$INSTDIR\doc\html\librrl_2test_2parameter__connection_8cpp.js"
  Delete "$INSTDIR\doc\html\librrl_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\librrl_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\libsignalflows_2baseline__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2baseline__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2baseline__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2core__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2core__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2core__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2delay__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2delay__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2delay__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2gain__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2gain__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2test_2baseline__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2test_2baseline__renderer_8cpp.js"
  Delete "$INSTDIR\doc\html\libsignalflows_2test_2delay__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2test_2delay__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\libsignalflows_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\libsignalflows_2time__frequency__feedthrough_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflowspython_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libsignalflowspython_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libsignalflowspython_2visr__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflowspython_2visr__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\libsignalflowspython_2visr__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libvisr_2atomic__component_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2audio__sample__type_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2audio__sample__type_8cpp.js"
  Delete "$INSTDIR\doc\html\libvisr_2channel__list_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2channel__list_8cpp.js"
  Delete "$INSTDIR\doc\html\libvisr_2communication__protocol__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2component_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2composite__component_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libvisr_2parameter__base_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2parameter__config__base_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2parameter__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2signal__flow__context_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2test_2audio__sample__type_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2test_2audio__sample__type_8cpp.js"
  Delete "$INSTDIR\doc\html\libvisr_2test_2parameter__connection_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2test_2parameter__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\listener__compensation_8hpp.html"
  Delete "$INSTDIR\doc\html\listener__compensation_8hpp_source.html"
  Delete "$INSTDIR\doc\html\listener__position_8hpp.html"
  Delete "$INSTDIR\doc\html\listener__position_8hpp.js"
  Delete "$INSTDIR\doc\html\listener__position_8hpp_source.html"
  Delete "$INSTDIR\doc\html\load__module_8cpp.html"
  Delete "$INSTDIR\doc\html\load__module_8cpp.js"
  Delete "$INSTDIR\doc\html\load__module_8hpp.html"
  Delete "$INSTDIR\doc\html\load__module_8hpp.js"
  Delete "$INSTDIR\doc\html\load__module_8hpp_source.html"
  Delete "$INSTDIR\doc\html\mainpage_8dox.html"
  Delete "$INSTDIR\doc\html\matrix__convolver_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\matrix__convolver_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\matrix__convolver_8cpp.html"
  Delete "$INSTDIR\doc\html\matrix__convolver_8cpp.js"
  Delete "$INSTDIR\doc\html\matrix__convolver_8hpp.html"
  Delete "$INSTDIR\doc\html\matrix__convolver_8hpp_source.html"
  Delete "$INSTDIR\doc\html\matrix__from__ndarray_8hpp.html"
  Delete "$INSTDIR\doc\html\matrix__from__ndarray_8hpp.js"
  Delete "$INSTDIR\doc\html\matrix__from__ndarray_8hpp_source.html"
  Delete "$INSTDIR\doc\html\matrix__functions_8cpp.html"
  Delete "$INSTDIR\doc\html\matrix__functions_8cpp.js"
  Delete "$INSTDIR\doc\html\matrix__functions_8hpp.html"
  Delete "$INSTDIR\doc\html\matrix__functions_8hpp.js"
  Delete "$INSTDIR\doc\html\matrix__functions_8hpp_source.html"
  Delete "$INSTDIR\doc\html\matrix__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\matrix__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\matrix__parameter__config_8cpp.html"
  Delete "$INSTDIR\doc\html\matrix__parameter__config_8hpp.html"
  Delete "$INSTDIR\doc\html\matrix__parameter__config_8hpp_source.html"
  Delete "$INSTDIR\doc\html\max__msp__externals_8dox.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2delay__vector_2delay__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2delay__vector_2delay__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\maxmspexternals_2delay__vector_2delay__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2delay__vector_2delay__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2gain__matrix_2gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2gain__matrix_2gain__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\maxmspexternals_2gain__matrix_2gain__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2gain__matrix_2gain__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2libmaxsupport_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2libmaxsupport_2export__symbols_8hpp.js"
  Delete "$INSTDIR\doc\html\maxmspexternals_2libmaxsupport_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2init__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2init__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2init__filter__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2init__filter__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2init__filter__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2visr__renderer_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2visr__renderer_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2visr__renderer_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2visr__renderer_2visr__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2visr__renderer_2visr__renderer_8cpp.js"
  Delete "$INSTDIR\doc\html\maxmspexternals_2visr__renderer_2visr__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2visr__renderer_2visr__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\menu.js"
  Delete "$INSTDIR\doc\html\menudata.js"
  Delete "$INSTDIR\doc\html\message__queue__protocol_8hpp.html"
  Delete "$INSTDIR\doc\html\message__queue__protocol_8hpp_source.html"
  Delete "$INSTDIR\doc\html\mex_2biquad__iir__filter_2biquad__iir__filter_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2biquad__iir__filter_2biquad__iir__filter_8cpp.js"
  Delete "$INSTDIR\doc\html\mex_2biquad__iir__filter_2signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2biquad__iir__filter_2signal__flow_8cpp.js"
  Delete "$INSTDIR\doc\html\mex_2biquad__iir__filter_2signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\mex_2biquad__iir__filter_2signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\mex_2delay__vector_2delay__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2delay__vector_2delay__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\mex_2feedthrough_2signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2feedthrough_2signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\mex_2feedthrough_2signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\mex_2gain__matrix_2gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2gain__matrix_2gain__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\mex_2gain__matrix_2signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2gain__matrix_2signal__flow_8cpp.js"
  Delete "$INSTDIR\doc\html\mex_2gain__matrix_2signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\mex_2gain__matrix_2signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\mex_2late__reverb__filter__calculator_2late__reverb__filter__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2late__reverb__filter__calculator_2late__reverb__filter__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\mex_2late__reverb__filter__calculator_2signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2late__reverb__filter__calculator_2signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\mex_2late__reverb__filter__calculator_2signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\mex_2multichannel__convolver_2multichannel__convolver_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2multichannel__convolver_2multichannel__convolver_8cpp.js"
  Delete "$INSTDIR\doc\html\mex_2time__frequency__feedthrough_2time__frequency__feedthrough_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2time__frequency__feedthrough_2time__frequency__feedthrough_8cpp.js"
  Delete "$INSTDIR\doc\html\mex__wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\mex__wrapper_8hpp.html"
  Delete "$INSTDIR\doc\html\mex__wrapper_8hpp_source.html"
  Delete "$INSTDIR\doc\html\multichannel__convolver__uniform_8hpp.html"
  Delete "$INSTDIR\doc\html\multichannel__convolver__uniform_8hpp_source.html"
  Delete "$INSTDIR\doc\html\multichannel__delay__line_8cpp.html"
  Delete "$INSTDIR\doc\html\multichannel__delay__line_8hpp.html"
  Delete "$INSTDIR\doc\html\multichannel__delay__line_8hpp_source.html"
  Delete "$INSTDIR\doc\html\namespacemembers.html"
  Delete "$INSTDIR\doc\html\namespacemembers_b.html"
  Delete "$INSTDIR\doc\html\namespacemembers_c.html"
  Delete "$INSTDIR\doc\html\namespacemembers_d.html"
  Delete "$INSTDIR\doc\html\namespacemembers_dup.js"
  Delete "$INSTDIR\doc\html\namespacemembers_e.html"
  Delete "$INSTDIR\doc\html\namespacemembers_enum.html"
  Delete "$INSTDIR\doc\html\namespacemembers_eval.html"
  Delete "$INSTDIR\doc\html\namespacemembers_f.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func.js"
  Delete "$INSTDIR\doc\html\namespacemembers_func_b.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_c.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_d.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_e.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_f.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_g.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_h.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_i.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_l.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_m.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_n.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_o.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_p.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_q.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_r.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_s.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_t.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_v.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_w.html"
  Delete "$INSTDIR\doc\html\namespacemembers_g.html"
  Delete "$INSTDIR\doc\html\namespacemembers_h.html"
  Delete "$INSTDIR\doc\html\namespacemembers_i.html"
  Delete "$INSTDIR\doc\html\namespacemembers_l.html"
  Delete "$INSTDIR\doc\html\namespacemembers_m.html"
  Delete "$INSTDIR\doc\html\namespacemembers_n.html"
  Delete "$INSTDIR\doc\html\namespacemembers_o.html"
  Delete "$INSTDIR\doc\html\namespacemembers_p.html"
  Delete "$INSTDIR\doc\html\namespacemembers_q.html"
  Delete "$INSTDIR\doc\html\namespacemembers_r.html"
  Delete "$INSTDIR\doc\html\namespacemembers_s.html"
  Delete "$INSTDIR\doc\html\namespacemembers_t.html"
  Delete "$INSTDIR\doc\html\namespacemembers_type.html"
  Delete "$INSTDIR\doc\html\namespacemembers_u.html"
  Delete "$INSTDIR\doc\html\namespacemembers_v.html"
  Delete "$INSTDIR\doc\html\namespacemembers_vars.html"
  Delete "$INSTDIR\doc\html\namespacemembers_w.html"
  Delete "$INSTDIR\doc\html\namespaceoscpkt.html"
  Delete "$INSTDIR\doc\html\namespacepybind11.html"
  Delete "$INSTDIR\doc\html\namespaces.html"
  Delete "$INSTDIR\doc\html\namespaces_8dox.html"
  Delete "$INSTDIR\doc\html\namespaces_dup.js"
  Delete "$INSTDIR\doc\html\namespacevisr.html"
  Delete "$INSTDIR\doc\html\namespacevisr.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1_audio_sample_type.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1_audio_sample_type.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1audio__network__streamer.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1audio__network__streamer.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1baseline__renderer.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1baseline__renderer.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1feedthrough.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1feedthrough.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1gain__matrix.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1gain__matrix.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1matrix__convolver.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1matrix__convolver.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1pythonflowrunner.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1pythonflowrunner.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1scene__decoder.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1scene__decoder.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apputilities.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apputilities.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1audiointerfaces.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1audiointerfaces.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1audiointerfaces_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1audiointerfaces_1_1test.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1detail.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1efl.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1efl.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1efl_1_1python.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1efl_1_1reference.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1efl_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1impl.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1impl.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1impl_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1impl_1_1test.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1maxmsp.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1maxmsp.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1maxmsp_1_1matrix__convolver.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1maxmsp_1_1matrix__convolver.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1maxmsp_1_1visr__renderer.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1maxmsp_1_1visr__renderer.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex_1_1feedthrough.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex_1_1feedthrough.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex_1_1gain__matrix.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex_1_1gain__matrix.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex_1_1late__reverb__filter__calculator.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex_1_1late__reverb__filter__calculator.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex_1_1multichannel__convolver.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mexsupport.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mexsupport.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1objectmodel.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1objectmodel.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1objectmodel_1_1python.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1objectmodel_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1panning.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1panning.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1panning_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pml.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pml.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pml_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1audiointerfaces.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1bindinghelpers.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1panning.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1pml.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1rcl.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1reverbobject.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1rrl.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1signalflows.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1visr.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1visr.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pythoncomponents.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pythoncomponents.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pythoncomponents_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pythonsupport.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pythonsupport.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rbbl.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rbbl.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rbbl_1_1_parametric_iir_coefficient_calculator.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rbbl_1_1python.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rbbl_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rcl.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rcl.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rcl_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1reverbobject.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1reverbobject.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1ril.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1ril.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1ril_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rrl.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rrl.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rrl_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1signalflows.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1signalflows.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1signalflows_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1signalflowspython.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1signalflowspython.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1version.html"
  Delete "$INSTDIR\doc\html\nav_f.png"
  Delete "$INSTDIR\doc\html\nav_g.png"
  Delete "$INSTDIR\doc\html\nav_h.png"
  Delete "$INSTDIR\doc\html\navtree.css"
  Delete "$INSTDIR\doc\html\navtree.js"
  Delete "$INSTDIR\doc\html\navtreedata.js"
  Delete "$INSTDIR\doc\html\navtreeindex0.js"
  Delete "$INSTDIR\doc\html\navtreeindex1.js"
  Delete "$INSTDIR\doc\html\navtreeindex10.js"
  Delete "$INSTDIR\doc\html\navtreeindex11.js"
  Delete "$INSTDIR\doc\html\navtreeindex12.js"
  Delete "$INSTDIR\doc\html\navtreeindex13.js"
  Delete "$INSTDIR\doc\html\navtreeindex14.js"
  Delete "$INSTDIR\doc\html\navtreeindex15.js"
  Delete "$INSTDIR\doc\html\navtreeindex16.js"
  Delete "$INSTDIR\doc\html\navtreeindex17.js"
  Delete "$INSTDIR\doc\html\navtreeindex18.js"
  Delete "$INSTDIR\doc\html\navtreeindex2.js"
  Delete "$INSTDIR\doc\html\navtreeindex3.js"
  Delete "$INSTDIR\doc\html\navtreeindex4.js"
  Delete "$INSTDIR\doc\html\navtreeindex5.js"
  Delete "$INSTDIR\doc\html\navtreeindex6.js"
  Delete "$INSTDIR\doc\html\navtreeindex7.js"
  Delete "$INSTDIR\doc\html\navtreeindex8.js"
  Delete "$INSTDIR\doc\html\navtreeindex9.js"
  Delete "$INSTDIR\doc\html\network__audio__streamer_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\network__audio__streamer_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\null__source_8hpp.html"
  Delete "$INSTDIR\doc\html\null__source_8hpp_source.html"
  Delete "$INSTDIR\doc\html\object_8hpp.html"
  Delete "$INSTDIR\doc\html\object_8hpp.js"
  Delete "$INSTDIR\doc\html\object_8hpp_source.html"
  Delete "$INSTDIR\doc\html\object__channel__allocator_8hpp.html"
  Delete "$INSTDIR\doc\html\object__channel__allocator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\object__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\object__factory_8cpp.js"
  Delete "$INSTDIR\doc\html\object__factory_8hpp.html"
  Delete "$INSTDIR\doc\html\object__factory_8hpp_source.html"
  Delete "$INSTDIR\doc\html\object__gain__eq__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\object__gain__eq__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\object__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\object__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\object__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\object__type_8hpp.html"
  Delete "$INSTDIR\doc\html\object__type_8hpp.js"
  Delete "$INSTDIR\doc\html\object__type_8hpp_source.html"
  Delete "$INSTDIR\doc\html\object__vector__assign_8cpp.html"
  Delete "$INSTDIR\doc\html\object__vector__assign_8cpp.js"
  Delete "$INSTDIR\doc\html\object__vector__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\object__vector__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\object__vector__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\objectmodel_8cpp.html"
  Delete "$INSTDIR\doc\html\objectmodel_8cpp.js"
  Delete "$INSTDIR\doc\html\open.png"
  Delete "$INSTDIR\doc\html\pages.html"
  Delete "$INSTDIR\doc\html\panning_8cpp.html"
  Delete "$INSTDIR\doc\html\panning_8cpp.js"
  Delete "$INSTDIR\doc\html\panning__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\panning__calculator_8hpp.js"
  Delete "$INSTDIR\doc\html\panning__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__base_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__config__base_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__config__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__connection__descriptor_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__connection__descriptor_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__connection__descriptor_8hpp.js"
  Delete "$INSTDIR\doc\html\parameter__connection__descriptor_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__connection__graph_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__connection__graph_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__connection__graph_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__connection__map_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__connection__map_8cpp.js"
  Delete "$INSTDIR\doc\html\parameter__connection__map_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__connection__map_8hpp.js"
  Delete "$INSTDIR\doc\html\parameter__connection__map_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__factory_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__factory_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__input_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__input_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__input_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__instantiation_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__instantiation_8cpp.js"
  Delete "$INSTDIR\doc\html\parameter__output_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__output_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__output_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__port_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__port_8cpp.js"
  Delete "$INSTDIR\doc\html\parameter__port__base_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__port__base_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__port__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__port__base__implementation_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__port__base__implementation_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__port__base__implementation_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__type_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__type_8hpp.js"
  Delete "$INSTDIR\doc\html\parameter__type_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient_8cpp.html"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient_8hpp.html"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient__calculator_8hpp.js"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\plane__wave_8hpp.html"
  Delete "$INSTDIR\doc\html\plane__wave_8hpp_source.html"
  Delete "$INSTDIR\doc\html\plane__wave__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\plane__wave__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\plane__wave__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\pml_8cpp.html"
  Delete "$INSTDIR\doc\html\pml_8cpp.js"
  Delete "$INSTDIR\doc\html\point__source_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source_8hpp_source.html"
  Delete "$INSTDIR\doc\html\point__source__extent_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source__extent_8hpp_source.html"
  Delete "$INSTDIR\doc\html\point__source__extent__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\point__source__extent__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source__extent__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\point__source__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\point__source__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\point__source__with__diffuseness_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source__with__diffuseness_8hpp_source.html"
  Delete "$INSTDIR\doc\html\point__source__with__diffuseness__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\point__source__with__diffuseness__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source__with__diffuseness__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\point__source__with__reverb_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source__with__reverb_8hpp_source.html"
  Delete "$INSTDIR\doc\html\point__source__with__reverb__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\point__source__with__reverb__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source__with__reverb__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__input_8cpp.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__input_8hpp.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__input_8hpp_source.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__output_8cpp.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__output_8hpp.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__output_8hpp_source.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__port_8cpp.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__port_8cpp.js"
  Delete "$INSTDIR\doc\html\port_8cpp.html"
  Delete "$INSTDIR\doc\html\port_8cpp.js"
  Delete "$INSTDIR\doc\html\port__base_8cpp.html"
  Delete "$INSTDIR\doc\html\port__base_8cpp.js"
  Delete "$INSTDIR\doc\html\port__base_8hpp.html"
  Delete "$INSTDIR\doc\html\port__base_8hpp.js"
  Delete "$INSTDIR\doc\html\port__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\port__base__implementation_8cpp.html"
  Delete "$INSTDIR\doc\html\port__base__implementation_8hpp.html"
  Delete "$INSTDIR\doc\html\port__base__implementation_8hpp_source.html"
  Delete "$INSTDIR\doc\html\port__utilities_8cpp.html"
  Delete "$INSTDIR\doc\html\port__utilities_8cpp.js"
  Delete "$INSTDIR\doc\html\port__utilities_8hpp.html"
  Delete "$INSTDIR\doc\html\port__utilities_8hpp.js"
  Delete "$INSTDIR\doc\html\port__utilities_8hpp_source.html"
  Delete "$INSTDIR\doc\html\portaudio__interface_8hpp.html"
  Delete "$INSTDIR\doc\html\portaudio__interface_8hpp_source.html"
  Delete "$INSTDIR\doc\html\portaudio_audio_interface_configuration.html"
  Delete "$INSTDIR\doc\html\portaudioconf_8dox.html"
  Delete "$INSTDIR\doc\html\position__decoder_8hpp.html"
  Delete "$INSTDIR\doc\html\position__decoder_8hpp_source.html"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2audio__interface_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2audio__interface_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2audio__interface__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2audio__interface__factory_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2jack__interface_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2jack__interface_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2portaudio__interface_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2portaudio__interface_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2efl_2basic__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2efl_2basic__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2efl_2basic__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2efl_2basic__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2channel__object_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2channel__object_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2diffuse__source_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2diffuse__source_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2hoa__source_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2hoa__source_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2object_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2object_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2object__type_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2object__type_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2object__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2object__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2plane__wave_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2plane__wave_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source__extent_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source__extent_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source__with__diffuseness_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source__with__diffuseness_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source__with__reverb_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source__with__reverb_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2panning_2_x_y_z_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2panning_2_x_y_z_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2panning_2loudspeaker__array_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2panning_2loudspeaker__array_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2panning_2vbap_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2panning_2vbap_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2double__buffering__protocol_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2double__buffering__protocol_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2empty__parameter__config_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2empty__parameter__config_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2filter__routing__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2filter__routing__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2indexed__value__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2indexed__value__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2interpolation__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2interpolation__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2listener__position_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2listener__position_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2matrix__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2matrix__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2message__queue__protocol_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2message__queue__protocol_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2object__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2object__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2scalar__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2scalar__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2shared__data__protocol_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2shared__data__protocol_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2signal__routing__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2signal__routing__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2sparse__gain__routing__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2sparse__gain__routing__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2string__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2string__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2vector__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2vector__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2biquad__coefficient_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2biquad__coefficient_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2core__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2core__convolver__uniform_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2crossfading__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2crossfading__convolver__uniform_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2filter__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2filter__routing_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2interpolating__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2interpolating__convolver__uniform_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2interpolation__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2interpolation__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2multichannel__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2multichannel__convolver__uniform_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2object__channel__allocator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2object__channel__allocator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2sparse__gain__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2sparse__gain__routing_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2add_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2add_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2biquad__iir__filter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2biquad__iir__filter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2cap__gain__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2cap__gain__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2channel__object__routing__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2channel__object__routing__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2crossfading__fir__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2crossfading__fir__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2delay__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2delay__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2delay__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2delay__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2diffusion__gain__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2diffusion__gain__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2fir__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2fir__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2gain__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2gain__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2gain__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2hoa__allrad__gain__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2hoa__allrad__gain__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2interpolating__fir__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2interpolating__fir__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2listener__compensation_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2listener__compensation_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2null__source_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2null__source_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2object__gain__eq__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2object__gain__eq__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2panning__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2panning__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2position__decoder_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2position__decoder_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2scalar__osc__decoder_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2scalar__osc__decoder_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2scene__decoder_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2scene__decoder_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2scene__encoder_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2scene__encoder_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2signal__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2signal__routing_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2sparse__gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2sparse__gain__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2time__frequency__inverse__transform_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2time__frequency__inverse__transform_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2time__frequency__transform_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2time__frequency__transform_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2udp__receiver_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2udp__receiver_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2udp__sender_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2udp__sender_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2late__reverb__filter__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2late__reverb__filter__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2late__reverb__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2late__reverb__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2reverb__object__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2reverb__object__renderer_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2reverb__parameter__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2reverb__parameter__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rrl_2audio__signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rrl_2audio__signal__flow_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rrl_2integrity__checking_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rrl_2integrity__checking_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2baseline__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2baseline__renderer_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2baseline__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2baseline__renderer_8hpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2baseline__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2core__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2core__renderer_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2core__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2core__renderer_8hpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2core__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__vector_8hpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2visr__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2visr__renderer_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2atomic__component_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2atomic__component_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2audio__sample__type_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2audio__sample__type_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2channel__list_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2channel__list_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2communication__protocol__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2communication__protocol__factory_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2component_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2component_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2composite__component_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2composite__component_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__base_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__base_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__config__base_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__config__base_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__connection_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__connection_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__factory_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2signal__flow__context_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2signal__flow__context_8cpp.js"
  Delete "$INSTDIR\doc\html\python__runner_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\python__runner_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\python__support_8dox.html"
  Delete "$INSTDIR\doc\html\python_support.html"
  Delete "$INSTDIR\doc\html\rbbl_8cpp.html"
  Delete "$INSTDIR\doc\html\rbbl_8cpp.js"
  Delete "$INSTDIR\doc\html\rcl_8cpp.html"
  Delete "$INSTDIR\doc\html\rcl_8cpp.js"
  Delete "$INSTDIR\doc\html\resize.js"
  Delete "$INSTDIR\doc\html\reverb__object__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\reverb__object__parser_8cpp.js"
  Delete "$INSTDIR\doc\html\reverb__object__parser_8h.html"
  Delete "$INSTDIR\doc\html\reverb__object__parser_8h_source.html"
  Delete "$INSTDIR\doc\html\reverb__object__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\reverb__object__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\reverb__parameter__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\reverb__parameter__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\reverbobject_8cpp.html"
  Delete "$INSTDIR\doc\html\reverbobject_8cpp.js"
  Delete "$INSTDIR\doc\html\rrl_8cpp.html"
  Delete "$INSTDIR\doc\html\rrl_8cpp.js"
  Delete "$INSTDIR\doc\html\s3a_logo.png"
  Delete "$INSTDIR\doc\html\scalar__osc__decoder_8hpp.html"
  Delete "$INSTDIR\doc\html\scalar__osc__decoder_8hpp_source.html"
  Delete "$INSTDIR\doc\html\scalar__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\scalar__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\scene__decoder_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\scene__decoder_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\scene__decoder_8hpp.html"
  Delete "$INSTDIR\doc\html\scene__decoder_8hpp_source.html"
  Delete "$INSTDIR\doc\html\scene__encoder_8hpp.html"
  Delete "$INSTDIR\doc\html\scene__encoder_8hpp_source.html"
  Delete "$INSTDIR\doc\html\scheduling__graph_8cpp.html"
  Delete "$INSTDIR\doc\html\scheduling__graph_8hpp.html"
  Delete "$INSTDIR\doc\html\scheduling__graph_8hpp_source.html"
  Delete "$INSTDIR\doc\html\search"
  Delete "$INSTDIR\doc\html\search\all_0.html"
  Delete "$INSTDIR\doc\html\search\all_0.js"
  Delete "$INSTDIR\doc\html\search\all_1.html"
  Delete "$INSTDIR\doc\html\search\all_1.js"
  Delete "$INSTDIR\doc\html\search\all_10.html"
  Delete "$INSTDIR\doc\html\search\all_10.js"
  Delete "$INSTDIR\doc\html\search\all_11.html"
  Delete "$INSTDIR\doc\html\search\all_11.js"
  Delete "$INSTDIR\doc\html\search\all_12.html"
  Delete "$INSTDIR\doc\html\search\all_12.js"
  Delete "$INSTDIR\doc\html\search\all_13.html"
  Delete "$INSTDIR\doc\html\search\all_13.js"
  Delete "$INSTDIR\doc\html\search\all_14.html"
  Delete "$INSTDIR\doc\html\search\all_14.js"
  Delete "$INSTDIR\doc\html\search\all_15.html"
  Delete "$INSTDIR\doc\html\search\all_15.js"
  Delete "$INSTDIR\doc\html\search\all_16.html"
  Delete "$INSTDIR\doc\html\search\all_16.js"
  Delete "$INSTDIR\doc\html\search\all_17.html"
  Delete "$INSTDIR\doc\html\search\all_17.js"
  Delete "$INSTDIR\doc\html\search\all_18.html"
  Delete "$INSTDIR\doc\html\search\all_18.js"
  Delete "$INSTDIR\doc\html\search\all_19.html"
  Delete "$INSTDIR\doc\html\search\all_19.js"
  Delete "$INSTDIR\doc\html\search\all_1a.html"
  Delete "$INSTDIR\doc\html\search\all_1a.js"
  Delete "$INSTDIR\doc\html\search\all_2.html"
  Delete "$INSTDIR\doc\html\search\all_2.js"
  Delete "$INSTDIR\doc\html\search\all_3.html"
  Delete "$INSTDIR\doc\html\search\all_3.js"
  Delete "$INSTDIR\doc\html\search\all_4.html"
  Delete "$INSTDIR\doc\html\search\all_4.js"
  Delete "$INSTDIR\doc\html\search\all_5.html"
  Delete "$INSTDIR\doc\html\search\all_5.js"
  Delete "$INSTDIR\doc\html\search\all_6.html"
  Delete "$INSTDIR\doc\html\search\all_6.js"
  Delete "$INSTDIR\doc\html\search\all_7.html"
  Delete "$INSTDIR\doc\html\search\all_7.js"
  Delete "$INSTDIR\doc\html\search\all_8.html"
  Delete "$INSTDIR\doc\html\search\all_8.js"
  Delete "$INSTDIR\doc\html\search\all_9.html"
  Delete "$INSTDIR\doc\html\search\all_9.js"
  Delete "$INSTDIR\doc\html\search\all_a.html"
  Delete "$INSTDIR\doc\html\search\all_a.js"
  Delete "$INSTDIR\doc\html\search\all_b.html"
  Delete "$INSTDIR\doc\html\search\all_b.js"
  Delete "$INSTDIR\doc\html\search\all_c.html"
  Delete "$INSTDIR\doc\html\search\all_c.js"
  Delete "$INSTDIR\doc\html\search\all_d.html"
  Delete "$INSTDIR\doc\html\search\all_d.js"
  Delete "$INSTDIR\doc\html\search\all_e.html"
  Delete "$INSTDIR\doc\html\search\all_e.js"
  Delete "$INSTDIR\doc\html\search\all_f.html"
  Delete "$INSTDIR\doc\html\search\all_f.js"
  Delete "$INSTDIR\doc\html\search\classes_0.html"
  Delete "$INSTDIR\doc\html\search\classes_0.js"
  Delete "$INSTDIR\doc\html\search\classes_1.html"
  Delete "$INSTDIR\doc\html\search\classes_1.js"
  Delete "$INSTDIR\doc\html\search\classes_10.html"
  Delete "$INSTDIR\doc\html\search\classes_10.js"
  Delete "$INSTDIR\doc\html\search\classes_11.html"
  Delete "$INSTDIR\doc\html\search\classes_11.js"
  Delete "$INSTDIR\doc\html\search\classes_12.html"
  Delete "$INSTDIR\doc\html\search\classes_12.js"
  Delete "$INSTDIR\doc\html\search\classes_13.html"
  Delete "$INSTDIR\doc\html\search\classes_13.js"
  Delete "$INSTDIR\doc\html\search\classes_14.html"
  Delete "$INSTDIR\doc\html\search\classes_14.js"
  Delete "$INSTDIR\doc\html\search\classes_15.html"
  Delete "$INSTDIR\doc\html\search\classes_15.js"
  Delete "$INSTDIR\doc\html\search\classes_16.html"
  Delete "$INSTDIR\doc\html\search\classes_16.js"
  Delete "$INSTDIR\doc\html\search\classes_2.html"
  Delete "$INSTDIR\doc\html\search\classes_2.js"
  Delete "$INSTDIR\doc\html\search\classes_3.html"
  Delete "$INSTDIR\doc\html\search\classes_3.js"
  Delete "$INSTDIR\doc\html\search\classes_4.html"
  Delete "$INSTDIR\doc\html\search\classes_4.js"
  Delete "$INSTDIR\doc\html\search\classes_5.html"
  Delete "$INSTDIR\doc\html\search\classes_5.js"
  Delete "$INSTDIR\doc\html\search\classes_6.html"
  Delete "$INSTDIR\doc\html\search\classes_6.js"
  Delete "$INSTDIR\doc\html\search\classes_7.html"
  Delete "$INSTDIR\doc\html\search\classes_7.js"
  Delete "$INSTDIR\doc\html\search\classes_8.html"
  Delete "$INSTDIR\doc\html\search\classes_8.js"
  Delete "$INSTDIR\doc\html\search\classes_9.html"
  Delete "$INSTDIR\doc\html\search\classes_9.js"
  Delete "$INSTDIR\doc\html\search\classes_a.html"
  Delete "$INSTDIR\doc\html\search\classes_a.js"
  Delete "$INSTDIR\doc\html\search\classes_b.html"
  Delete "$INSTDIR\doc\html\search\classes_b.js"
  Delete "$INSTDIR\doc\html\search\classes_c.html"
  Delete "$INSTDIR\doc\html\search\classes_c.js"
  Delete "$INSTDIR\doc\html\search\classes_d.html"
  Delete "$INSTDIR\doc\html\search\classes_d.js"
  Delete "$INSTDIR\doc\html\search\classes_e.html"
  Delete "$INSTDIR\doc\html\search\classes_e.js"
  Delete "$INSTDIR\doc\html\search\classes_f.html"
  Delete "$INSTDIR\doc\html\search\classes_f.js"
  Delete "$INSTDIR\doc\html\search\close.png"
  Delete "$INSTDIR\doc\html\search\defines_0.html"
  Delete "$INSTDIR\doc\html\search\defines_0.js"
  Delete "$INSTDIR\doc\html\search\defines_1.html"
  Delete "$INSTDIR\doc\html\search\defines_1.js"
  Delete "$INSTDIR\doc\html\search\defines_2.html"
  Delete "$INSTDIR\doc\html\search\defines_2.js"
  Delete "$INSTDIR\doc\html\search\defines_3.html"
  Delete "$INSTDIR\doc\html\search\defines_3.js"
  Delete "$INSTDIR\doc\html\search\defines_4.html"
  Delete "$INSTDIR\doc\html\search\defines_4.js"
  Delete "$INSTDIR\doc\html\search\defines_5.html"
  Delete "$INSTDIR\doc\html\search\defines_5.js"
  Delete "$INSTDIR\doc\html\search\defines_6.html"
  Delete "$INSTDIR\doc\html\search\defines_6.js"
  Delete "$INSTDIR\doc\html\search\defines_7.html"
  Delete "$INSTDIR\doc\html\search\defines_7.js"
  Delete "$INSTDIR\doc\html\search\defines_8.html"
  Delete "$INSTDIR\doc\html\search\defines_8.js"
  Delete "$INSTDIR\doc\html\search\defines_9.html"
  Delete "$INSTDIR\doc\html\search\defines_9.js"
  Delete "$INSTDIR\doc\html\search\enums_0.html"
  Delete "$INSTDIR\doc\html\search\enums_0.js"
  Delete "$INSTDIR\doc\html\search\enums_1.html"
  Delete "$INSTDIR\doc\html\search\enums_1.js"
  Delete "$INSTDIR\doc\html\search\enums_2.html"
  Delete "$INSTDIR\doc\html\search\enums_2.js"
  Delete "$INSTDIR\doc\html\search\enums_3.html"
  Delete "$INSTDIR\doc\html\search\enums_3.js"
  Delete "$INSTDIR\doc\html\search\enums_4.html"
  Delete "$INSTDIR\doc\html\search\enums_4.js"
  Delete "$INSTDIR\doc\html\search\enums_5.html"
  Delete "$INSTDIR\doc\html\search\enums_5.js"
  Delete "$INSTDIR\doc\html\search\enums_6.html"
  Delete "$INSTDIR\doc\html\search\enums_6.js"
  Delete "$INSTDIR\doc\html\search\enums_7.html"
  Delete "$INSTDIR\doc\html\search\enums_7.js"
  Delete "$INSTDIR\doc\html\search\enums_8.html"
  Delete "$INSTDIR\doc\html\search\enums_8.js"
  Delete "$INSTDIR\doc\html\search\enums_9.html"
  Delete "$INSTDIR\doc\html\search\enums_9.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_0.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_0.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_1.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_1.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_10.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_10.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_11.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_11.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_2.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_2.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_3.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_3.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_4.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_4.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_5.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_5.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_6.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_6.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_7.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_7.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_8.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_8.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_9.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_9.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_a.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_a.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_b.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_b.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_c.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_c.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_d.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_d.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_e.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_e.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_f.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_f.js"
  Delete "$INSTDIR\doc\html\search\files_0.html"
  Delete "$INSTDIR\doc\html\search\files_0.js"
  Delete "$INSTDIR\doc\html\search\files_1.html"
  Delete "$INSTDIR\doc\html\search\files_1.js"
  Delete "$INSTDIR\doc\html\search\files_10.html"
  Delete "$INSTDIR\doc\html\search\files_10.js"
  Delete "$INSTDIR\doc\html\search\files_11.html"
  Delete "$INSTDIR\doc\html\search\files_11.js"
  Delete "$INSTDIR\doc\html\search\files_12.html"
  Delete "$INSTDIR\doc\html\search\files_12.js"
  Delete "$INSTDIR\doc\html\search\files_13.html"
  Delete "$INSTDIR\doc\html\search\files_13.js"
  Delete "$INSTDIR\doc\html\search\files_14.html"
  Delete "$INSTDIR\doc\html\search\files_14.js"
  Delete "$INSTDIR\doc\html\search\files_15.html"
  Delete "$INSTDIR\doc\html\search\files_15.js"
  Delete "$INSTDIR\doc\html\search\files_16.html"
  Delete "$INSTDIR\doc\html\search\files_16.js"
  Delete "$INSTDIR\doc\html\search\files_2.html"
  Delete "$INSTDIR\doc\html\search\files_2.js"
  Delete "$INSTDIR\doc\html\search\files_3.html"
  Delete "$INSTDIR\doc\html\search\files_3.js"
  Delete "$INSTDIR\doc\html\search\files_4.html"
  Delete "$INSTDIR\doc\html\search\files_4.js"
  Delete "$INSTDIR\doc\html\search\files_5.html"
  Delete "$INSTDIR\doc\html\search\files_5.js"
  Delete "$INSTDIR\doc\html\search\files_6.html"
  Delete "$INSTDIR\doc\html\search\files_6.js"
  Delete "$INSTDIR\doc\html\search\files_7.html"
  Delete "$INSTDIR\doc\html\search\files_7.js"
  Delete "$INSTDIR\doc\html\search\files_8.html"
  Delete "$INSTDIR\doc\html\search\files_8.js"
  Delete "$INSTDIR\doc\html\search\files_9.html"
  Delete "$INSTDIR\doc\html\search\files_9.js"
  Delete "$INSTDIR\doc\html\search\files_a.html"
  Delete "$INSTDIR\doc\html\search\files_a.js"
  Delete "$INSTDIR\doc\html\search\files_b.html"
  Delete "$INSTDIR\doc\html\search\files_b.js"
  Delete "$INSTDIR\doc\html\search\files_c.html"
  Delete "$INSTDIR\doc\html\search\files_c.js"
  Delete "$INSTDIR\doc\html\search\files_d.html"
  Delete "$INSTDIR\doc\html\search\files_d.js"
  Delete "$INSTDIR\doc\html\search\files_e.html"
  Delete "$INSTDIR\doc\html\search\files_e.js"
  Delete "$INSTDIR\doc\html\search\files_f.html"
  Delete "$INSTDIR\doc\html\search\files_f.js"
  Delete "$INSTDIR\doc\html\search\functions_0.html"
  Delete "$INSTDIR\doc\html\search\functions_0.js"
  Delete "$INSTDIR\doc\html\search\functions_1.html"
  Delete "$INSTDIR\doc\html\search\functions_1.js"
  Delete "$INSTDIR\doc\html\search\functions_10.html"
  Delete "$INSTDIR\doc\html\search\functions_10.js"
  Delete "$INSTDIR\doc\html\search\functions_11.html"
  Delete "$INSTDIR\doc\html\search\functions_11.js"
  Delete "$INSTDIR\doc\html\search\functions_12.html"
  Delete "$INSTDIR\doc\html\search\functions_12.js"
  Delete "$INSTDIR\doc\html\search\functions_13.html"
  Delete "$INSTDIR\doc\html\search\functions_13.js"
  Delete "$INSTDIR\doc\html\search\functions_14.html"
  Delete "$INSTDIR\doc\html\search\functions_14.js"
  Delete "$INSTDIR\doc\html\search\functions_15.html"
  Delete "$INSTDIR\doc\html\search\functions_15.js"
  Delete "$INSTDIR\doc\html\search\functions_16.html"
  Delete "$INSTDIR\doc\html\search\functions_16.js"
  Delete "$INSTDIR\doc\html\search\functions_17.html"
  Delete "$INSTDIR\doc\html\search\functions_17.js"
  Delete "$INSTDIR\doc\html\search\functions_18.html"
  Delete "$INSTDIR\doc\html\search\functions_18.js"
  Delete "$INSTDIR\doc\html\search\functions_19.html"
  Delete "$INSTDIR\doc\html\search\functions_19.js"
  Delete "$INSTDIR\doc\html\search\functions_1a.html"
  Delete "$INSTDIR\doc\html\search\functions_1a.js"
  Delete "$INSTDIR\doc\html\search\functions_2.html"
  Delete "$INSTDIR\doc\html\search\functions_2.js"
  Delete "$INSTDIR\doc\html\search\functions_3.html"
  Delete "$INSTDIR\doc\html\search\functions_3.js"
  Delete "$INSTDIR\doc\html\search\functions_4.html"
  Delete "$INSTDIR\doc\html\search\functions_4.js"
  Delete "$INSTDIR\doc\html\search\functions_5.html"
  Delete "$INSTDIR\doc\html\search\functions_5.js"
  Delete "$INSTDIR\doc\html\search\functions_6.html"
  Delete "$INSTDIR\doc\html\search\functions_6.js"
  Delete "$INSTDIR\doc\html\search\functions_7.html"
  Delete "$INSTDIR\doc\html\search\functions_7.js"
  Delete "$INSTDIR\doc\html\search\functions_8.html"
  Delete "$INSTDIR\doc\html\search\functions_8.js"
  Delete "$INSTDIR\doc\html\search\functions_9.html"
  Delete "$INSTDIR\doc\html\search\functions_9.js"
  Delete "$INSTDIR\doc\html\search\functions_a.html"
  Delete "$INSTDIR\doc\html\search\functions_a.js"
  Delete "$INSTDIR\doc\html\search\functions_b.html"
  Delete "$INSTDIR\doc\html\search\functions_b.js"
  Delete "$INSTDIR\doc\html\search\functions_c.html"
  Delete "$INSTDIR\doc\html\search\functions_c.js"
  Delete "$INSTDIR\doc\html\search\functions_d.html"
  Delete "$INSTDIR\doc\html\search\functions_d.js"
  Delete "$INSTDIR\doc\html\search\functions_e.html"
  Delete "$INSTDIR\doc\html\search\functions_e.js"
  Delete "$INSTDIR\doc\html\search\functions_f.html"
  Delete "$INSTDIR\doc\html\search\functions_f.js"
  Delete "$INSTDIR\doc\html\search\mag_sel.png"
  Delete "$INSTDIR\doc\html\search\namespaces_0.html"
  Delete "$INSTDIR\doc\html\search\namespaces_0.js"
  Delete "$INSTDIR\doc\html\search\namespaces_1.html"
  Delete "$INSTDIR\doc\html\search\namespaces_1.js"
  Delete "$INSTDIR\doc\html\search\namespaces_2.html"
  Delete "$INSTDIR\doc\html\search\namespaces_2.js"
  Delete "$INSTDIR\doc\html\search\nomatches.html"
  Delete "$INSTDIR\doc\html\search\pages_0.html"
  Delete "$INSTDIR\doc\html\search\pages_0.js"
  Delete "$INSTDIR\doc\html\search\pages_1.html"
  Delete "$INSTDIR\doc\html\search\pages_1.js"
  Delete "$INSTDIR\doc\html\search\pages_2.html"
  Delete "$INSTDIR\doc\html\search\pages_2.js"
  Delete "$INSTDIR\doc\html\search\pages_3.html"
  Delete "$INSTDIR\doc\html\search\pages_3.js"
  Delete "$INSTDIR\doc\html\search\pages_4.html"
  Delete "$INSTDIR\doc\html\search\pages_4.js"
  Delete "$INSTDIR\doc\html\search\pages_5.html"
  Delete "$INSTDIR\doc\html\search\pages_5.js"
  Delete "$INSTDIR\doc\html\search\pages_6.html"
  Delete "$INSTDIR\doc\html\search\pages_6.js"
  Delete "$INSTDIR\doc\html\search\pages_7.html"
  Delete "$INSTDIR\doc\html\search\pages_7.js"
  Delete "$INSTDIR\doc\html\search\related_0.html"
  Delete "$INSTDIR\doc\html\search\related_0.js"
  Delete "$INSTDIR\doc\html\search\related_1.html"
  Delete "$INSTDIR\doc\html\search\related_1.js"
  Delete "$INSTDIR\doc\html\search\related_2.html"
  Delete "$INSTDIR\doc\html\search\related_2.js"
  Delete "$INSTDIR\doc\html\search\related_3.html"
  Delete "$INSTDIR\doc\html\search\related_3.js"
  Delete "$INSTDIR\doc\html\search\related_4.html"
  Delete "$INSTDIR\doc\html\search\related_4.js"
  Delete "$INSTDIR\doc\html\search\related_5.html"
  Delete "$INSTDIR\doc\html\search\related_5.js"
  Delete "$INSTDIR\doc\html\search\related_6.html"
  Delete "$INSTDIR\doc\html\search\related_6.js"
  Delete "$INSTDIR\doc\html\search\related_7.html"
  Delete "$INSTDIR\doc\html\search\related_7.js"
  Delete "$INSTDIR\doc\html\search\search.css"
  Delete "$INSTDIR\doc\html\search\search.js"
  Delete "$INSTDIR\doc\html\search\search_l.png"
  Delete "$INSTDIR\doc\html\search\search_m.png"
  Delete "$INSTDIR\doc\html\search\search_r.png"
  Delete "$INSTDIR\doc\html\search\searchdata.js"
  Delete "$INSTDIR\doc\html\search\typedefs_0.html"
  Delete "$INSTDIR\doc\html\search\typedefs_0.js"
  Delete "$INSTDIR\doc\html\search\typedefs_1.html"
  Delete "$INSTDIR\doc\html\search\typedefs_1.js"
  Delete "$INSTDIR\doc\html\search\typedefs_10.html"
  Delete "$INSTDIR\doc\html\search\typedefs_10.js"
  Delete "$INSTDIR\doc\html\search\typedefs_2.html"
  Delete "$INSTDIR\doc\html\search\typedefs_2.js"
  Delete "$INSTDIR\doc\html\search\typedefs_3.html"
  Delete "$INSTDIR\doc\html\search\typedefs_3.js"
  Delete "$INSTDIR\doc\html\search\typedefs_4.html"
  Delete "$INSTDIR\doc\html\search\typedefs_4.js"
  Delete "$INSTDIR\doc\html\search\typedefs_5.html"
  Delete "$INSTDIR\doc\html\search\typedefs_5.js"
  Delete "$INSTDIR\doc\html\search\typedefs_6.html"
  Delete "$INSTDIR\doc\html\search\typedefs_6.js"
  Delete "$INSTDIR\doc\html\search\typedefs_7.html"
  Delete "$INSTDIR\doc\html\search\typedefs_7.js"
  Delete "$INSTDIR\doc\html\search\typedefs_8.html"
  Delete "$INSTDIR\doc\html\search\typedefs_8.js"
  Delete "$INSTDIR\doc\html\search\typedefs_9.html"
  Delete "$INSTDIR\doc\html\search\typedefs_9.js"
  Delete "$INSTDIR\doc\html\search\typedefs_a.html"
  Delete "$INSTDIR\doc\html\search\typedefs_a.js"
  Delete "$INSTDIR\doc\html\search\typedefs_b.html"
  Delete "$INSTDIR\doc\html\search\typedefs_b.js"
  Delete "$INSTDIR\doc\html\search\typedefs_c.html"
  Delete "$INSTDIR\doc\html\search\typedefs_c.js"
  Delete "$INSTDIR\doc\html\search\typedefs_d.html"
  Delete "$INSTDIR\doc\html\search\typedefs_d.js"
  Delete "$INSTDIR\doc\html\search\typedefs_e.html"
  Delete "$INSTDIR\doc\html\search\typedefs_e.js"
  Delete "$INSTDIR\doc\html\search\typedefs_f.html"
  Delete "$INSTDIR\doc\html\search\typedefs_f.js"
  Delete "$INSTDIR\doc\html\search\variables_0.html"
  Delete "$INSTDIR\doc\html\search\variables_0.js"
  Delete "$INSTDIR\doc\html\search\variables_1.html"
  Delete "$INSTDIR\doc\html\search\variables_1.js"
  Delete "$INSTDIR\doc\html\search\variables_2.html"
  Delete "$INSTDIR\doc\html\search\variables_2.js"
  Delete "$INSTDIR\doc\html\search\variables_3.html"
  Delete "$INSTDIR\doc\html\search\variables_3.js"
  Delete "$INSTDIR\doc\html\search\variables_4.html"
  Delete "$INSTDIR\doc\html\search\variables_4.js"
  Delete "$INSTDIR\doc\html\search\variables_5.html"
  Delete "$INSTDIR\doc\html\search\variables_5.js"
  Delete "$INSTDIR\doc\html\search\variables_6.html"
  Delete "$INSTDIR\doc\html\search\variables_6.js"
  Delete "$INSTDIR\doc\html\search\variables_7.html"
  Delete "$INSTDIR\doc\html\search\variables_7.js"
  Delete "$INSTDIR\doc\html\search\variables_8.html"
  Delete "$INSTDIR\doc\html\search\variables_8.js"
  Delete "$INSTDIR\doc\html\search\variables_9.html"
  Delete "$INSTDIR\doc\html\search\variables_9.js"
  Delete "$INSTDIR\doc\html\search\variables_a.html"
  Delete "$INSTDIR\doc\html\search\variables_a.js"
  Delete "$INSTDIR\doc\html\search\variables_b.html"
  Delete "$INSTDIR\doc\html\search\variables_b.js"
  Delete "$INSTDIR\doc\html\search\variables_c.html"
  Delete "$INSTDIR\doc\html\search\variables_c.js"
  Delete "$INSTDIR\doc\html\search\variables_d.html"
  Delete "$INSTDIR\doc\html\search\variables_d.js"
  Delete "$INSTDIR\doc\html\search\variables_e.html"
  Delete "$INSTDIR\doc\html\search\variables_e.js"
  Delete "$INSTDIR\doc\html\shared__data__protocol_8hpp.html"
  Delete "$INSTDIR\doc\html\shared__data__protocol_8hpp_source.html"
  Delete "$INSTDIR\doc\html\signal__flow__context_8hpp.html"
  Delete "$INSTDIR\doc\html\signal__flow__context_8hpp_source.html"
  Delete "$INSTDIR\doc\html\signal__flow__wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\signal__flow__wrapper_8hpp.html"
  Delete "$INSTDIR\doc\html\signal__flow__wrapper_8hpp_source.html"
  Delete "$INSTDIR\doc\html\signal__routing_8hpp.html"
  Delete "$INSTDIR\doc\html\signal__routing_8hpp_source.html"
  Delete "$INSTDIR\doc\html\signal__routing__internal_8cpp.html"
  Delete "$INSTDIR\doc\html\signal__routing__internal_8cpp.js"
  Delete "$INSTDIR\doc\html\signal__routing__internal_8hpp.html"
  Delete "$INSTDIR\doc\html\signal__routing__internal_8hpp.js"
  Delete "$INSTDIR\doc\html\signal__routing__internal_8hpp_source.html"
  Delete "$INSTDIR\doc\html\signal__routing__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\signal__routing__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\signalflows_8cpp.html"
  Delete "$INSTDIR\doc\html\signalflows_8cpp.js"
  Delete "$INSTDIR\doc\html\sparse__gain__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\sparse__gain__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\sparse__gain__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\sparse__gain__routing_8hpp.html"
  Delete "$INSTDIR\doc\html\sparse__gain__routing_8hpp_source.html"
  Delete "$INSTDIR\doc\html\sparse__gain__routing__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\sparse__gain__routing__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\splitbar.png"
  Delete "$INSTDIR\doc\html\status__message_8hpp.html"
  Delete "$INSTDIR\doc\html\status__message_8hpp_source.html"
  Delete "$INSTDIR\doc\html\string__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\string__parameter_8hpp.js"
  Delete "$INSTDIR\doc\html\string__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1_audio_sample_type_1_1_id_to_type.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1_audio_sample_type_1_1_type_to_id.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1_communication_protocol_to_id.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1_id_to_communication_protocol.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1_id_to_parameter.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1_parameter_to_config_type.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1_parameter_to_id.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_instantiate_audio_interface_factory-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_instantiate_audio_interface_factory.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_instantiate_audio_interface_factory.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_jack_interface_1_1_config-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_jack_interface_1_1_config.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_jack_interface_1_1_config.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_config-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_config.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_config.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1efl_1_1_denormalised_numbers-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1efl_1_1_denormalised_numbers.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1efl_1_1_denormalised_numbers.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1efl_1_1_denormalised_numbers_1_1_state-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1efl_1_1_denormalised_numbers_1_1_state.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1efl_1_1_denormalised_numbers_1_1_state.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_audio_connection-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_audio_connection.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_audio_connection.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_compare_ports-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_compare_ports.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_compare_ports.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_parameter_connection-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_parameter_connection.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_parameter_connection.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1objectmodel_1_1_instantiate_object_factory-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1objectmodel_1_1_instantiate_object_factory.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1objectmodel_1_1_instantiate_object_factory.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_array_configuration_1_1_compare_speakers-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_array_configuration_1_1_compare_speakers.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_array_configuration_1_1_compare_speakers.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_array_configuration_1_1_speaker-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_array_configuration_1_1_speaker.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_array_configuration_1_1_speaker.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_filter_routing_parameter-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_filter_routing_parameter.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_filter_routing_parameter.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_filter_routing_parameter.png"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_signal_routing_parameter_1_1_entry-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_signal_routing_parameter_1_1_entry.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_signal_routing_parameter_1_1_entry.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_filter_routing-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_filter_routing.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_filter_routing.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_filter_routing.png"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_instantiate_fft_wrapper_factory-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_instantiate_fft_wrapper_factory.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_instantiate_fft_wrapper_factory.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_sparse_gain_routing-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_sparse_gain_routing.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_sparse_gain_routing.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_sparse_gain_routing.png"
  Delete "$INSTDIR\doc\html\sync_off.png"
  Delete "$INSTDIR\doc\html\sync_on.png"
  Delete "$INSTDIR\doc\html\t-design__t8___p40_8xml_8cpp.html"
  Delete "$INSTDIR\doc\html\tab_a.png"
  Delete "$INSTDIR\doc\html\tab_b.png"
  Delete "$INSTDIR\doc\html\tab_h.png"
  Delete "$INSTDIR\doc\html\tab_s.png"
  Delete "$INSTDIR\doc\html\tabs.css"
  Delete "$INSTDIR\doc\html\test_2_all_r_a_d_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2_all_r_a_d_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2_c_a_p_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2_c_a_p_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2audio__connection__descriptor_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2audio__connection__descriptor_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2channel__object__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2channel__object__parser_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2circular__buffer_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2circular__buffer_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2ffts__wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2ffts__wrapper_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2float__sequence_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2float__sequence_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2index__sequence_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2index__sequence_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2ipp__fft__wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2ipp__fft__wrapper_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2parametric__iir__coefficient_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2parametric__iir__coefficient_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2wrapper_8cpp.js"
  Delete "$INSTDIR\doc\html\test___f_i_r_8cpp.html"
  Delete "$INSTDIR\doc\html\test___f_i_r_8cpp.js"
  Delete "$INSTDIR\doc\html\test__kinect__receiver_8cpp.html"
  Delete "$INSTDIR\doc\html\test__listener__compensation_8cpp.html"
  Delete "$INSTDIR\doc\html\test__listener__compensation_8cpp.js"
  Delete "$INSTDIR\doc\html\time__frequency__feedthrough_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__feedthrough_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\time__frequency__feedthrough_8hpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__feedthrough_8hpp_source.html"
  Delete "$INSTDIR\doc\html\time__frequency__inverse__transform_8hpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__inverse__transform_8hpp_source.html"
  Delete "$INSTDIR\doc\html\time__frequency__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\time__frequency__parameter__config_8cpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__parameter__config_8hpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__parameter__config_8hpp_source.html"
  Delete "$INSTDIR\doc\html\time__frequency__transform_8hpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__transform_8hpp_source.html"
  Delete "$INSTDIR\doc\html\todo.html"
  Delete "$INSTDIR\doc\html\typed__parameter__base_8hpp.html"
  Delete "$INSTDIR\doc\html\typed__parameter__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\udp__receiver_8hpp.html"
  Delete "$INSTDIR\doc\html\udp__receiver_8hpp_source.html"
  Delete "$INSTDIR\doc\html\udp__sender_8hpp.html"
  Delete "$INSTDIR\doc\html\udp__sender_8hpp_source.html"
  Delete "$INSTDIR\doc\html\utility__functions_8cpp.html"
  Delete "$INSTDIR\doc\html\utility__functions_8cpp.js"
  Delete "$INSTDIR\doc\html\utility__functions_8hpp.html"
  Delete "$INSTDIR\doc\html\utility__functions_8hpp.js"
  Delete "$INSTDIR\doc\html\utility__functions_8hpp_source.html"
  Delete "$INSTDIR\doc\html\vector__conversions_8cpp.html"
  Delete "$INSTDIR\doc\html\vector__conversions_8cpp.js"
  Delete "$INSTDIR\doc\html\vector__conversions_8hpp.html"
  Delete "$INSTDIR\doc\html\vector__conversions_8hpp.js"
  Delete "$INSTDIR\doc\html\vector__conversions_8hpp_source.html"
  Delete "$INSTDIR\doc\html\vector__from__ndarray_8hpp.html"
  Delete "$INSTDIR\doc\html\vector__from__ndarray_8hpp.js"
  Delete "$INSTDIR\doc\html\vector__from__ndarray_8hpp_source.html"
  Delete "$INSTDIR\doc\html\vector__functions_8cpp.html"
  Delete "$INSTDIR\doc\html\vector__functions_8cpp.js"
  Delete "$INSTDIR\doc\html\vector__functions_8hpp.html"
  Delete "$INSTDIR\doc\html\vector__functions_8hpp.js"
  Delete "$INSTDIR\doc\html\vector__functions_8hpp_source.html"
  Delete "$INSTDIR\doc\html\vector__functions__ipp_8cpp.html"
  Delete "$INSTDIR\doc\html\vector__functions__ipp_8cpp.js"
  Delete "$INSTDIR\doc\html\vector__functions__reference_8cpp.html"
  Delete "$INSTDIR\doc\html\vector__functions__reference_8cpp.js"
  Delete "$INSTDIR\doc\html\vector__functions__reference_8hpp.html"
  Delete "$INSTDIR\doc\html\vector__functions__reference_8hpp.js"
  Delete "$INSTDIR\doc\html\vector__functions__reference_8hpp_source.html"
  Delete "$INSTDIR\doc\html\vector__functions__reference__impl_8hpp.html"
  Delete "$INSTDIR\doc\html\vector__functions__reference__impl_8hpp.js"
  Delete "$INSTDIR\doc\html\vector__functions__reference__impl_8hpp_source.html"
  Delete "$INSTDIR\doc\html\vector__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\vector__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\vector__parameter__config_8cpp.html"
  Delete "$INSTDIR\doc\html\vector__parameter__config_8hpp.html"
  Delete "$INSTDIR\doc\html\vector__parameter__config_8hpp_source.html"
  Delete "$INSTDIR\doc\html\version_8cpp.html"
  Delete "$INSTDIR\doc\html\version_8cpp.js"
  Delete "$INSTDIR\doc\html\version_8hpp.html"
  Delete "$INSTDIR\doc\html\version_8hpp.js"
  Delete "$INSTDIR\doc\html\version_8hpp_source.html"
  Delete "$INSTDIR\doc\html\visr_8cpp.html"
  Delete "$INSTDIR\doc\html\visr_8cpp.js"
  Delete "$INSTDIR\doc\html\visr__renderer_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\visr__renderer_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\visr__renderer__usage_8dox.html"
  Delete "$INSTDIR\doc\html\visr_renderer_usage.html"
  Delete "$INSTDIR\doc\html\wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\wrapper_8hpp.html"
  Delete "$INSTDIR\doc\html\wrapper_8hpp_source.html"
  RMDir "$INSTDIR\doc\html\search"
  RMDir "$INSTDIR\doc\html"
  RMDir "$INSTDIR\doc"
  noremove_doxygen_documentation_html:
!macroend
!macro Select_doxygen_documentation_html_depends
!macroend
!macro Deselect_required_by_doxygen_documentation_html
!macroend
!macro Remove_${user_documentation_pdf}
  IntCmp $user_documentation_pdf_was_installed 0 noremove_user_documentation_pdf
  Delete "$INSTDIR\doc"
  Delete "$INSTDIR\doc\userdoc-0.11.1.pdf"
  RMDir "$INSTDIR\doc"
  noremove_user_documentation_pdf:
!macroend
!macro Select_user_documentation_pdf_depends
!macroend
!macro Deselect_required_by_user_documentation_pdf
!macroend
!macro Remove_${python_package_bst}
  IntCmp $python_package_bst_was_installed 0 noremove_python_package_bst
  Delete "$INSTDIR\python"
  Delete "$INSTDIR\python\visr_bst"
  Delete "$INSTDIR\python\visr_bst\Changelog"
  Delete "$INSTDIR\python\visr_bst\LICENSE.md"
  Delete "$INSTDIR\python\visr_bst\README.md"
  Delete "$INSTDIR\python\visr_bst\__init__.py"
  Delete "$INSTDIR\python\visr_bst\controllers"
  Delete "$INSTDIR\python\visr_bst\controllers\dynamic_hrir_controller.py"
  Delete "$INSTDIR\python\visr_bst\controllers\virtual_loudspeaker_controller.py"
  Delete "$INSTDIR\python\visr_bst\data"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_all_speakers.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemA.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemB.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemC.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemD.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemE.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemF.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemG.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemH.xml"
  Delete "$INSTDIR\python\visr_bst\data\sofa"
  Delete "$INSTDIR\python\visr_bst\data\sofa\brir"
  Delete "$INSTDIR\python\visr_bst\data\sofa\brir\.gitignore"
  Delete "$INSTDIR\python\visr_bst\data\sofa\hoa2binaural"
  Delete "$INSTDIR\python\visr_bst\data\sofa\hoa2binaural\.gitignore"
  Delete "$INSTDIR\python\visr_bst\data\sofa\hrir"
  Delete "$INSTDIR\python\visr_bst\data\sofa\hrir\.gitignore"
  Delete "$INSTDIR\python\visr_bst\examples"
  Delete "$INSTDIR\python\visr_bst\examples\.gitignore"
  Delete "$INSTDIR\python\visr_bst\examples\run_realtime_dynamic_hrir_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\run_realtime_hoa_binaural_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\run_realtime_hoa_object_to_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\run_realtime_virtual_loudspeaker_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\simulate_dynamic_hrir_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\simulate_hoa_binaural_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\simulate_hoa_object_to_binaural_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\simulate_object_to_virtual_loudspeaker_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\simulate_virtual_loudspeaker_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\simulate_virtual_loudspeaker_renderer_late_static.py"
  Delete "$INSTDIR\python\visr_bst\examples\test_dynamic_hrir_controller.py"
  Delete "$INSTDIR\python\visr_bst\examples\test_virtual_loudspeaker_controller.py"
  Delete "$INSTDIR\python\visr_bst\hoa_components"
  Delete "$INSTDIR\python\visr_bst\hoa_components\__init__.py"
  Delete "$INSTDIR\python\visr_bst\hoa_components\hoa_coefficient_rotation.py"
  Delete "$INSTDIR\python\visr_bst\hoa_components\hoa_object_encoder.py"
  Delete "$INSTDIR\python\visr_bst\hoa_components\hoa_rotation_matrix_calculator.py"
  Delete "$INSTDIR\python\visr_bst\realtime_renderers"
  Delete "$INSTDIR\python\visr_bst\realtime_renderers\realtime_dynamic_hrir_renderer.py"
  Delete "$INSTDIR\python\visr_bst\realtime_renderers\realtime_hoa_binaural_renderer.py"
  Delete "$INSTDIR\python\visr_bst\realtime_renderers\realtime_hoa_object_to_binaural_renderer.py"
  Delete "$INSTDIR\python\visr_bst\realtime_renderers\realtime_virtual_loudspeaker_renderer.py"
  Delete "$INSTDIR\python\visr_bst\renderers"
  Delete "$INSTDIR\python\visr_bst\renderers\dynamic_hrir_renderer.py"
  Delete "$INSTDIR\python\visr_bst\renderers\hoa_binaural_renderer.py"
  Delete "$INSTDIR\python\visr_bst\renderers\hoa_object_to_binaural_renderer.py"
  Delete "$INSTDIR\python\visr_bst\renderers\object_to_virtual_loudspeaker_renderer.py"
  Delete "$INSTDIR\python\visr_bst\renderers\virtual_loudspeaker_renderer.py"
  Delete "$INSTDIR\python\visr_bst\tracker"
  Delete "$INSTDIR\python\visr_bst\tracker\__init__.py"
  Delete "$INSTDIR\python\visr_bst\tracker\razor_ahrs.py"
  Delete "$INSTDIR\python\visr_bst\tracker\razor_ahrs_with_udp_calibration_trigger.py"
  Delete "$INSTDIR\python\visr_bst\util"
  Delete "$INSTDIR\python\visr_bst\util\__init__.py"
  Delete "$INSTDIR\python\visr_bst\util\read_sofa_file.py"
  Delete "$INSTDIR\python\visr_bst\util\real_spherical_harmonics.py"
  Delete "$INSTDIR\python\visr_bst\util\rotation_functions.py"
  Delete "$INSTDIR\python\visr_bst\util\sofa_extract_delay.py"
  Delete "$INSTDIR\python\visr_bst\util\spherical_harmonics_rotation.py"
  RMDir "$INSTDIR\python\visr_bst\controllers"
  RMDir "$INSTDIR\python\visr_bst\data\loudspeaker_configs"
  RMDir "$INSTDIR\python\visr_bst\data\sofa\brir"
  RMDir "$INSTDIR\python\visr_bst\data\sofa\hoa2binaural"
  RMDir "$INSTDIR\python\visr_bst\data\sofa\hrir"
  RMDir "$INSTDIR\python\visr_bst\data\sofa"
  RMDir "$INSTDIR\python\visr_bst\data"
  RMDir "$INSTDIR\python\visr_bst\examples"
  RMDir "$INSTDIR\python\visr_bst\hoa_components"
  RMDir "$INSTDIR\python\visr_bst\realtime_renderers"
  RMDir "$INSTDIR\python\visr_bst\renderers"
  RMDir "$INSTDIR\python\visr_bst\tracker"
  RMDir "$INSTDIR\python\visr_bst\util"
  RMDir "$INSTDIR\python\visr_bst"
  RMDir "$INSTDIR\python"
  noremove_python_package_bst:
!macroend
!macro Select_python_package_bst_depends
  SectionGetFlags ${python_externals} $0
  IntOp $0 $0 | ${SF_SELECTED}
  SectionSetFlags ${python_externals} $0
  IntOp $python_externals_selected 0 + ${SF_SELECTED}
!macroend
!macro Deselect_required_by_python_package_bst
!macroend
!macro Remove_${python_package_metadapter}
  IntCmp $python_package_metadapter_was_installed 0 noremove_python_package_metadapter
  Delete "$INSTDIR\python"
  Delete "$INSTDIR\python\metadapter"
  Delete "$INSTDIR\python\metadapter\.gitignore"
  Delete "$INSTDIR\python\metadapter\__init__.py"
  Delete "$INSTDIR\python\metadapter\condition_processors"
  Delete "$INSTDIR\python\metadapter\condition_processors\__init__.py"
  Delete "$INSTDIR\python\metadapter\condition_processors\condition_processor_factory.py"
  Delete "$INSTDIR\python\metadapter\condition_processors\select_group_condition.py"
  Delete "$INSTDIR\python\metadapter\config"
  Delete "$INSTDIR\python\metadapter\config\C4_demo1_init.xml"
  Delete "$INSTDIR\python\metadapter\config\TMMExperiment2config.xml"
  Delete "$INSTDIR\python\metadapter\config\TMMExperiment3config.xml"
  Delete "$INSTDIR\python\metadapter\config\channel_object.xml"
  Delete "$INSTDIR\python\metadapter\config\conditional_empty_branch.xml"
  Delete "$INSTDIR\python\metadapter\config\conditional_no_default.xml"
  Delete "$INSTDIR\python\metadapter\config\conditional_reordered_branches.xml"
  Delete "$INSTDIR\python\metadapter\config\conditional_single_branch.xml"
  Delete "$INSTDIR\python\metadapter\config\conditional_single_branch_no_merge.xml"
  Delete "$INSTDIR\python\metadapter\config\conditional_test1.xml"
  Delete "$INSTDIR\python\metadapter\config\diffuse_source_test.xml"
  Delete "$INSTDIR\python\metadapter\config\envelopment.xml"
  Delete "$INSTDIR\python\metadapter\config\example_processing_chain_1.xml"
  Delete "$INSTDIR\python\metadapter\config\example_processing_chain_1_quot.xml"
  Delete "$INSTDIR\python\metadapter\config\mdo.xml"
  Delete "$INSTDIR\python\metadapter\config\mdoAzimuth.xml"
  Delete "$INSTDIR\python\metadapter\config\mdoPhones.xml"
  Delete "$INSTDIR\python\metadapter\config\mdo_production.xml"
  Delete "$INSTDIR\python\metadapter\config\narrative_importance.xml"
  Delete "$INSTDIR\python\metadapter\config\nested_sequences.xml"
  Delete "$INSTDIR\python\metadapter\config\processing_chain_no_sequence_layer.xml"
  Delete "$INSTDIR\python\metadapter\config\radio_drama_adaptation_FOW.xml"
  Delete "$INSTDIR\python\metadapter\config\radio_drama_adaptation_demo.xml"
  Delete "$INSTDIR\python\metadapter\config\reverb_config_notes.txt"
  Delete "$INSTDIR\python\metadapter\config\room_acoustic_adaptation_test.xml"
  Delete "$INSTDIR\python\metadapter\config\semantic_stereo.xml"
  Delete "$INSTDIR\python\metadapter\config\simple_mdo.xml"
  Delete "$INSTDIR\python\metadapter\config\smpte_demo.xml"
  Delete "$INSTDIR\python\metadapter\config\source_merge_1.xml"
  Delete "$INSTDIR\python\metadapter\config\speech_steering.xml"
  Delete "$INSTDIR\python\metadapter\config\split_sink_1.xml"
  Delete "$INSTDIR\python\metadapter\config\static_point_source_extent.xml"
  Delete "$INSTDIR\python\metadapter\config\test_add_advanced_metadata.xml"
  Delete "$INSTDIR\python\metadapter\config\test_add_mdo_metadata.xml"
  Delete "$INSTDIR\python\metadapter\config\test_add_reverb.xml"
  Delete "$INSTDIR\python\metadapter\config\test_add_reverb_hoa.xml"
  Delete "$INSTDIR\python\metadapter\config\test_edit_distance.xml"
  Delete "$INSTDIR\python\metadapter\config\test_edit_reverb.xml"
  Delete "$INSTDIR\python\metadapter\config\test_edit_reverb_frame.xml"
  Delete "$INSTDIR\python\metadapter\config\test_edit_reverb_scene.xml"
  Delete "$INSTDIR\python\metadapter\core"
  Delete "$INSTDIR\python\metadapter\core\base_processor_interface.py"
  Delete "$INSTDIR\python\metadapter\core\composite_conditional.py"
  Delete "$INSTDIR\python\metadapter\core\composite_sequence.py"
  Delete "$INSTDIR\python\metadapter\core\composite_source_merge.py"
  Delete "$INSTDIR\python\metadapter\core\composite_split_sink.py"
  Delete "$INSTDIR\python\metadapter\core\condition_processor_interface.py"
  Delete "$INSTDIR\python\metadapter\core\instantiate_subflow.py"
  Delete "$INSTDIR\python\metadapter\core\merge_processor_interface.py"
  Delete "$INSTDIR\python\metadapter\core\metadapter_engine.py"
  Delete "$INSTDIR\python\metadapter\core\metadata_processor_factory.py"
  Delete "$INSTDIR\python\metadapter\core\sequence_processor_interface.py"
  Delete "$INSTDIR\python\metadapter\core\sink_processor_interface.py"
  Delete "$INSTDIR\python\metadapter\core\source_processor_interface.py"
  Delete "$INSTDIR\python\metadapter\examples"
  Delete "$INSTDIR\python\metadapter\examples\test_metadapter.py"
  Delete "$INSTDIR\python\metadapter\max"
  Delete "$INSTDIR\python\metadapter\max\AddReverbOSCController.maxpat"
  Delete "$INSTDIR\python\metadapter\max\ReverbControllerv0.maxpat"
  Delete "$INSTDIR\python\metadapter\merge_processors"
  Delete "$INSTDIR\python\metadapter\merge_processors\__init__.py"
  Delete "$INSTDIR\python\metadapter\merge_processors\branch_order_merge_processor.py"
  Delete "$INSTDIR\python\metadapter\merge_processors\merge_processor_factory.py"
  Delete "$INSTDIR\python\metadapter\metadapter.py"
  Delete "$INSTDIR\python\metadapter\networked"
  Delete "$INSTDIR\python\metadapter\networked\engine.py"
  Delete "$INSTDIR\python\metadapter\processors"
  Delete "$INSTDIR\python\metadapter\processors\Nuendo_height_fix_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\TMMExperiment2_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\TMMExperiment3_MUSHRA_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\TMMExperiment3_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\TMMGeneric_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\__init__.py"
  Delete "$INSTDIR\python\metadapter\processors\adapt_diffuseness_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\adapt_position_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\add_mdo_metadata_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\add_objects_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\add_reverb_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\addadvancedmetadata_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\attenuate_action_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\az_remap.py"
  Delete "$INSTDIR\python\metadapter\processors\change_volume_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\compress_above_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\compress_below_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\compress_height_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\edit_distance_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\edit_reverb_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\envelopment_functions.py"
  Delete "$INSTDIR\python\metadapter\processors\envelopment_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\envelopmentopt_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\fakegroups_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\globaleq_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\group_adapt_radius_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\group_volume_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\mdoAzimuth_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\mdoPhones_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\mdoProduction_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\mdo_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\narrativeimportance_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\print_advanced_metadata_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\print_object_coordinate.py"
  Delete "$INSTDIR\python\metadapter\processors\reverb"
  Delete "$INSTDIR\python\metadapter\processors\reverb\AES142Library.json"
  Delete "$INSTDIR\python\metadapter\processors\reverb\AES142Library1.json"
  Delete "$INSTDIR\python\metadapter\processors\reverb\BFormatParamTest.json"
  Delete "$INSTDIR\python\metadapter\processors\reverb\JAESLibrary.json"
  Delete "$INSTDIR\python\metadapter\processors\room"
  Delete "$INSTDIR\python\metadapter\processors\room\ArrayConfig.xml"
  Delete "$INSTDIR\python\metadapter\processors\room\RoomArrayResponse.json"
  Delete "$INSTDIR\python\metadapter\processors\room_acoustic_adaptation_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\routetosoundbar_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\semantic_stereo_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\solo_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\spat_message_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\spread_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\utilities.py"
  Delete "$INSTDIR\python\metadapter\python_examples"
  Delete "$INSTDIR\python\metadapter\python_examples\README.txt"
  Delete "$INSTDIR\python\metadapter\python_examples\reactor_udp_tutorial.py"
  Delete "$INSTDIR\python\metadapter\python_examples\udpreactor.py"
  Delete "$INSTDIR\python\metadapter\sink_processors"
  Delete "$INSTDIR\python\metadapter\sink_processors\__init__.py"
  Delete "$INSTDIR\python\metadapter\sink_processors\print_scene_processor.py"
  Delete "$INSTDIR\python\metadapter\sink_processors\sink_processor_factory.py"
  Delete "$INSTDIR\python\metadapter\source_processors"
  Delete "$INSTDIR\python\metadapter\source_processors\__init__.py"
  Delete "$INSTDIR\python\metadapter\source_processors\source_processor_factory.py"
  Delete "$INSTDIR\python\metadapter\source_processors\static_scene_source.py"
  Delete "$INSTDIR\python\metadapter\templates"
  Delete "$INSTDIR\python\metadapter\templates\add_objects.py"
  Delete "$INSTDIR\python\metadapter\templates\modify_all_objects.py"
  Delete "$INSTDIR\python\metadapter\thirdparty"
  Delete "$INSTDIR\python\metadapter\thirdparty\OSC.py"
  Delete "$INSTDIR\python\metadapter\thirdparty\__init__.py"
  Delete "$INSTDIR\python\metadapter\thirdparty\thirdparty"
  Delete "$INSTDIR\python\metadapter\thirdparty\thirdparty\OSC.py"
  Delete "$INSTDIR\python\metadapter\thirdparty\thirdparty\__init__.py"
  Delete "$INSTDIR\python\metadapter\visr"
  Delete "$INSTDIR\python\metadapter\visr\component.py"
  RMDir "$INSTDIR\python\metadapter\condition_processors"
  RMDir "$INSTDIR\python\metadapter\config"
  RMDir "$INSTDIR\python\metadapter\core"
  RMDir "$INSTDIR\python\metadapter\examples"
  RMDir "$INSTDIR\python\metadapter\max"
  RMDir "$INSTDIR\python\metadapter\merge_processors"
  RMDir "$INSTDIR\python\metadapter\networked"
  RMDir "$INSTDIR\python\metadapter\processors\reverb"
  RMDir "$INSTDIR\python\metadapter\processors\room"
  RMDir "$INSTDIR\python\metadapter\processors"
  RMDir "$INSTDIR\python\metadapter\python_examples"
  RMDir "$INSTDIR\python\metadapter\sink_processors"
  RMDir "$INSTDIR\python\metadapter\source_processors"
  RMDir "$INSTDIR\python\metadapter\templates"
  RMDir "$INSTDIR\python\metadapter\thirdparty\thirdparty"
  RMDir "$INSTDIR\python\metadapter\thirdparty"
  RMDir "$INSTDIR\python\metadapter\visr"
  RMDir "$INSTDIR\python\metadapter"
  RMDir "$INSTDIR\python"
  noremove_python_package_metadapter:
!macroend
!macro Select_python_package_metadapter_depends
  SectionGetFlags ${python_externals} $0
  IntOp $0 $0 | ${SF_SELECTED}
  SectionSetFlags ${python_externals} $0
  IntOp $python_externals_selected 0 + ${SF_SELECTED}
!macroend
!macro Deselect_required_by_python_package_metadapter
!macroend
!macro Remove_${base}
  IntCmp $base_was_installed 0 noremove_base
  Delete "$INSTDIR\ChangeLog.txt"
  Delete "$INSTDIR\Contributors.md"
  Delete "$INSTDIR\LICENSE.md"
  Delete "$INSTDIR\Readme.md"
  noremove_base:
!macroend
!macro Select_base_depends
!macroend
!macro Deselect_required_by_base
!macroend
!macro Remove_${development_files}
  IntCmp $development_files_was_installed 0 noremove_development_files
  Delete "$INSTDIR\cmake"
  Delete "$INSTDIR\cmake\visr-config-version.cmake"
  Delete "$INSTDIR\cmake\visr-config.cmake"
  Delete "$INSTDIR\cmake\visr_exports-release.cmake"
  Delete "$INSTDIR\cmake\visr_exports.cmake"
  Delete "$INSTDIR\include"
  Delete "$INSTDIR\include\visr"
  Delete "$INSTDIR\include\visr\libapputilities"
  Delete "$INSTDIR\include\visr\libapputilities\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libapputilities\options.hpp"
  Delete "$INSTDIR\include\visr\libaudiointerfaces"
  Delete "$INSTDIR\include\visr\libaudiointerfaces\audio_interface.hpp"
  Delete "$INSTDIR\include\visr\libaudiointerfaces\audio_interface_factory.hpp"
  Delete "$INSTDIR\include\visr\libaudiointerfaces\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libaudiointerfaces\portaudio_interface.hpp"
  Delete "$INSTDIR\include\visr\libefl"
  Delete "$INSTDIR\include\visr\libefl\aligned_array.hpp"
  Delete "$INSTDIR\include\visr\libefl\alignment.hpp"
  Delete "$INSTDIR\include\visr\libefl\basic_matrix.hpp"
  Delete "$INSTDIR\include\visr\libefl\basic_vector.hpp"
  Delete "$INSTDIR\include\visr\libefl\cartesian_spherical_conversion.hpp"
  Delete "$INSTDIR\include\visr\libefl\db_linear_conversion.hpp"
  Delete "$INSTDIR\include\visr\libefl\degree_radian_conversion.hpp"
  Delete "$INSTDIR\include\visr\libefl\denormalised_number_handling.hpp"
  Delete "$INSTDIR\include\visr\libefl\error_codes.hpp"
  Delete "$INSTDIR\include\visr\libefl\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libefl\lagrange_coefficient_calculator.hpp"
  Delete "$INSTDIR\include\visr\libefl\matrix_functions.hpp"
  Delete "$INSTDIR\include\visr\libefl\vector_conversions.hpp"
  Delete "$INSTDIR\include\visr\libefl\vector_functions.hpp"
  Delete "$INSTDIR\include\visr\libefl\vector_functions_reference.hpp"
  Delete "$INSTDIR\include\visr\libefl\vector_functions_reference_impl.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel"
  Delete "$INSTDIR\include\visr\libobjectmodel\channel_object.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\channel_object_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\diffuse_source.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\diffuse_source_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\hoa_source.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\hoa_source_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\object.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\object_factory.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\object_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\object_type.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\object_vector.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\object_vector_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\plane_wave.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\plane_wave_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source_extent.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source_extent_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source_with_diffuseness.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source_with_diffuseness_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source_with_reverb.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source_with_reverb_parser.hpp"
  Delete "$INSTDIR\include\visr\libpanning"
  Delete "$INSTDIR\include\visr\libpanning\AllRAD.h"
  Delete "$INSTDIR\include\visr\libpanning\CAP.h"
  Delete "$INSTDIR\include\visr\libpanning\LoudspeakerArray.h"
  Delete "$INSTDIR\include\visr\libpanning\VBAP.h"
  Delete "$INSTDIR\include\visr\libpanning\XYZ.h"
  Delete "$INSTDIR\include\visr\libpanning\defs.h"
  Delete "$INSTDIR\include\visr\libpanning\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libpml"
  Delete "$INSTDIR\include\visr\libpml\array_configuration.hpp"
  Delete "$INSTDIR\include\visr\libpml\biquad_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\double_buffering_protocol.hpp"
  Delete "$INSTDIR\include\visr\libpml\empty_parameter_config.hpp"
  Delete "$INSTDIR\include\visr\libpml\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libpml\filter_routing_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\indexed_value_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\initialise_parameter_library.hpp"
  Delete "$INSTDIR\include\visr\libpml\interpolation_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\listener_position.hpp"
  Delete "$INSTDIR\include\visr\libpml\matrix_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\matrix_parameter_config.hpp"
  Delete "$INSTDIR\include\visr\libpml\message_queue_protocol.hpp"
  Delete "$INSTDIR\include\visr\libpml\object_vector.hpp"
  Delete "$INSTDIR\include\visr\libpml\scalar_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\shared_data_protocol.hpp"
  Delete "$INSTDIR\include\visr\libpml\signal_routing_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\sparse_gain_routing_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\string_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\time_frequency_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\time_frequency_parameter_config.hpp"
  Delete "$INSTDIR\include\visr\libpml\vector_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\vector_parameter_config.hpp"
  Delete "$INSTDIR\include\visr\libpythoncomponents"
  Delete "$INSTDIR\include\visr\libpythoncomponents\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libpythoncomponents\wrapper.hpp"
  Delete "$INSTDIR\include\visr\libpythonsupport"
  Delete "$INSTDIR\include\visr\libpythonsupport\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libpythonsupport\gil_ensure_guard.hpp"
  Delete "$INSTDIR\include\visr\libpythonsupport\initialisation_guard.hpp"
  Delete "$INSTDIR\include\visr\libpythonsupport\load_module.hpp"
  Delete "$INSTDIR\include\visr\librbbl"
  Delete "$INSTDIR\include\visr\librbbl\biquad_coefficient.hpp"
  Delete "$INSTDIR\include\visr\librbbl\circular_buffer.hpp"
  Delete "$INSTDIR\include\visr\librbbl\core_convolver_uniform.hpp"
  Delete "$INSTDIR\include\visr\librbbl\crossfading_convolver_uniform.hpp"
  Delete "$INSTDIR\include\visr\librbbl\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\librbbl\fft_wrapper_base.hpp"
  Delete "$INSTDIR\include\visr\librbbl\fft_wrapper_factory.hpp"
  Delete "$INSTDIR\include\visr\librbbl\ffts_wrapper.hpp"
  Delete "$INSTDIR\include\visr\librbbl\filter_routing.hpp"
  Delete "$INSTDIR\include\visr\librbbl\fir.hpp"
  Delete "$INSTDIR\include\visr\librbbl\float_sequence.hpp"
  Delete "$INSTDIR\include\visr\librbbl\fractional_delay_base.hpp"
  Delete "$INSTDIR\include\visr\librbbl\fractional_delay_factory.hpp"
  Delete "$INSTDIR\include\visr\librbbl\gain_fader.hpp"
  Delete "$INSTDIR\include\visr\librbbl\gain_matrix.hpp"
  Delete "$INSTDIR\include\visr\librbbl\index_sequence.hpp"
  Delete "$INSTDIR\include\visr\librbbl\interpolating_convolver_uniform.hpp"
  Delete "$INSTDIR\include\visr\librbbl\interpolation_parameter.hpp"
  Delete "$INSTDIR\include\visr\librbbl\kiss_fft_wrapper.hpp"
  Delete "$INSTDIR\include\visr\librbbl\lagrange_interpolator.hpp"
  Delete "$INSTDIR\include\visr\librbbl\multichannel_convolver_uniform.hpp"
  Delete "$INSTDIR\include\visr\librbbl\multichannel_delay_line.hpp"
  Delete "$INSTDIR\include\visr\librbbl\object_channel_allocator.hpp"
  Delete "$INSTDIR\include\visr\librbbl\parametric_iir_coefficient.hpp"
  Delete "$INSTDIR\include\visr\librbbl\parametric_iir_coefficient_calculator.hpp"
  Delete "$INSTDIR\include\visr\librbbl\sparse_gain_routing.hpp"
  Delete "$INSTDIR\include\visr\librcl"
  Delete "$INSTDIR\include\visr\librcl\add.hpp"
  Delete "$INSTDIR\include\visr\librcl\biquad_iir_filter.hpp"
  Delete "$INSTDIR\include\visr\librcl\cap_gain_calculator.hpp"
  Delete "$INSTDIR\include\visr\librcl\channel_object_routing_calculator.hpp"
  Delete "$INSTDIR\include\visr\librcl\crossfading_fir_filter_matrix.hpp"
  Delete "$INSTDIR\include\visr\librcl\delay_matrix.hpp"
  Delete "$INSTDIR\include\visr\librcl\delay_vector.hpp"
  Delete "$INSTDIR\include\visr\librcl\diffusion_gain_calculator.hpp"
  Delete "$INSTDIR\include\visr\librcl\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\librcl\fir_filter_matrix.hpp"
  Delete "$INSTDIR\include\visr\librcl\gain_matrix.hpp"
  Delete "$INSTDIR\include\visr\librcl\gain_vector.hpp"
  Delete "$INSTDIR\include\visr\librcl\hoa_allrad_gain_calculator.hpp"
  Delete "$INSTDIR\include\visr\librcl\interpolating_fir_filter_matrix.hpp"
  Delete "$INSTDIR\include\visr\librcl\listener_compensation.hpp"
  Delete "$INSTDIR\include\visr\librcl\null_source.hpp"
  Delete "$INSTDIR\include\visr\librcl\object_gain_eq_calculator.hpp"
  Delete "$INSTDIR\include\visr\librcl\panning_calculator.hpp"
  Delete "$INSTDIR\include\visr\librcl\position_decoder.hpp"
  Delete "$INSTDIR\include\visr\librcl\scalar_osc_decoder.hpp"
  Delete "$INSTDIR\include\visr\librcl\scene_decoder.hpp"
  Delete "$INSTDIR\include\visr\librcl\scene_encoder.hpp"
  Delete "$INSTDIR\include\visr\librcl\signal_routing.hpp"
  Delete "$INSTDIR\include\visr\librcl\sparse_gain_matrix.hpp"
  Delete "$INSTDIR\include\visr\librcl\time_frequency_inverse_transform.hpp"
  Delete "$INSTDIR\include\visr\librcl\time_frequency_transform.hpp"
  Delete "$INSTDIR\include\visr\librcl\udp_receiver.hpp"
  Delete "$INSTDIR\include\visr\librcl\udp_sender.hpp"
  Delete "$INSTDIR\include\visr\libreverbobject"
  Delete "$INSTDIR\include\visr\libreverbobject\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libreverbobject\late_reverb_filter_calculator.hpp"
  Delete "$INSTDIR\include\visr\libreverbobject\late_reverb_parameter.hpp"
  Delete "$INSTDIR\include\visr\libreverbobject\reverb_object_renderer.hpp"
  Delete "$INSTDIR\include\visr\libreverbobject\reverb_parameter_calculator.hpp"
  Delete "$INSTDIR\include\visr\librrl"
  Delete "$INSTDIR\include\visr\librrl\audio_signal_flow.hpp"
  Delete "$INSTDIR\include\visr\librrl\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\librrl\integrity_checking.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows"
  Delete "$INSTDIR\include\visr\libsignalflows\baseline_renderer.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows\bunch_renderer.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows\core_renderer.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows\delay_vector.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows\gain_matrix.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows\time_frequency_feedthrough.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows\visr_renderer.hpp"
  Delete "$INSTDIR\include\visr\libvisr"
  Delete "$INSTDIR\include\visr\libvisr\atomic_component.hpp"
  Delete "$INSTDIR\include\visr\libvisr\audio_input.hpp"
  Delete "$INSTDIR\include\visr\libvisr\audio_output.hpp"
  Delete "$INSTDIR\include\visr\libvisr\audio_port_base.hpp"
  Delete "$INSTDIR\include\visr\libvisr\audio_sample_type.hpp"
  Delete "$INSTDIR\include\visr\libvisr\channel_list.hpp"
  Delete "$INSTDIR\include\visr\libvisr\communication_protocol_base.hpp"
  Delete "$INSTDIR\include\visr\libvisr\communication_protocol_factory.hpp"
  Delete "$INSTDIR\include\visr\libvisr\communication_protocol_type.hpp"
  Delete "$INSTDIR\include\visr\libvisr\component.hpp"
  Delete "$INSTDIR\include\visr\libvisr\composite_component.hpp"
  Delete "$INSTDIR\include\visr\libvisr\constants.hpp"
  Delete "$INSTDIR\include\visr\libvisr\detail"
  Delete "$INSTDIR\include\visr\libvisr\detail\compile_time_hash_fnv1.hpp"
  Delete "$INSTDIR\include\visr\libvisr\detail\compose_message_string.hpp"
  Delete "$INSTDIR\include\visr\libvisr\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libvisr\parameter_base.hpp"
  Delete "$INSTDIR\include\visr\libvisr\parameter_config_base.hpp"
  Delete "$INSTDIR\include\visr\libvisr\parameter_factory.hpp"
  Delete "$INSTDIR\include\visr\libvisr\parameter_input.hpp"
  Delete "$INSTDIR\include\visr\libvisr\parameter_output.hpp"
  Delete "$INSTDIR\include\visr\libvisr\parameter_port_base.hpp"
  Delete "$INSTDIR\include\visr\libvisr\parameter_type.hpp"
  Delete "$INSTDIR\include\visr\libvisr\polymorphic_parameter_input.hpp"
  Delete "$INSTDIR\include\visr\libvisr\polymorphic_parameter_output.hpp"
  Delete "$INSTDIR\include\visr\libvisr\port_base.hpp"
  Delete "$INSTDIR\include\visr\libvisr\signal_flow_context.hpp"
  Delete "$INSTDIR\include\visr\libvisr\status_message.hpp"
  Delete "$INSTDIR\include\visr\libvisr\typed_parameter_base.hpp"
  Delete "$INSTDIR\include\visr\libvisr\version.hpp"
  Delete "$INSTDIR\lib"
  Delete "$INSTDIR\lib\apputilities_shared.lib"
  Delete "$INSTDIR\lib\audiointerfaces_shared.lib"
  Delete "$INSTDIR\lib\efl_shared.lib"
  Delete "$INSTDIR\lib\objectmodel_shared.lib"
  Delete "$INSTDIR\lib\panning_shared.lib"
  Delete "$INSTDIR\lib\pml_shared.lib"
  Delete "$INSTDIR\lib\pythoncomponents_shared.lib"
  Delete "$INSTDIR\lib\pythonsupport_shared.lib"
  Delete "$INSTDIR\lib\rbbl_shared.lib"
  Delete "$INSTDIR\lib\rcl_shared.lib"
  Delete "$INSTDIR\lib\reverbobject_shared.lib"
  Delete "$INSTDIR\lib\rrl_shared.lib"
  Delete "$INSTDIR\lib\signalflows_shared.lib"
  Delete "$INSTDIR\lib\signalflowspython_shared.lib"
  Delete "$INSTDIR\lib\visr_shared.lib"
  RMDir "$INSTDIR\cmake"
  RMDir "$INSTDIR\include\visr\libapputilities"
  RMDir "$INSTDIR\include\visr\libaudiointerfaces"
  RMDir "$INSTDIR\include\visr\libefl"
  RMDir "$INSTDIR\include\visr\libobjectmodel"
  RMDir "$INSTDIR\include\visr\libpanning"
  RMDir "$INSTDIR\include\visr\libpml"
  RMDir "$INSTDIR\include\visr\libpythoncomponents"
  RMDir "$INSTDIR\include\visr\libpythonsupport"
  RMDir "$INSTDIR\include\visr\librbbl"
  RMDir "$INSTDIR\include\visr\librcl"
  RMDir "$INSTDIR\include\visr\libreverbobject"
  RMDir "$INSTDIR\include\visr\librrl"
  RMDir "$INSTDIR\include\visr\libsignalflows"
  RMDir "$INSTDIR\include\visr\libvisr\detail"
  RMDir "$INSTDIR\include\visr\libvisr"
  RMDir "$INSTDIR\include\visr"
  RMDir "$INSTDIR\include"
  RMDir "$INSTDIR\lib"
  noremove_development_files:
!macroend
!macro Select_development_files_depends
!macroend
!macro Deselect_required_by_development_files
!macroend
!macro Remove_${loudspeaker_configs}
  IntCmp $loudspeaker_configs_was_installed 0 noremove_loudspeaker_configs
  Delete "$INSTDIR\config"
  Delete "$INSTDIR\config\bbc"
  Delete "$INSTDIR\config\bbc\bbc-listening-room-full.xml"
  Delete "$INSTDIR\config\bbc\bs2051-0+2+0.xml"
  Delete "$INSTDIR\config\bbc\bs2051-0+5+0.xml"
  Delete "$INSTDIR\config\bbc\bs2051-2+5+0-speakers-s3a.txt"
  Delete "$INSTDIR\config\bbc\bs2051-2+5+0.xml"
  Delete "$INSTDIR\config\bbc\bs2051-3+7+0-speakers-s3a.txt"
  Delete "$INSTDIR\config\bbc\bs2051-4+5+0-speakers-s3a.txt"
  Delete "$INSTDIR\config\bbc\bs2051-4+5+0.xml"
  Delete "$INSTDIR\config\bbc\bs2051-4+5+1-speakers-s3a.txt"
  Delete "$INSTDIR\config\bbc\bs2051-4+9+0-speakers-s3a.txt"
  Delete "$INSTDIR\config\bbc\bs2051-4+9+0.xml"
  Delete "$INSTDIR\config\bbc\bs2051-9+10+3-speakers-s3a.txt"
  Delete "$INSTDIR\config\bbc\bs2051-9+10+3.xml"
  Delete "$INSTDIR\config\bbc\listeningroomspeakers-s3a.txt"
  Delete "$INSTDIR\config\bbc\scripts"
  Delete "$INSTDIR\config\bbc\scripts\createBBCConfigs.m"
  Delete "$INSTDIR\config\bbc\speaker_config_original"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-0+2+0-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-0+5+0-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-2+5+0-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-3+7+0-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-4+5+0-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-4+5+1-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-4+9+0-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-9+10+3-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\listeningroomspeakers.txt"
  Delete "$INSTDIR\config\filters"
  Delete "$INSTDIR\config\filters\random_phase_allpass_64ch_1024taps.wav"
  Delete "$INSTDIR\config\filters\random_phase_allpass_64ch_128taps.wav"
  Delete "$INSTDIR\config\filters\random_phase_allpass_64ch_256taps.wav"
  Delete "$INSTDIR\config\filters\random_phase_allpass_64ch_512taps.wav"
  Delete "$INSTDIR\config\filters\random_phase_allpass_64ch_512taps_16bit.wav"
  Delete "$INSTDIR\config\filters\random_phase_allpass_64ch_512taps_32bit.wav"
  Delete "$INSTDIR\config\generic"
  Delete "$INSTDIR\config\generic\Transaural_39.txt"
  Delete "$INSTDIR\config\generic\Transaural_42.txt"
  Delete "$INSTDIR\config\generic\Transaural_42_cart.txt"
  Delete "$INSTDIR\config\generic\bs2051-0+2+0.xml"
  Delete "$INSTDIR\config\generic\bs2051-0+5+0.xml"
  Delete "$INSTDIR\config\generic\bs2051-4+5+0.xml"
  Delete "$INSTDIR\config\generic\bs2051-9+10+3.xml"
  Delete "$INSTDIR\config\generic\bs2051-9+10+3_linear.xml"
  Delete "$INSTDIR\config\generic\decode_N8_P40_t-design_t8_P40.txt"
  Delete "$INSTDIR\config\generic\downmix_matrices"
  Delete "$INSTDIR\config\generic\downmix_matrices\22_2_to_2_0.dat"
  Delete "$INSTDIR\config\generic\downmix_matrices\downmix_coefs.mat"
  Delete "$INSTDIR\config\generic\octahedron.xml"
  Delete "$INSTDIR\config\generic\salfordBRIRs.xml"
  Delete "$INSTDIR\config\generic\scripts"
  Delete "$INSTDIR\config\generic\scripts\createBS2051Configs.m"
  Delete "$INSTDIR\config\generic\stereo.xml"
  Delete "$INSTDIR\config\generic\t-design_t8_P40.txt"
  Delete "$INSTDIR\config\isvr"
  Delete "$INSTDIR\config\isvr\audioIfc"
  Delete "$INSTDIR\config\isvr\audioIfc\jackComplexConf.json"
  Delete "$INSTDIR\config\isvr\audioIfc\jackDefConf.json"
  Delete "$INSTDIR\config\isvr\audioIfc\jackPortConf.json"
  Delete "$INSTDIR\config\isvr\audioIfc\jackSimpleConf.json"
  Delete "$INSTDIR\config\isvr\audioIfc\portAudioDefConf.json"
  Delete "$INSTDIR\config\isvr\audiolab_22speakers.xml"
  Delete "$INSTDIR\config\isvr\audiolab_22speakers_1subwoofer.xml"
  Delete "$INSTDIR\config\isvr\audiolab_39speakers.xml"
  Delete "$INSTDIR\config\isvr\audiolab_39speakers_1subwoofer.xml"
  Delete "$INSTDIR\config\isvr\audiolab_5.1_1subwoofer.xml"
  Delete "$INSTDIR\config\isvr\audiolab_9.1_1subwoofer.xml"
  Delete "$INSTDIR\config\isvr\audiolab_cube_1subwoofer.xml"
  Delete "$INSTDIR\config\isvr\audiolab_stereo_1sub_with_rerouting.xml"
  Delete "$INSTDIR\config\isvr\audiolab_stereo_1subwoofer.xml"
  Delete "$INSTDIR\config\isvr\bak"
  Delete "$INSTDIR\config\isvr\bak\22.1_audiolab.txt"
  Delete "$INSTDIR\config\isvr\bak\5.1_audiolab.txt"
  Delete "$INSTDIR\config\isvr\bak\9.1_audiolab.txt"
  Delete "$INSTDIR\config\isvr\bak\cube_audiolab.txt"
  Delete "$INSTDIR\config\isvr\bak\full_39_loudspeaker_layout_audiolab.txt"
  Delete "$INSTDIR\config\isvr\bak\stereo_audiolab.txt"
  Delete "$INSTDIR\config\isvr\data"
  Delete "$INSTDIR\config\isvr\data\audiolab_gain_delay.xls"
  Delete "$INSTDIR\config\isvr\data\gains_distances.mat"
  Delete "$INSTDIR\config\isvr\scripts"
  Delete "$INSTDIR\config\isvr\scripts\createIsvrConfigs.m"
  Delete "$INSTDIR\config\isvr\scripts\createSphereConfig.m"
  Delete "$INSTDIR\config\isvr\scripts\sparSpeakerPos.m"
  Delete "$INSTDIR\config\isvr\scripts\writeArrayConfigXml.m"
  Delete "$INSTDIR\config\isvr\sphere_30channels_1subwoofer.xml"
  Delete "$INSTDIR\config\isvr\sphere_30channels_1subwoofer_crossover.xml"
  Delete "$INSTDIR\config\surrey"
  Delete "$INSTDIR\config\surrey\2.0_surrey.txt"
  Delete "$INSTDIR\config\surrey\22.0_surrey.txt"
  Delete "$INSTDIR\config\surrey\5.0_surrey.txt"
  Delete "$INSTDIR\config\surrey\9.0_surrey.txt"
  Delete "$INSTDIR\config\surrey\obsolete_configurations"
  Delete "$INSTDIR\config\surrey\obsolete_configurations\22.0_surrey_mod_DM.txt"
  Delete "$INSTDIR\config\surrey\obsolete_configurations\22.1_surrey.txt"
  Delete "$INSTDIR\config\surrey\obsolete_configurations\5.1_surrey.txt"
  Delete "$INSTDIR\config\surrey\obsolete_configurations\9.0_surrey_mod_DM.txt"
  Delete "$INSTDIR\config\surrey\obsolete_configurations\9.1_surrey.txt"
  Delete "$INSTDIR\config\surrey\speaker_configuration_surrey_sphere.xml"
  RMDir "$INSTDIR\config\bbc\scripts"
  RMDir "$INSTDIR\config\bbc\speaker_config_original"
  RMDir "$INSTDIR\config\bbc"
  RMDir "$INSTDIR\config\filters"
  RMDir "$INSTDIR\config\generic\downmix_matrices"
  RMDir "$INSTDIR\config\generic\scripts"
  RMDir "$INSTDIR\config\generic"
  RMDir "$INSTDIR\config\isvr\audioIfc"
  RMDir "$INSTDIR\config\isvr\bak"
  RMDir "$INSTDIR\config\isvr\data"
  RMDir "$INSTDIR\config\isvr\scripts"
  RMDir "$INSTDIR\config\isvr"
  RMDir "$INSTDIR\config\surrey\obsolete_configurations"
  RMDir "$INSTDIR\config\surrey"
  RMDir "$INSTDIR\config"
  noremove_loudspeaker_configs:
!macroend
!macro Select_loudspeaker_configs_depends
!macroend
!macro Deselect_required_by_loudspeaker_configs
!macroend
!macro Remove_${python_externals}
  IntCmp $python_externals_was_installed 0 noremove_python_externals
  Delete "$INSTDIR\python"
  Delete "$INSTDIR\python\audiointerfaces.pyd"
  Delete "$INSTDIR\python\efl.pyd"
  Delete "$INSTDIR\python\objectmodel.pyd"
  Delete "$INSTDIR\python\panning.pyd"
  Delete "$INSTDIR\python\pml.pyd"
  Delete "$INSTDIR\python\rbbl.pyd"
  Delete "$INSTDIR\python\rcl.pyd"
  Delete "$INSTDIR\python\reverbobject.pyd"
  Delete "$INSTDIR\python\rrl.pyd"
  Delete "$INSTDIR\python\signalflows.pyd"
  Delete "$INSTDIR\python\visr.pyd"
  RMDir "$INSTDIR\python"
  noremove_python_externals:
!macroend
!macro Select_python_externals_depends
!macroend
!macro Deselect_required_by_python_externals
  SectionGetFlags ${python_package_bst} $0
  IntOp $1 ${SF_SELECTED} ~
  IntOp $0 $0 & $1
  SectionSetFlags ${python_package_bst} $0
  IntOp $python_package_bst_selected 0 + 0
  SectionGetFlags ${python_package_metadapter} $0
  IntOp $1 ${SF_SELECTED} ~
  IntOp $0 $0 & $1
  SectionSetFlags ${python_package_metadapter} $0
  IntOp $python_package_metadapter_selected 0 + 0
  SectionGetFlags ${python_templates} $0
  IntOp $1 ${SF_SELECTED} ~
  IntOp $0 $0 & $1
  SectionSetFlags ${python_templates} $0
  IntOp $python_templates_selected 0 + 0
!macroend
!macro Remove_${python_templates}
  IntCmp $python_templates_was_installed 0 noremove_python_templates
  Delete "$INSTDIR\python"
  Delete "$INSTDIR\python\templates"
  Delete "$INSTDIR\python\templates\audio_meter.py"
  Delete "$INSTDIR\python\templates\audio_meter_evaluation.py"
  Delete "$INSTDIR\python\templates\audio_meter_gui.py"
  RMDir "$INSTDIR\python\templates"
  RMDir "$INSTDIR\python"
  noremove_python_templates:
!macroend
!macro Select_python_templates_depends
  SectionGetFlags ${python_externals} $0
  IntOp $0 $0 | ${SF_SELECTED}
  SectionSetFlags ${python_externals} $0
  IntOp $python_externals_selected 0 + ${SF_SELECTED}
!macroend
!macro Deselect_required_by_python_templates
!macroend
!macro Remove_${shared_libraries}
  IntCmp $shared_libraries_was_installed 0 noremove_shared_libraries
  Delete "$INSTDIR\lib"
  Delete "$INSTDIR\lib\apputilities.dll"
  Delete "$INSTDIR\lib\audiointerfaces.dll"
  Delete "$INSTDIR\lib\efl.dll"
  Delete "$INSTDIR\lib\objectmodel.dll"
  Delete "$INSTDIR\lib\panning.dll"
  Delete "$INSTDIR\lib\pml.dll"
  Delete "$INSTDIR\lib\pythoncomponents.dll"
  Delete "$INSTDIR\lib\pythonsupport.dll"
  Delete "$INSTDIR\lib\rbbl.dll"
  Delete "$INSTDIR\lib\rcl.dll"
  Delete "$INSTDIR\lib\reverbobject.dll"
  Delete "$INSTDIR\lib\rrl.dll"
  Delete "$INSTDIR\lib\signalflows.dll"
  Delete "$INSTDIR\lib\signalflowspython.dll"
  Delete "$INSTDIR\lib\visr.dll"
  RMDir "$INSTDIR\lib"
  noremove_shared_libraries:
!macroend
!macro Select_shared_libraries_depends
!macroend
!macro Deselect_required_by_shared_libraries
  SectionGetFlags ${standalone_applications} $0
  IntOp $1 ${SF_SELECTED} ~
  IntOp $0 $0 & $1
  SectionSetFlags ${standalone_applications} $0
  IntOp $standalone_applications_selected 0 + 0
!macroend
!macro Remove_${standalone_applications}
  IntCmp $standalone_applications_was_installed 0 noremove_standalone_applications
  Delete "$INSTDIR\bin"
  Delete "$INSTDIR\bin\baseline_renderer.exe"
  Delete "$INSTDIR\bin\matrix_convolver.exe"
  Delete "$INSTDIR\bin\network_audio_streamer.exe"
  Delete "$INSTDIR\bin\python_runner.exe"
  Delete "$INSTDIR\bin\visr_renderer.exe"
  RMDir "$INSTDIR\bin"
  noremove_standalone_applications:
!macroend
!macro Select_standalone_applications_depends
  SectionGetFlags ${shared_libraries} $0
  IntOp $0 $0 | ${SF_SELECTED}
  SectionSetFlags ${shared_libraries} $0
  IntOp $shared_libraries_selected 0 + ${SF_SELECTED}
!macroend
!macro Deselect_required_by_standalone_applications
!macroend
!macro Remove_${thirdparty_libraries}
  IntCmp $thirdparty_libraries_was_installed 0 noremove_thirdparty_libraries
  Delete "$INSTDIR\3rd"
  Delete "$INSTDIR\3rd\boost_atomic-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_chrono-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_date_time-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_filesystem-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_program_options-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_regex-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_system-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_thread-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_timer-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\libsndfile-1.dll"
  Delete "$INSTDIR\3rd\portaudio_x64.dll"
  RMDir "$INSTDIR\3rd"
  noremove_thirdparty_libraries:
!macroend
!macro Select_thirdparty_libraries_depends
!macroend
!macro Deselect_required_by_thirdparty_libraries
!macroend


;--------------------------------
; Define some macro setting for the gui




!insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
  !insertmacro MUI_DESCRIPTION_TEXT ${api_documentation_pdf} "Code reference documentation"
  !insertmacro MUI_DESCRIPTION_TEXT ${base} "Basic required files."
  !insertmacro MUI_DESCRIPTION_TEXT ${development_files} "Header files and CMake support"
  !insertmacro MUI_DESCRIPTION_TEXT ${doxygen_documentation_html} "Code reference documentation in Doxygen format (deprecated)"
  !insertmacro MUI_DESCRIPTION_TEXT ${loudspeaker_configs} "Example configuration files for the loudspeaker renderers."
  !insertmacro MUI_DESCRIPTION_TEXT ${python_externals} "Python modules to access the VISR functionality from Python"
  !insertmacro MUI_DESCRIPTION_TEXT ${python_package_bst} "Toolbox for binaural synthesis"
  !insertmacro MUI_DESCRIPTION_TEXT ${python_package_metadapter} "Extensible system for object metadata adaptation"
  !insertmacro MUI_DESCRIPTION_TEXT ${python_templates} "Python template files"
  !insertmacro MUI_DESCRIPTION_TEXT ${shared_libraries} "Core VISR libraries (shared)"
  !insertmacro MUI_DESCRIPTION_TEXT ${standalone_applications} "Standalone command-line applications"
  !insertmacro MUI_DESCRIPTION_TEXT ${thirdparty_libraries} "Shared 3rd-party libraries."
  !insertmacro MUI_DESCRIPTION_TEXT ${user_documentation_pdf} "User documentation"
  !insertmacro MUI_DESCRIPTION_TEXT ${documentation} "User and API documentation"
  !insertmacro MUI_DESCRIPTION_TEXT ${python_packages} "VISR extension packages implemented in Python"
!insertmacro MUI_FUNCTION_DESCRIPTION_END



;--------------------------------
;Pages
  !insertmacro MUI_PAGE_WELCOME

  !insertmacro MUI_PAGE_LICENSE "C:/Local/dev/gitlab-runner/builds/a913a013/0/s3a/VISR/build_py36/package_resources/LICENSE.txt"
  Page custom InstallOptionsPage
  !insertmacro MUI_PAGE_DIRECTORY

  ;Start Menu Folder Page Configuration
  !define MUI_STARTMENUPAGE_REGISTRY_ROOT "SHCTX"
  !define MUI_STARTMENUPAGE_REGISTRY_KEY "Software\ISVR\VISR-0.11.1"
  !define MUI_STARTMENUPAGE_REGISTRY_VALUENAME "Start Menu Folder"
  !insertmacro MUI_PAGE_STARTMENU Application $STARTMENU_FOLDER

  !insertmacro MUI_PAGE_COMPONENTS

  !insertmacro MUI_PAGE_INSTFILES
  !insertmacro MUI_PAGE_FINISH

  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES

;--------------------------------
;Languages

  !insertmacro MUI_LANGUAGE "English" ;first language is the default language
  !insertmacro MUI_LANGUAGE "Albanian"
  !insertmacro MUI_LANGUAGE "Arabic"
  !insertmacro MUI_LANGUAGE "Basque"
  !insertmacro MUI_LANGUAGE "Belarusian"
  !insertmacro MUI_LANGUAGE "Bosnian"
  !insertmacro MUI_LANGUAGE "Breton"
  !insertmacro MUI_LANGUAGE "Bulgarian"
  !insertmacro MUI_LANGUAGE "Croatian"
  !insertmacro MUI_LANGUAGE "Czech"
  !insertmacro MUI_LANGUAGE "Danish"
  !insertmacro MUI_LANGUAGE "Dutch"
  !insertmacro MUI_LANGUAGE "Estonian"
  !insertmacro MUI_LANGUAGE "Farsi"
  !insertmacro MUI_LANGUAGE "Finnish"
  !insertmacro MUI_LANGUAGE "French"
  !insertmacro MUI_LANGUAGE "German"
  !insertmacro MUI_LANGUAGE "Greek"
  !insertmacro MUI_LANGUAGE "Hebrew"
  !insertmacro MUI_LANGUAGE "Hungarian"
  !insertmacro MUI_LANGUAGE "Icelandic"
  !insertmacro MUI_LANGUAGE "Indonesian"
  !insertmacro MUI_LANGUAGE "Irish"
  !insertmacro MUI_LANGUAGE "Italian"
  !insertmacro MUI_LANGUAGE "Japanese"
  !insertmacro MUI_LANGUAGE "Korean"
  !insertmacro MUI_LANGUAGE "Kurdish"
  !insertmacro MUI_LANGUAGE "Latvian"
  !insertmacro MUI_LANGUAGE "Lithuanian"
  !insertmacro MUI_LANGUAGE "Luxembourgish"
  !insertmacro MUI_LANGUAGE "Macedonian"
  !insertmacro MUI_LANGUAGE "Malay"
  !insertmacro MUI_LANGUAGE "Mongolian"
  !insertmacro MUI_LANGUAGE "Norwegian"
  !insertmacro MUI_LANGUAGE "Polish"
  !insertmacro MUI_LANGUAGE "Portuguese"
  !insertmacro MUI_LANGUAGE "PortugueseBR"
  !insertmacro MUI_LANGUAGE "Romanian"
  !insertmacro MUI_LANGUAGE "Russian"
  !insertmacro MUI_LANGUAGE "Serbian"
  !insertmacro MUI_LANGUAGE "SerbianLatin"
  !insertmacro MUI_LANGUAGE "SimpChinese"
  !insertmacro MUI_LANGUAGE "Slovak"
  !insertmacro MUI_LANGUAGE "Slovenian"
  !insertmacro MUI_LANGUAGE "Spanish"
  !insertmacro MUI_LANGUAGE "Swedish"
  !insertmacro MUI_LANGUAGE "Thai"
  !insertmacro MUI_LANGUAGE "TradChinese"
  !insertmacro MUI_LANGUAGE "Turkish"
  !insertmacro MUI_LANGUAGE "Ukrainian"
  !insertmacro MUI_LANGUAGE "Welsh"


;--------------------------------
;Reserve Files

  ;These files should be inserted before other files in the data block
  ;Keep these lines before any File command
  ;Only for solid compression (by default, solid compression is enabled for BZIP2 and LZMA)

  ReserveFile "NSIS.InstallOptions.ini"
  !insertmacro MUI_RESERVEFILE_INSTALLOPTIONS

;--------------------------------
;Installer Sections

Section "-Core installation"
  ;Use the entire tree produced by the INSTALL target.  Keep the
  ;list of directories here in sync with the RMDir commands below.
  SetOutPath "$INSTDIR"
  
  

  ;Store installation folder
  WriteRegStr SHCTX "Software\ISVR\VISR-0.11.1" "" $INSTDIR

  ;Create uninstaller
  WriteUninstaller "$INSTDIR\Uninstall.exe"
  Push "DisplayName"
  Push "VISR-0.11.1"
  Call ConditionalAddToRegisty
  Push "DisplayVersion"
  Push "0.11.1"
  Call ConditionalAddToRegisty
  Push "Publisher"
  Push "ISVR"
  Call ConditionalAddToRegisty
  Push "UninstallString"
  Push "$INSTDIR\Uninstall.exe"
  Call ConditionalAddToRegisty
  Push "NoRepair"
  Push "1"
  Call ConditionalAddToRegisty

  !ifdef CPACK_NSIS_ADD_REMOVE
  ;Create add/remove functionality
  Push "ModifyPath"
  Push "$INSTDIR\AddRemove.exe"
  Call ConditionalAddToRegisty
  !else
  Push "NoModify"
  Push "1"
  Call ConditionalAddToRegisty
  !endif

  ; Optional registration
  Push "DisplayIcon"
  Push "$INSTDIR\"
  Call ConditionalAddToRegisty
  Push "HelpLink"
  Push ""
  Call ConditionalAddToRegisty
  Push "URLInfoAbout"
  Push ""
  Call ConditionalAddToRegisty
  Push "Contact"
  Push ""
  Call ConditionalAddToRegisty
  !insertmacro MUI_INSTALLOPTIONS_READ $INSTALL_DESKTOP "NSIS.InstallOptions.ini" "Field 5" "State"
  !insertmacro MUI_STARTMENU_WRITE_BEGIN Application

  ;Create shortcuts
  CreateDirectory "$SMPROGRAMS\$STARTMENU_FOLDER"


  CreateShortCut "$SMPROGRAMS\$STARTMENU_FOLDER\Uninstall.lnk" "$INSTDIR\Uninstall.exe"

  ;Read a value from an InstallOptions INI file
  !insertmacro MUI_INSTALLOPTIONS_READ $DO_NOT_ADD_TO_PATH "NSIS.InstallOptions.ini" "Field 2" "State"
  !insertmacro MUI_INSTALLOPTIONS_READ $ADD_TO_PATH_ALL_USERS "NSIS.InstallOptions.ini" "Field 3" "State"
  !insertmacro MUI_INSTALLOPTIONS_READ $ADD_TO_PATH_CURRENT_USER "NSIS.InstallOptions.ini" "Field 4" "State"

  ; Write special uninstall registry entries
  Push "StartMenu"
  Push "$STARTMENU_FOLDER"
  Call ConditionalAddToRegisty
  Push "DoNotAddToPath"
  Push "$DO_NOT_ADD_TO_PATH"
  Call ConditionalAddToRegisty
  Push "AddToPathAllUsers"
  Push "$ADD_TO_PATH_ALL_USERS"
  Call ConditionalAddToRegisty
  Push "AddToPathCurrentUser"
  Push "$ADD_TO_PATH_CURRENT_USER"
  Call ConditionalAddToRegisty
  Push "InstallToDesktop"
  Push "$INSTALL_DESKTOP"
  Call ConditionalAddToRegisty

  !insertmacro MUI_STARTMENU_WRITE_END



SectionEnd

Section "-Add to path"
  Push $INSTDIR\bin
  StrCmp "" "ON" 0 doNotAddToPath
  StrCmp $DO_NOT_ADD_TO_PATH "1" doNotAddToPath 0
    Call AddToPath
  doNotAddToPath:
SectionEnd

;--------------------------------
; Create custom pages
Function InstallOptionsPage
  !insertmacro MUI_HEADER_TEXT "Install Options" "Choose options for installing VISR-0.11.1"
  !insertmacro MUI_INSTALLOPTIONS_DISPLAY "NSIS.InstallOptions.ini"

FunctionEnd

;--------------------------------
; determine admin versus local install
Function un.onInit

  ClearErrors
  UserInfo::GetName
  IfErrors noLM
  Pop $0
  UserInfo::GetAccountType
  Pop $1
  StrCmp $1 "Admin" 0 +3
    SetShellVarContext all
    ;MessageBox MB_OK 'User "$0" is in the Admin group'
    Goto done
  StrCmp $1 "Power" 0 +3
    SetShellVarContext all
    ;MessageBox MB_OK 'User "$0" is in the Power Users group'
    Goto done

  noLM:
    ;Get installation folder from registry if available

  done:

FunctionEnd

;--- Add/Remove callback functions: ---
!macro SectionList MacroName
  ;This macro used to perform operation on multiple sections.
  ;List all of your components in following manner here.
  !insertmacro "${MacroName}" "api_documentation_pdf"
  !insertmacro "${MacroName}" "base"
  !insertmacro "${MacroName}" "development_files"
  !insertmacro "${MacroName}" "doxygen_documentation_html"
  !insertmacro "${MacroName}" "loudspeaker_configs"
  !insertmacro "${MacroName}" "python_externals"
  !insertmacro "${MacroName}" "python_package_bst"
  !insertmacro "${MacroName}" "python_package_metadapter"
  !insertmacro "${MacroName}" "python_templates"
  !insertmacro "${MacroName}" "shared_libraries"
  !insertmacro "${MacroName}" "standalone_applications"
  !insertmacro "${MacroName}" "thirdparty_libraries"
  !insertmacro "${MacroName}" "user_documentation_pdf"

!macroend

Section -FinishComponents
  ;Removes unselected components and writes component status to registry
  !insertmacro SectionList "FinishSection"

!ifdef CPACK_NSIS_ADD_REMOVE
  ; Get the name of the installer executable
  System::Call 'kernel32::GetModuleFileNameA(i 0, t .R0, i 1024) i r1'
  StrCpy $R3 $R0

  ; Strip off the last 13 characters, to see if we have AddRemove.exe
  StrLen $R1 $R0
  IntOp $R1 $R0 - 13
  StrCpy $R2 $R0 13 $R1
  StrCmp $R2 "AddRemove.exe" addremove_installed

  ; We're not running AddRemove.exe, so install it
  CopyFiles $R3 $INSTDIR\AddRemove.exe

  addremove_installed:
!endif
SectionEnd
;--- End of Add/Remove callback functions ---

;--------------------------------
; Component dependencies
Function .onSelChange
  !insertmacro SectionList MaybeSelectionChanged
FunctionEnd

;--------------------------------
;Uninstaller Section

Section "Uninstall"
  ReadRegStr $START_MENU SHCTX \
   "Software\Microsoft\Windows\CurrentVersion\Uninstall\VISR-0.11.1" "StartMenu"
  ;MessageBox MB_OK "Start menu is in: $START_MENU"
  ReadRegStr $DO_NOT_ADD_TO_PATH SHCTX \
    "Software\Microsoft\Windows\CurrentVersion\Uninstall\VISR-0.11.1" "DoNotAddToPath"
  ReadRegStr $ADD_TO_PATH_ALL_USERS SHCTX \
    "Software\Microsoft\Windows\CurrentVersion\Uninstall\VISR-0.11.1" "AddToPathAllUsers"
  ReadRegStr $ADD_TO_PATH_CURRENT_USER SHCTX \
    "Software\Microsoft\Windows\CurrentVersion\Uninstall\VISR-0.11.1" "AddToPathCurrentUser"
  ;MessageBox MB_OK "Add to path: $DO_NOT_ADD_TO_PATH all users: $ADD_TO_PATH_ALL_USERS"
  ReadRegStr $INSTALL_DESKTOP SHCTX \
    "Software\Microsoft\Windows\CurrentVersion\Uninstall\VISR-0.11.1" "InstallToDesktop"
  ;MessageBox MB_OK "Install to desktop: $INSTALL_DESKTOP "



  ;Remove files we installed.
  ;Keep the list of directories here in sync with the File commands above.
  Delete "$INSTDIR\api_documentation_pdf"
  Delete "$INSTDIR\doc"
  Delete "$INSTDIR\doc\apidoc-0.11.1.pdf"
  Delete "$INSTDIR\base"
  Delete "$INSTDIR\ChangeLog.txt"
  Delete "$INSTDIR\Contributors.md"
  Delete "$INSTDIR\LICENSE.md"
  Delete "$INSTDIR\Readme.md"
  Delete "$INSTDIR\development_files"
  Delete "$INSTDIR\cmake"
  Delete "$INSTDIR\cmake\visr-config-version.cmake"
  Delete "$INSTDIR\cmake\visr-config.cmake"
  Delete "$INSTDIR\cmake\visr_exports-release.cmake"
  Delete "$INSTDIR\cmake\visr_exports.cmake"
  Delete "$INSTDIR\include"
  Delete "$INSTDIR\include\visr"
  Delete "$INSTDIR\include\visr\libapputilities"
  Delete "$INSTDIR\include\visr\libapputilities\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libapputilities\options.hpp"
  Delete "$INSTDIR\include\visr\libaudiointerfaces"
  Delete "$INSTDIR\include\visr\libaudiointerfaces\audio_interface.hpp"
  Delete "$INSTDIR\include\visr\libaudiointerfaces\audio_interface_factory.hpp"
  Delete "$INSTDIR\include\visr\libaudiointerfaces\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libaudiointerfaces\portaudio_interface.hpp"
  Delete "$INSTDIR\include\visr\libefl"
  Delete "$INSTDIR\include\visr\libefl\aligned_array.hpp"
  Delete "$INSTDIR\include\visr\libefl\alignment.hpp"
  Delete "$INSTDIR\include\visr\libefl\basic_matrix.hpp"
  Delete "$INSTDIR\include\visr\libefl\basic_vector.hpp"
  Delete "$INSTDIR\include\visr\libefl\cartesian_spherical_conversion.hpp"
  Delete "$INSTDIR\include\visr\libefl\db_linear_conversion.hpp"
  Delete "$INSTDIR\include\visr\libefl\degree_radian_conversion.hpp"
  Delete "$INSTDIR\include\visr\libefl\denormalised_number_handling.hpp"
  Delete "$INSTDIR\include\visr\libefl\error_codes.hpp"
  Delete "$INSTDIR\include\visr\libefl\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libefl\lagrange_coefficient_calculator.hpp"
  Delete "$INSTDIR\include\visr\libefl\matrix_functions.hpp"
  Delete "$INSTDIR\include\visr\libefl\vector_conversions.hpp"
  Delete "$INSTDIR\include\visr\libefl\vector_functions.hpp"
  Delete "$INSTDIR\include\visr\libefl\vector_functions_reference.hpp"
  Delete "$INSTDIR\include\visr\libefl\vector_functions_reference_impl.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel"
  Delete "$INSTDIR\include\visr\libobjectmodel\channel_object.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\channel_object_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\diffuse_source.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\diffuse_source_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\hoa_source.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\hoa_source_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\object.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\object_factory.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\object_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\object_type.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\object_vector.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\object_vector_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\plane_wave.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\plane_wave_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source_extent.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source_extent_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source_with_diffuseness.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source_with_diffuseness_parser.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source_with_reverb.hpp"
  Delete "$INSTDIR\include\visr\libobjectmodel\point_source_with_reverb_parser.hpp"
  Delete "$INSTDIR\include\visr\libpanning"
  Delete "$INSTDIR\include\visr\libpanning\AllRAD.h"
  Delete "$INSTDIR\include\visr\libpanning\CAP.h"
  Delete "$INSTDIR\include\visr\libpanning\defs.h"
  Delete "$INSTDIR\include\visr\libpanning\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libpanning\LoudspeakerArray.h"
  Delete "$INSTDIR\include\visr\libpanning\VBAP.h"
  Delete "$INSTDIR\include\visr\libpanning\XYZ.h"
  Delete "$INSTDIR\include\visr\libpml"
  Delete "$INSTDIR\include\visr\libpml\array_configuration.hpp"
  Delete "$INSTDIR\include\visr\libpml\biquad_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\double_buffering_protocol.hpp"
  Delete "$INSTDIR\include\visr\libpml\empty_parameter_config.hpp"
  Delete "$INSTDIR\include\visr\libpml\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libpml\filter_routing_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\indexed_value_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\initialise_parameter_library.hpp"
  Delete "$INSTDIR\include\visr\libpml\interpolation_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\listener_position.hpp"
  Delete "$INSTDIR\include\visr\libpml\matrix_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\matrix_parameter_config.hpp"
  Delete "$INSTDIR\include\visr\libpml\message_queue_protocol.hpp"
  Delete "$INSTDIR\include\visr\libpml\object_vector.hpp"
  Delete "$INSTDIR\include\visr\libpml\scalar_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\shared_data_protocol.hpp"
  Delete "$INSTDIR\include\visr\libpml\signal_routing_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\sparse_gain_routing_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\string_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\time_frequency_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\time_frequency_parameter_config.hpp"
  Delete "$INSTDIR\include\visr\libpml\vector_parameter.hpp"
  Delete "$INSTDIR\include\visr\libpml\vector_parameter_config.hpp"
  Delete "$INSTDIR\include\visr\libpythoncomponents"
  Delete "$INSTDIR\include\visr\libpythoncomponents\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libpythoncomponents\wrapper.hpp"
  Delete "$INSTDIR\include\visr\libpythonsupport"
  Delete "$INSTDIR\include\visr\libpythonsupport\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libpythonsupport\gil_ensure_guard.hpp"
  Delete "$INSTDIR\include\visr\libpythonsupport\initialisation_guard.hpp"
  Delete "$INSTDIR\include\visr\libpythonsupport\load_module.hpp"
  Delete "$INSTDIR\include\visr\librbbl"
  Delete "$INSTDIR\include\visr\librbbl\biquad_coefficient.hpp"
  Delete "$INSTDIR\include\visr\librbbl\circular_buffer.hpp"
  Delete "$INSTDIR\include\visr\librbbl\core_convolver_uniform.hpp"
  Delete "$INSTDIR\include\visr\librbbl\crossfading_convolver_uniform.hpp"
  Delete "$INSTDIR\include\visr\librbbl\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\librbbl\ffts_wrapper.hpp"
  Delete "$INSTDIR\include\visr\librbbl\fft_wrapper_base.hpp"
  Delete "$INSTDIR\include\visr\librbbl\fft_wrapper_factory.hpp"
  Delete "$INSTDIR\include\visr\librbbl\filter_routing.hpp"
  Delete "$INSTDIR\include\visr\librbbl\fir.hpp"
  Delete "$INSTDIR\include\visr\librbbl\float_sequence.hpp"
  Delete "$INSTDIR\include\visr\librbbl\fractional_delay_base.hpp"
  Delete "$INSTDIR\include\visr\librbbl\fractional_delay_factory.hpp"
  Delete "$INSTDIR\include\visr\librbbl\gain_fader.hpp"
  Delete "$INSTDIR\include\visr\librbbl\gain_matrix.hpp"
  Delete "$INSTDIR\include\visr\librbbl\index_sequence.hpp"
  Delete "$INSTDIR\include\visr\librbbl\interpolating_convolver_uniform.hpp"
  Delete "$INSTDIR\include\visr\librbbl\interpolation_parameter.hpp"
  Delete "$INSTDIR\include\visr\librbbl\kiss_fft_wrapper.hpp"
  Delete "$INSTDIR\include\visr\librbbl\lagrange_interpolator.hpp"
  Delete "$INSTDIR\include\visr\librbbl\multichannel_convolver_uniform.hpp"
  Delete "$INSTDIR\include\visr\librbbl\multichannel_delay_line.hpp"
  Delete "$INSTDIR\include\visr\librbbl\object_channel_allocator.hpp"
  Delete "$INSTDIR\include\visr\librbbl\parametric_iir_coefficient.hpp"
  Delete "$INSTDIR\include\visr\librbbl\parametric_iir_coefficient_calculator.hpp"
  Delete "$INSTDIR\include\visr\librbbl\sparse_gain_routing.hpp"
  Delete "$INSTDIR\include\visr\librcl"
  Delete "$INSTDIR\include\visr\librcl\add.hpp"
  Delete "$INSTDIR\include\visr\librcl\biquad_iir_filter.hpp"
  Delete "$INSTDIR\include\visr\librcl\cap_gain_calculator.hpp"
  Delete "$INSTDIR\include\visr\librcl\channel_object_routing_calculator.hpp"
  Delete "$INSTDIR\include\visr\librcl\crossfading_fir_filter_matrix.hpp"
  Delete "$INSTDIR\include\visr\librcl\delay_matrix.hpp"
  Delete "$INSTDIR\include\visr\librcl\delay_vector.hpp"
  Delete "$INSTDIR\include\visr\librcl\diffusion_gain_calculator.hpp"
  Delete "$INSTDIR\include\visr\librcl\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\librcl\fir_filter_matrix.hpp"
  Delete "$INSTDIR\include\visr\librcl\gain_matrix.hpp"
  Delete "$INSTDIR\include\visr\librcl\gain_vector.hpp"
  Delete "$INSTDIR\include\visr\librcl\hoa_allrad_gain_calculator.hpp"
  Delete "$INSTDIR\include\visr\librcl\interpolating_fir_filter_matrix.hpp"
  Delete "$INSTDIR\include\visr\librcl\listener_compensation.hpp"
  Delete "$INSTDIR\include\visr\librcl\null_source.hpp"
  Delete "$INSTDIR\include\visr\librcl\object_gain_eq_calculator.hpp"
  Delete "$INSTDIR\include\visr\librcl\panning_calculator.hpp"
  Delete "$INSTDIR\include\visr\librcl\position_decoder.hpp"
  Delete "$INSTDIR\include\visr\librcl\scalar_osc_decoder.hpp"
  Delete "$INSTDIR\include\visr\librcl\scene_decoder.hpp"
  Delete "$INSTDIR\include\visr\librcl\scene_encoder.hpp"
  Delete "$INSTDIR\include\visr\librcl\signal_routing.hpp"
  Delete "$INSTDIR\include\visr\librcl\sparse_gain_matrix.hpp"
  Delete "$INSTDIR\include\visr\librcl\time_frequency_inverse_transform.hpp"
  Delete "$INSTDIR\include\visr\librcl\time_frequency_transform.hpp"
  Delete "$INSTDIR\include\visr\librcl\udp_receiver.hpp"
  Delete "$INSTDIR\include\visr\librcl\udp_sender.hpp"
  Delete "$INSTDIR\include\visr\libreverbobject"
  Delete "$INSTDIR\include\visr\libreverbobject\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libreverbobject\late_reverb_filter_calculator.hpp"
  Delete "$INSTDIR\include\visr\libreverbobject\late_reverb_parameter.hpp"
  Delete "$INSTDIR\include\visr\libreverbobject\reverb_object_renderer.hpp"
  Delete "$INSTDIR\include\visr\libreverbobject\reverb_parameter_calculator.hpp"
  Delete "$INSTDIR\include\visr\librrl"
  Delete "$INSTDIR\include\visr\librrl\audio_signal_flow.hpp"
  Delete "$INSTDIR\include\visr\librrl\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\librrl\integrity_checking.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows"
  Delete "$INSTDIR\include\visr\libsignalflows\baseline_renderer.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows\bunch_renderer.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows\core_renderer.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows\delay_vector.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows\gain_matrix.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows\time_frequency_feedthrough.hpp"
  Delete "$INSTDIR\include\visr\libsignalflows\visr_renderer.hpp"
  Delete "$INSTDIR\include\visr\libvisr"
  Delete "$INSTDIR\include\visr\libvisr\atomic_component.hpp"
  Delete "$INSTDIR\include\visr\libvisr\audio_input.hpp"
  Delete "$INSTDIR\include\visr\libvisr\audio_output.hpp"
  Delete "$INSTDIR\include\visr\libvisr\audio_port_base.hpp"
  Delete "$INSTDIR\include\visr\libvisr\audio_sample_type.hpp"
  Delete "$INSTDIR\include\visr\libvisr\channel_list.hpp"
  Delete "$INSTDIR\include\visr\libvisr\communication_protocol_base.hpp"
  Delete "$INSTDIR\include\visr\libvisr\communication_protocol_factory.hpp"
  Delete "$INSTDIR\include\visr\libvisr\communication_protocol_type.hpp"
  Delete "$INSTDIR\include\visr\libvisr\component.hpp"
  Delete "$INSTDIR\include\visr\libvisr\composite_component.hpp"
  Delete "$INSTDIR\include\visr\libvisr\constants.hpp"
  Delete "$INSTDIR\include\visr\libvisr\detail"
  Delete "$INSTDIR\include\visr\libvisr\detail\compile_time_hash_fnv1.hpp"
  Delete "$INSTDIR\include\visr\libvisr\detail\compose_message_string.hpp"
  Delete "$INSTDIR\include\visr\libvisr\export_symbols.hpp"
  Delete "$INSTDIR\include\visr\libvisr\parameter_base.hpp"
  Delete "$INSTDIR\include\visr\libvisr\parameter_config_base.hpp"
  Delete "$INSTDIR\include\visr\libvisr\parameter_factory.hpp"
  Delete "$INSTDIR\include\visr\libvisr\parameter_input.hpp"
  Delete "$INSTDIR\include\visr\libvisr\parameter_output.hpp"
  Delete "$INSTDIR\include\visr\libvisr\parameter_port_base.hpp"
  Delete "$INSTDIR\include\visr\libvisr\parameter_type.hpp"
  Delete "$INSTDIR\include\visr\libvisr\polymorphic_parameter_input.hpp"
  Delete "$INSTDIR\include\visr\libvisr\polymorphic_parameter_output.hpp"
  Delete "$INSTDIR\include\visr\libvisr\port_base.hpp"
  Delete "$INSTDIR\include\visr\libvisr\signal_flow_context.hpp"
  Delete "$INSTDIR\include\visr\libvisr\status_message.hpp"
  Delete "$INSTDIR\include\visr\libvisr\typed_parameter_base.hpp"
  Delete "$INSTDIR\include\visr\libvisr\version.hpp"
  Delete "$INSTDIR\lib"
  Delete "$INSTDIR\lib\apputilities_shared.lib"
  Delete "$INSTDIR\lib\audiointerfaces_shared.lib"
  Delete "$INSTDIR\lib\efl_shared.lib"
  Delete "$INSTDIR\lib\objectmodel_shared.lib"
  Delete "$INSTDIR\lib\panning_shared.lib"
  Delete "$INSTDIR\lib\pml_shared.lib"
  Delete "$INSTDIR\lib\pythoncomponents_shared.lib"
  Delete "$INSTDIR\lib\pythonsupport_shared.lib"
  Delete "$INSTDIR\lib\rbbl_shared.lib"
  Delete "$INSTDIR\lib\rcl_shared.lib"
  Delete "$INSTDIR\lib\reverbobject_shared.lib"
  Delete "$INSTDIR\lib\rrl_shared.lib"
  Delete "$INSTDIR\lib\signalflowspython_shared.lib"
  Delete "$INSTDIR\lib\signalflows_shared.lib"
  Delete "$INSTDIR\lib\visr_shared.lib"
  Delete "$INSTDIR\doxygen_documentation_html"
  Delete "$INSTDIR\doc"
  Delete "$INSTDIR\doc\html"
  Delete "$INSTDIR\doc\html\add_8hpp.html"
  Delete "$INSTDIR\doc\html\add_8hpp_source.html"
  Delete "$INSTDIR\doc\html\aligned__array_8hpp.html"
  Delete "$INSTDIR\doc\html\aligned__array_8hpp_source.html"
  Delete "$INSTDIR\doc\html\alignment_8hpp.html"
  Delete "$INSTDIR\doc\html\alignment_8hpp.js"
  Delete "$INSTDIR\doc\html\alignment_8hpp_source.html"
  Delete "$INSTDIR\doc\html\annotated.html"
  Delete "$INSTDIR\doc\html\annotated_dup.js"
  Delete "$INSTDIR\doc\html\apps_2baseline__renderer_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2baseline__renderer_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2baseline__renderer_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2feedthrough_2signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2feedthrough_2signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2feedthrough_2signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2gain__matrix_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2gain__matrix_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2gain__matrix_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2init__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2init__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2init__filter__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2init__filter__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2init__filter__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2matrix__convolver_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2network__audio__streamer_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2network__audio__streamer_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2network__audio__streamer_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2network__audio__streamer_2signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2network__audio__streamer_2signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2network__audio__streamer_2signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2python__runner_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2python__runner_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2python__runner_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2scene__decoder_2signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2scene__decoder_2signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2scene__decoder_2signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\apps_2visr__renderer_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\apps_2visr__renderer_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\apps_2visr__renderer_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\argument__list_8cpp.html"
  Delete "$INSTDIR\doc\html\argument__list_8hpp.html"
  Delete "$INSTDIR\doc\html\argument__list_8hpp_source.html"
  Delete "$INSTDIR\doc\html\array__configuration_8cpp.html"
  Delete "$INSTDIR\doc\html\array__configuration_8hpp.html"
  Delete "$INSTDIR\doc\html\array__configuration_8hpp_source.html"
  Delete "$INSTDIR\doc\html\atomic__component_8hpp.html"
  Delete "$INSTDIR\doc\html\atomic__component_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audiointerfaces_8cpp.html"
  Delete "$INSTDIR\doc\html\audiointerfaces_8cpp.js"
  Delete "$INSTDIR\doc\html\audio__connection_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__connection_8cpp.js"
  Delete "$INSTDIR\doc\html\audio__connection__descriptor_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__connection__descriptor_8hpp.js"
  Delete "$INSTDIR\doc\html\audio__connection__descriptor_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__connection__map_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__connection__map_8cpp.js"
  Delete "$INSTDIR\doc\html\audio__connection__map_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__connection__map_8hpp.js"
  Delete "$INSTDIR\doc\html\audio__connection__map_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__input_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__input_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__input_8hpp.js"
  Delete "$INSTDIR\doc\html\audio__input_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__interface_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__interface_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__interface__configuration_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__interface__configuration_8cpp.js"
  Delete "$INSTDIR\doc\html\audio__interface__factory_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__interface__factory_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__network__encoder_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__network__encoder_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__network__encoder_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__object__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__object__parser_8cpp.js"
  Delete "$INSTDIR\doc\html\audio__output_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__output_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__output_8hpp.js"
  Delete "$INSTDIR\doc\html\audio__output_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__port_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__port_8cpp.js"
  Delete "$INSTDIR\doc\html\audio__port__base_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__port__base_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__port__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__port__base__implementation_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__port__base__implementation_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__port__base__implementation_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__sample__type_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__sample__type_8hpp.js"
  Delete "$INSTDIR\doc\html\audio__sample__type_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\audio__signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\audio__signal__flow__checking_8cpp.html"
  Delete "$INSTDIR\doc\html\audio__signal__flow__checking_8cpp.js"
  Delete "$INSTDIR\doc\html\baseline_renderer_usage.html"
  Delete "$INSTDIR\doc\html\baseline__renderer_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\baseline__renderer_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\baseline__renderer__usage_8dox.html"
  Delete "$INSTDIR\doc\html\basic__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\basic__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\basic__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\basic__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\bc_s.png"
  Delete "$INSTDIR\doc\html\bdwn.png"
  Delete "$INSTDIR\doc\html\biquad__coefficient_8hpp.html"
  Delete "$INSTDIR\doc\html\biquad__coefficient_8hpp_source.html"
  Delete "$INSTDIR\doc\html\biquad__iir__filter_8hpp.html"
  Delete "$INSTDIR\doc\html\biquad__iir__filter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\biquad__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\biquad__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\biquad__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\build_8dox.html"
  Delete "$INSTDIR\doc\html\build_instructions.html"
  Delete "$INSTDIR\doc\html\build_instructions_max_msp_externals.html"
  Delete "$INSTDIR\doc\html\bunch__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\bunch__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\bunch__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\cap__gain__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\cap__gain__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\cartesian__spherical__conversion_8hpp.html"
  Delete "$INSTDIR\doc\html\cartesian__spherical__conversion_8hpp.js"
  Delete "$INSTDIR\doc\html\cartesian__spherical__conversion_8hpp_source.html"
  Delete "$INSTDIR\doc\html\channel__list_8hpp.html"
  Delete "$INSTDIR\doc\html\channel__list_8hpp.js"
  Delete "$INSTDIR\doc\html\channel__list_8hpp_source.html"
  Delete "$INSTDIR\doc\html\channel__object_8hpp.html"
  Delete "$INSTDIR\doc\html\channel__object_8hpp_source.html"
  Delete "$INSTDIR\doc\html\channel__object__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\channel__object__parser_8cpp.js"
  Delete "$INSTDIR\doc\html\channel__object__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\channel__object__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\channel__object__routing__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\channel__object__routing__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\circular__buffer_8cpp.html"
  Delete "$INSTDIR\doc\html\circular__buffer_8hpp.html"
  Delete "$INSTDIR\doc\html\circular__buffer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\classes.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_audio_network_encoder-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_audio_network_encoder.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_audio_network_encoder.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_audio_network_encoder.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_options.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_signal_flow-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_signal_flow.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_signal_flow.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1audio__network__streamer_1_1_signal_flow.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1baseline__renderer_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1baseline__renderer_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1baseline__renderer_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1baseline__renderer_1_1_options.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1feedthrough_1_1_feedthrough-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1feedthrough_1_1_feedthrough.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1feedthrough_1_1_feedthrough.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1feedthrough_1_1_feedthrough.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1gain__matrix_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1gain__matrix_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1gain__matrix_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1matrix__convolver_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1matrix__convolver_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1matrix__convolver_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1matrix__convolver_1_1_options.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1pythonflowrunner_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1pythonflowrunner_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1pythonflowrunner_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1pythonflowrunner_1_1_options.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1scene__decoder_1_1_signal_flow-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1scene__decoder_1_1_signal_flow.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1scene__decoder_1_1_signal_flow.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apps_1_1scene__decoder_1_1_signal_flow.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1apputilities_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apputilities_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1apputilities_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1apputilities_1_1_options.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1test_1_1_feedthrough-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1test_1_1_feedthrough.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1test_1_1_feedthrough.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1test_1_1_feedthrough.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_1_1_configuration-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_1_1_configuration.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_1_1_configuration.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_factory-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_factory.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_factory.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_factory_1_1_registrar-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_factory_1_1_registrar.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_audio_interface_factory_1_1_registrar.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_jack_interface-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_jack_interface.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_jack_interface.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_jack_interface.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_jack_interface_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_jack_interface_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_jack_interface_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_config_1_1_sample_format-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_config_1_1_sample_format.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_config_1_1_sample_format.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_aligned_array-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_aligned_array.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_aligned_array.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_vector-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_vector.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_vector.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_basic_vector.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_lagrange_coefficient_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_lagrange_coefficient_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1efl_1_1_lagrange_coefficient_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1test_1_1_test_atom-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1test_1_1_test_atom.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1test_1_1_test_atom.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1test_1_1_test_atom.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_audio_port_base_implementation-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_audio_port_base_implementation.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_audio_port_base_implementation.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_audio_port_base_implementation.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_component_implementation-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_component_implementation.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_component_implementation.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_component_implementation.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_composite_component_implementation-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_composite_component_implementation.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_composite_component_implementation.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_composite_component_implementation.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_parameter_port_base_implementation-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_parameter_port_base_implementation.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_parameter_port_base_implementation.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_parameter_port_base_implementation.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_port_base_implementation-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_port_base_implementation.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_port_base_implementation.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1impl_1_1_port_base_implementation.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_matrix_convolver-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_matrix_convolver.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_matrix_convolver.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_matrix_convolver.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1matrix__convolver_1_1_options.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_options-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_options.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_options.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_options.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_visr_renderer-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_visr_renderer.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_visr_renderer.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1visr__renderer_1_1_visr_renderer.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_argument_list-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_argument_list.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_argument_list.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_class_registrar-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_class_registrar.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_class_registrar.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_delay_vector-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_delay_vector.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_delay_vector.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_delay_vector.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_external_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_external_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_external_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_external_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_external_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_external_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_external_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_gain_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_gain_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_gain_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_gain_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_pml_initialiser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_pml_initialiser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_pml_initialiser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_signal_flow_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_signal_flow_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1maxmsp_1_1_signal_flow_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1mexsupport_1_1_mex_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mexsupport_1_1_mex_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mexsupport_1_1_mex_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1feedthrough_1_1_signal_flow-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1feedthrough_1_1_signal_flow.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1feedthrough_1_1_signal_flow.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1feedthrough_1_1_signal_flow.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1gain__matrix_1_1_signal_flow-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1gain__matrix_1_1_signal_flow.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1gain__matrix_1_1_signal_flow.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1gain__matrix_1_1_signal_flow.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1late__reverb__filter__calculator_1_1_signal_flow-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1late__reverb__filter__calculator_1_1_signal_flow.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1late__reverb__filter__calculator_1_1_signal_flow.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1mex_1_1late__reverb__filter__calculator_1_1_signal_flow.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_channel_object_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_diffuse_source_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_hoa_source_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_factory-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_factory.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_1_1const__iterator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_1_1const__iterator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_1_1const__iterator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_1_1iterator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_1_1iterator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_1_1iterator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_object_vector_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_plane_wave_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_extent_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_diffuseness_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_1_1_discrete_reflection-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_1_1_discrete_reflection.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_1_1_discrete_reflection.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_1_1_late_reverb-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_1_1_late_reverb.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_1_1_late_reverb.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_parser-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_parser.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_parser.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1objectmodel_1_1_point_source_with_reverb_parser.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_all_r_a_d-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_all_r_a_d.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_all_r_a_d.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_c_a_p-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_c_a_p.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_c_a_p.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_loudspeaker_array-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_loudspeaker_array.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_loudspeaker_array.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_v_b_a_p-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_v_b_a_p.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_v_b_a_p.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_x_y_z-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_x_y_z.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1panning_1_1_x_y_z.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_array_configuration-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_array_configuration.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_array_configuration.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_biquad_parameter_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_biquad_parameter_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_biquad_parameter_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_biquad_parameter_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_input-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_input.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_input.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_input.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_output-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_output.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_output.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_double_buffering_protocol_1_1_output.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_empty_parameter_config-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_empty_parameter_config.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_empty_parameter_config.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_empty_parameter_config.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_filter_routing_list_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_filter_routing_list_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_filter_routing_list_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_filter_routing_list_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_indexed_value_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_indexed_value_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_indexed_value_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_indexed_value_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter_config-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter_config.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter_config.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_interpolation_parameter_config.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_listener_position-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_listener_position.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_listener_position.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_listener_position.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter_config-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter_config.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter_config.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_matrix_parameter_config.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_input_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_message_queue_protocol_1_1_output_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_object_vector-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_object_vector.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_object_vector.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_object_vector.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_scalar_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_scalar_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_scalar_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_scalar_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_input_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_shared_data_protocol_1_1_output_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_signal_routing_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_signal_routing_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_signal_routing_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_signal_routing_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_signal_routing_parameter_1_1_compare_entries-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_signal_routing_parameter_1_1_compare_entries.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_signal_routing_parameter_1_1_compare_entries.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_list_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_list_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_list_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_list_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_sparse_gain_routing_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_string_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_string_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_string_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_string_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter_config-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter_config.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter_config.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_time_frequency_parameter_config.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter_config-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter_config.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter_config.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pml_1_1_vector_parameter_config.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythoncomponents_1_1_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythoncomponents_1_1_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythoncomponents_1_1_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythoncomponents_1_1_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythoncomponents_1_1_wrapper_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythoncomponents_1_1_wrapper_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythoncomponents_1_1_wrapper_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_gil_ensure_guard-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_gil_ensure_guard.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_gil_ensure_guard.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_gil_ensure_guard_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_gil_ensure_guard_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_gil_ensure_guard_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_initialisation_guard-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_initialisation_guard.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_initialisation_guard.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_initialisation_guard_1_1_internal-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_initialisation_guard_1_1_internal.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1pythonsupport_1_1_initialisation_guard_1_1_internal.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_atomic_component_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_atomic_component_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_atomic_component_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_atomic_component_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_communication_protocol_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_communication_protocol_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_communication_protocol_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_communication_protocol_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_composite_component_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_composite_component_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_composite_component_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_composite_component_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_base_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_base_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_base_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_base_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_config_base_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_config_base_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_config_base_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1python_1_1visr_1_1_parameter_config_base_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient_list-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient_list.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient_list.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_biquad_coefficient_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_circular_buffer-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_circular_buffer.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_circular_buffer.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_circular_buffer_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_circular_buffer_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_circular_buffer_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_core_convolver_uniform-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_core_convolver_uniform.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_core_convolver_uniform.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_crossfading_convolver_uniform-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_crossfading_convolver_uniform.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_crossfading_convolver_uniform.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ffts_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ffts_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ffts_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ffts_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ffts_wrapper_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ffts_wrapper_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ffts_wrapper_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fft_wrapper_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fft_wrapper_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fft_wrapper_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fft_wrapper_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fft_wrapper_factory-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fft_wrapper_factory.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_filter_routing_list-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_filter_routing_list.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_filter_routing_list.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_filter_routing_list.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_filter_routing_list_1_1_compare_entries-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_filter_routing_list_1_1_compare_entries.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_filter_routing_list_1_1_compare_entries.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_float_sequence-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_float_sequence.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_float_sequence.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fractional_delay_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fractional_delay_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fractional_delay_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fractional_delay_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fractional_delay_factory-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_fractional_delay_factory.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_f_i_r-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_f_i_r.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_f_i_r.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_gain_fader-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_gain_fader.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_gain_fader.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_gain_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_gain_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_gain_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_index_sequence-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_index_sequence.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_index_sequence.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolating_convolver_uniform-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolating_convolver_uniform.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolating_convolver_uniform.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter_set-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter_set.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter_set.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_interpolation_parameter_set.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ipp_fft_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ipp_fft_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ipp_fft_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ipp_fft_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ipp_fft_wrapper_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ipp_fft_wrapper_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_ipp_fft_wrapper_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_kiss_fft_wrapper-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_kiss_fft_wrapper.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_kiss_fft_wrapper.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_kiss_fft_wrapper.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_kiss_fft_wrapper_1_1_impl-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_kiss_fft_wrapper_1_1_impl.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_kiss_fft_wrapper_1_1_impl.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_lagrange_interpolator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_lagrange_interpolator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_lagrange_interpolator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_lagrange_interpolator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_multichannel_convolver_uniform-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_multichannel_convolver_uniform.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_multichannel_convolver_uniform.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_multichannel_delay_line-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_multichannel_delay_line.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_multichannel_delay_line.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_object_channel_allocator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_object_channel_allocator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_object_channel_allocator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_base_1_1_type_id_translator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_base_1_1_type_id_translator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_base_1_1_type_id_translator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_list-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_list.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_parametric_iir_coefficient_list.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_sparse_gain_routing_list-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_sparse_gain_routing_list.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_sparse_gain_routing_list.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_sparse_gain_routing_list.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_sparse_gain_routing_list_1_1_compare_entries-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_sparse_gain_routing_list_1_1_compare_entries.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rbbl_1_1_sparse_gain_routing_list_1_1_compare_entries.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_add-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_add.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_add.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_add.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_biquad_iir_filter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_biquad_iir_filter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_biquad_iir_filter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_biquad_iir_filter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_channel_object_routing_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_channel_object_routing_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_channel_object_routing_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_channel_object_routing_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_crossfading_fir_filter_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_crossfading_fir_filter_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_crossfading_fir_filter_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_crossfading_fir_filter_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_c_a_p_gain_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_c_a_p_gain_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_c_a_p_gain_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_c_a_p_gain_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_vector-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_vector.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_vector.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_delay_vector.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_diffusion_gain_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_diffusion_gain_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_diffusion_gain_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_diffusion_gain_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_fir_filter_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_fir_filter_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_fir_filter_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_fir_filter_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_vector-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_vector.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_vector.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_gain_vector.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_hoa_all_rad_gain_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_hoa_all_rad_gain_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_hoa_all_rad_gain_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_hoa_all_rad_gain_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_interpolating_fir_filter_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_interpolating_fir_filter_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_interpolating_fir_filter_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_interpolating_fir_filter_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_listener_compensation-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_listener_compensation.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_listener_compensation.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_listener_compensation.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_null_source-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_null_source.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_null_source.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_null_source.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_object_gain_eq_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_object_gain_eq_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_object_gain_eq_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_object_gain_eq_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_panning_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_panning_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_panning_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_panning_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_position_decoder-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_position_decoder.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_position_decoder.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_position_decoder.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scalar_osc_decoder-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scalar_osc_decoder.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scalar_osc_decoder.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scalar_osc_decoder.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_decoder-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_decoder.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_decoder.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_decoder.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_encoder-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_encoder.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_encoder.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_scene_encoder.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_signal_routing-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_signal_routing.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_signal_routing.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_signal_routing.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_sparse_gain_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_sparse_gain_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_sparse_gain_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_sparse_gain_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_inverse_transform-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_inverse_transform.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_inverse_transform.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_inverse_transform.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_transform-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_transform.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_transform.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_time_frequency_transform.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_receiver-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_receiver.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_receiver.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_receiver.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_sender-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_sender.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_sender.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rcl_1_1_udp_sender.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_filter_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_filter_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_filter_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_filter_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_parameter-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_parameter.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_parameter.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_late_reverb_parameter.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_object_renderer-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_object_renderer.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_object_renderer.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_object_renderer.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_parameter_calculator-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_parameter_calculator.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_parameter_calculator.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1reverbobject_1_1_reverb_parameter_calculator.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_channel-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_channel.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_channel.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_connection_map-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_connection_map.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_connection_map.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_signal_flow-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_signal_flow.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_signal_flow.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_signal_flow.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_signal_pool-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_signal_pool.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_audio_signal_pool.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_communication_area-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_communication_area.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_communication_area.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_graph-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_graph.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_graph.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_graph_1_1_connected_ports-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_graph_1_1_connected_ports.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_graph_1_1_connected_ports.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_map.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_parameter_connection_map.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_port_lookup-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_port_lookup.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_port_lookup.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_scheduling_graph-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_scheduling_graph.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_scheduling_graph.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_signal_routing_internal-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_signal_routing_internal.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_signal_routing_internal.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1rrl_1_1_signal_routing_internal.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflowspython_1_1_visr_renderer-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflowspython_1_1_visr_renderer.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflowspython_1_1_visr_renderer.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflowspython_1_1_visr_renderer.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_baseline_renderer-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_baseline_renderer.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_baseline_renderer.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_baseline_renderer.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_bunch_renderer-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_bunch_renderer.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_bunch_renderer.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_bunch_renderer.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_core_renderer-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_core_renderer.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_core_renderer.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_core_renderer.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_delay_vector-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_delay_vector.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_delay_vector.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_delay_vector.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_gain_matrix-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_gain_matrix.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_gain_matrix.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_gain_matrix.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_time_frequency_feedthrough-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_time_frequency_feedthrough.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_time_frequency_feedthrough.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1signalflows_1_1_time_frequency_feedthrough.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_atomic_component-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_atomic_component.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_atomic_component.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_atomic_component.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_t-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_t.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_t.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_input_t.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_t-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_t.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_t.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_output_t.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_port_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_port_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_port_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_audio_port_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_channel_list-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_channel_list.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_channel_list.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_channel_range-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_channel_range.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_channel_range.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_input-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_input.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_input.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_input.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_output-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_output.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_output.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_base_1_1_output.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_factory-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_factory.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_communication_protocol_factory.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_component-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_component.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_component.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_component.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_composite_component-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_composite_component.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_composite_component.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_composite_component.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_config_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_config_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_config_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_config_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_factory-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_factory.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_factory.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_factory_1_1_registrar-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_factory_1_1_registrar.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_factory_1_1_registrar.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_input_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_output_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_port_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_port_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_port_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_parameter_port_base.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_input-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_input.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_input.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_input.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_output-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_output.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_output.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_polymorphic_parameter_output.png"
  Delete "$INSTDIR\doc\html\classvisr_1_1_port_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_port_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_port_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_signal_flow_context-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_signal_flow_context.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_signal_flow_context.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_status_message-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_status_message.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_status_message.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_typed_parameter_base-members.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_typed_parameter_base.html"
  Delete "$INSTDIR\doc\html\classvisr_1_1_typed_parameter_base.js"
  Delete "$INSTDIR\doc\html\classvisr_1_1_typed_parameter_base.png"
  Delete "$INSTDIR\doc\html\class__registrar_8hpp.html"
  Delete "$INSTDIR\doc\html\class__registrar_8hpp_source.html"
  Delete "$INSTDIR\doc\html\closed.png"
  Delete "$INSTDIR\doc\html\communication__area_8cpp.html"
  Delete "$INSTDIR\doc\html\communication__area_8hpp.html"
  Delete "$INSTDIR\doc\html\communication__area_8hpp_source.html"
  Delete "$INSTDIR\doc\html\communication__protocol_8cpp.html"
  Delete "$INSTDIR\doc\html\communication__protocol_8cpp.js"
  Delete "$INSTDIR\doc\html\communication__protocol__base_8cpp.html"
  Delete "$INSTDIR\doc\html\communication__protocol__base_8hpp.html"
  Delete "$INSTDIR\doc\html\communication__protocol__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\communication__protocol__factory_8hpp.html"
  Delete "$INSTDIR\doc\html\communication__protocol__factory_8hpp_source.html"
  Delete "$INSTDIR\doc\html\communication__protocol__type_8hpp.html"
  Delete "$INSTDIR\doc\html\communication__protocol__type_8hpp.js"
  Delete "$INSTDIR\doc\html\communication__protocol__type_8hpp_source.html"
  Delete "$INSTDIR\doc\html\compile__time__hash__fnv1_8hpp.html"
  Delete "$INSTDIR\doc\html\compile__time__hash__fnv1_8hpp.js"
  Delete "$INSTDIR\doc\html\compile__time__hash__fnv1_8hpp_source.html"
  Delete "$INSTDIR\doc\html\complex__multiply_8cpp.html"
  Delete "$INSTDIR\doc\html\complex__multiply_8cpp.js"
  Delete "$INSTDIR\doc\html\component_8hpp.html"
  Delete "$INSTDIR\doc\html\component_8hpp_source.html"
  Delete "$INSTDIR\doc\html\component__implementation_8cpp.html"
  Delete "$INSTDIR\doc\html\component__implementation_8cpp.js"
  Delete "$INSTDIR\doc\html\component__implementation_8hpp.html"
  Delete "$INSTDIR\doc\html\component__implementation_8hpp_source.html"
  Delete "$INSTDIR\doc\html\component__names_8cpp.html"
  Delete "$INSTDIR\doc\html\component__names_8cpp.js"
  Delete "$INSTDIR\doc\html\compose__message__string_8hpp.html"
  Delete "$INSTDIR\doc\html\compose__message__string_8hpp_source.html"
  Delete "$INSTDIR\doc\html\composite__component_8hpp.html"
  Delete "$INSTDIR\doc\html\composite__component_8hpp_source.html"
  Delete "$INSTDIR\doc\html\composite__component__implementation_8cpp.html"
  Delete "$INSTDIR\doc\html\composite__component__implementation_8hpp.html"
  Delete "$INSTDIR\doc\html\composite__component__implementation_8hpp_source.html"
  Delete "$INSTDIR\doc\html\constants_8hpp.html"
  Delete "$INSTDIR\doc\html\constants_8hpp.js"
  Delete "$INSTDIR\doc\html\constants_8hpp_source.html"
  Delete "$INSTDIR\doc\html\core__convolver__uniform_8hpp.html"
  Delete "$INSTDIR\doc\html\core__convolver__uniform_8hpp_source.html"
  Delete "$INSTDIR\doc\html\crossfading__convolver__uniform_8hpp.html"
  Delete "$INSTDIR\doc\html\crossfading__convolver__uniform_8hpp_source.html"
  Delete "$INSTDIR\doc\html\crossfading__fir__filter__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\crossfading__fir__filter__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\crossfading__fir__filter__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\db__linear__conversion_8hpp.html"
  Delete "$INSTDIR\doc\html\db__linear__conversion_8hpp.js"
  Delete "$INSTDIR\doc\html\db__linear__conversion_8hpp_source.html"
  Delete "$INSTDIR\doc\html\decode___n8___p40__t-design__t8___p40_8txt_8cpp.html"
  Delete "$INSTDIR\doc\html\defs_8h.html"
  Delete "$INSTDIR\doc\html\defs_8h.js"
  Delete "$INSTDIR\doc\html\defs_8h_source.html"
  Delete "$INSTDIR\doc\html\degree__radian__conversion_8hpp.html"
  Delete "$INSTDIR\doc\html\degree__radian__conversion_8hpp.js"
  Delete "$INSTDIR\doc\html\degree__radian__conversion_8hpp_source.html"
  Delete "$INSTDIR\doc\html\delay__line_8cpp.html"
  Delete "$INSTDIR\doc\html\delay__line_8cpp.js"
  Delete "$INSTDIR\doc\html\denormalised__number__handling_8cpp.html"
  Delete "$INSTDIR\doc\html\denormalised__number__handling_8hpp.html"
  Delete "$INSTDIR\doc\html\denormalised__number__handling_8hpp_source.html"
  Delete "$INSTDIR\doc\html\deprecated.html"
  Delete "$INSTDIR\doc\html\diffuse__source_8hpp.html"
  Delete "$INSTDIR\doc\html\diffuse__source_8hpp_source.html"
  Delete "$INSTDIR\doc\html\diffuse__source__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\diffuse__source__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\diffuse__source__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\diffusion__gain__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\diffusion__gain__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\dir_014c6c332b259aea005167b130f4b4a9.html"
  Delete "$INSTDIR\doc\html\dir_014c6c332b259aea005167b130f4b4a9.js"
  Delete "$INSTDIR\doc\html\dir_02ecb9b96232f0e6802f6b197f581ed9.html"
  Delete "$INSTDIR\doc\html\dir_02ecb9b96232f0e6802f6b197f581ed9.js"
  Delete "$INSTDIR\doc\html\dir_0645e67c857f3ed5fe452946d333ec84.html"
  Delete "$INSTDIR\doc\html\dir_0645e67c857f3ed5fe452946d333ec84.js"
  Delete "$INSTDIR\doc\html\dir_0b263b942c0be7e5862088d0b3304562.html"
  Delete "$INSTDIR\doc\html\dir_0b263b942c0be7e5862088d0b3304562.js"
  Delete "$INSTDIR\doc\html\dir_0d54c8da732fd7ef3ebc9bd8970f5f73.html"
  Delete "$INSTDIR\doc\html\dir_0d54c8da732fd7ef3ebc9bd8970f5f73.js"
  Delete "$INSTDIR\doc\html\dir_0f00094951dfe5f0dac235ea92ea9e02.html"
  Delete "$INSTDIR\doc\html\dir_0f00094951dfe5f0dac235ea92ea9e02.js"
  Delete "$INSTDIR\doc\html\dir_0f4e3eebf44c879e7b7bbd92038c452e.html"
  Delete "$INSTDIR\doc\html\dir_0f4e3eebf44c879e7b7bbd92038c452e.js"
  Delete "$INSTDIR\doc\html\dir_1040b460d8b674aa121d87fe413d7826.html"
  Delete "$INSTDIR\doc\html\dir_1040b460d8b674aa121d87fe413d7826.js"
  Delete "$INSTDIR\doc\html\dir_11c82a1c6d49869166f7ddedda96e670.html"
  Delete "$INSTDIR\doc\html\dir_11c82a1c6d49869166f7ddedda96e670.js"
  Delete "$INSTDIR\doc\html\dir_13e2d675882fbd9c5da8ed54f0a9f3ec.html"
  Delete "$INSTDIR\doc\html\dir_13e2d675882fbd9c5da8ed54f0a9f3ec.js"
  Delete "$INSTDIR\doc\html\dir_16df2321f6a74705e2f8557f30970449.html"
  Delete "$INSTDIR\doc\html\dir_16df2321f6a74705e2f8557f30970449.js"
  Delete "$INSTDIR\doc\html\dir_189e8538bc549a9aeb0e4f2c948817e9.html"
  Delete "$INSTDIR\doc\html\dir_189e8538bc549a9aeb0e4f2c948817e9.js"
  Delete "$INSTDIR\doc\html\dir_1d1c3bb45d911e5755cc628c0f9a04d1.html"
  Delete "$INSTDIR\doc\html\dir_1d1c3bb45d911e5755cc628c0f9a04d1.js"
  Delete "$INSTDIR\doc\html\dir_203e0c315d1beb6785357a49fa3f979d.html"
  Delete "$INSTDIR\doc\html\dir_203e0c315d1beb6785357a49fa3f979d.js"
  Delete "$INSTDIR\doc\html\dir_260fe735f3cb79f89ea46b4e9fca3d19.html"
  Delete "$INSTDIR\doc\html\dir_260fe735f3cb79f89ea46b4e9fca3d19.js"
  Delete "$INSTDIR\doc\html\dir_2b4d203186da8b3994156779da527cab.html"
  Delete "$INSTDIR\doc\html\dir_2b4d203186da8b3994156779da527cab.js"
  Delete "$INSTDIR\doc\html\dir_2c465f66ec8d3aa6317d374a4bd24cdb.html"
  Delete "$INSTDIR\doc\html\dir_2c465f66ec8d3aa6317d374a4bd24cdb.js"
  Delete "$INSTDIR\doc\html\dir_2e00633f6c8c861df92e83ef05b46ad6.html"
  Delete "$INSTDIR\doc\html\dir_2e00633f6c8c861df92e83ef05b46ad6.js"
  Delete "$INSTDIR\doc\html\dir_39135db674f1851f170545dbae5cf6fe.html"
  Delete "$INSTDIR\doc\html\dir_39135db674f1851f170545dbae5cf6fe.js"
  Delete "$INSTDIR\doc\html\dir_395a501f50135c7b2f9ffa49479fda50.html"
  Delete "$INSTDIR\doc\html\dir_395a501f50135c7b2f9ffa49479fda50.js"
  Delete "$INSTDIR\doc\html\dir_3be19a81f182e7332ca41dfde2c67825.html"
  Delete "$INSTDIR\doc\html\dir_3be19a81f182e7332ca41dfde2c67825.js"
  Delete "$INSTDIR\doc\html\dir_3d62a41c1001155905fae299516ac865.html"
  Delete "$INSTDIR\doc\html\dir_3d62a41c1001155905fae299516ac865.js"
  Delete "$INSTDIR\doc\html\dir_41fe5a68be84f836d7091e1273488cbe.html"
  Delete "$INSTDIR\doc\html\dir_41fe5a68be84f836d7091e1273488cbe.js"
  Delete "$INSTDIR\doc\html\dir_4495670b9be8bc659cd773f191b4e213.html"
  Delete "$INSTDIR\doc\html\dir_4495670b9be8bc659cd773f191b4e213.js"
  Delete "$INSTDIR\doc\html\dir_47fd8888ef5b1409ed82e01ef4c0bb3d.html"
  Delete "$INSTDIR\doc\html\dir_47fd8888ef5b1409ed82e01ef4c0bb3d.js"
  Delete "$INSTDIR\doc\html\dir_493a5415a20e2e5f2c6e45c090b9b342.html"
  Delete "$INSTDIR\doc\html\dir_493a5415a20e2e5f2c6e45c090b9b342.js"
  Delete "$INSTDIR\doc\html\dir_49415fe834e80fa493f89c8e73a06250.html"
  Delete "$INSTDIR\doc\html\dir_49415fe834e80fa493f89c8e73a06250.js"
  Delete "$INSTDIR\doc\html\dir_4bede4f65f26296b0614ac618d002cd0.html"
  Delete "$INSTDIR\doc\html\dir_4bede4f65f26296b0614ac618d002cd0.js"
  Delete "$INSTDIR\doc\html\dir_4f859de48b9bbd587a4999ae58b80ad7.html"
  Delete "$INSTDIR\doc\html\dir_4f859de48b9bbd587a4999ae58b80ad7.js"
  Delete "$INSTDIR\doc\html\dir_4fce6a84a1f36302d1b3a007a424b188.html"
  Delete "$INSTDIR\doc\html\dir_4fce6a84a1f36302d1b3a007a424b188.js"
  Delete "$INSTDIR\doc\html\dir_5103cbbdad57e3a7b8feeefa49fd7b0e.html"
  Delete "$INSTDIR\doc\html\dir_5103cbbdad57e3a7b8feeefa49fd7b0e.js"
  Delete "$INSTDIR\doc\html\dir_53adf0b982dc8545998aae3f283a5a58.html"
  Delete "$INSTDIR\doc\html\dir_53adf0b982dc8545998aae3f283a5a58.js"
  Delete "$INSTDIR\doc\html\dir_556b2188208462a0df1d460b6e1fb55d.html"
  Delete "$INSTDIR\doc\html\dir_556b2188208462a0df1d460b6e1fb55d.js"
  Delete "$INSTDIR\doc\html\dir_5c0d64f70903e893b1efe571a4b8de29.html"
  Delete "$INSTDIR\doc\html\dir_5c0d64f70903e893b1efe571a4b8de29.js"
  Delete "$INSTDIR\doc\html\dir_68267d1309a1af8e8297ef4c3efbcdba.html"
  Delete "$INSTDIR\doc\html\dir_68267d1309a1af8e8297ef4c3efbcdba.js"
  Delete "$INSTDIR\doc\html\dir_68fc8eca6a76365009b89f36d4deca27.html"
  Delete "$INSTDIR\doc\html\dir_68fc8eca6a76365009b89f36d4deca27.js"
  Delete "$INSTDIR\doc\html\dir_6ab670b730245e0968deb2276ab39377.html"
  Delete "$INSTDIR\doc\html\dir_6ab670b730245e0968deb2276ab39377.js"
  Delete "$INSTDIR\doc\html\dir_6c57063d49b019795782b93903c47c0f.html"
  Delete "$INSTDIR\doc\html\dir_6c57063d49b019795782b93903c47c0f.js"
  Delete "$INSTDIR\doc\html\dir_729a4516a537cf6e64e41697e523b001.html"
  Delete "$INSTDIR\doc\html\dir_729a4516a537cf6e64e41697e523b001.js"
  Delete "$INSTDIR\doc\html\dir_740094eb819e8a2c8c4709fffd68c5ee.html"
  Delete "$INSTDIR\doc\html\dir_740094eb819e8a2c8c4709fffd68c5ee.js"
  Delete "$INSTDIR\doc\html\dir_77e0c9fe4173c72b875ae9562407de2c.html"
  Delete "$INSTDIR\doc\html\dir_77e0c9fe4173c72b875ae9562407de2c.js"
  Delete "$INSTDIR\doc\html\dir_7c408682270efa343e1328cf24c8bddb.html"
  Delete "$INSTDIR\doc\html\dir_7c408682270efa343e1328cf24c8bddb.js"
  Delete "$INSTDIR\doc\html\dir_86a697919b01f146390e616c449e50ab.html"
  Delete "$INSTDIR\doc\html\dir_86a697919b01f146390e616c449e50ab.js"
  Delete "$INSTDIR\doc\html\dir_92c2de66b023d858b4a99ee08753a706.html"
  Delete "$INSTDIR\doc\html\dir_92c2de66b023d858b4a99ee08753a706.js"
  Delete "$INSTDIR\doc\html\dir_9894b53404392327e63a436630cc83b5.html"
  Delete "$INSTDIR\doc\html\dir_9894b53404392327e63a436630cc83b5.js"
  Delete "$INSTDIR\doc\html\dir_9ee58dd100ef9e6e22b6913e7160f612.html"
  Delete "$INSTDIR\doc\html\dir_9ee58dd100ef9e6e22b6913e7160f612.js"
  Delete "$INSTDIR\doc\html\dir_a004b8f2a10531e25389c8daac04d9d7.html"
  Delete "$INSTDIR\doc\html\dir_a004b8f2a10531e25389c8daac04d9d7.js"
  Delete "$INSTDIR\doc\html\dir_a693e0399f729a4c656e47adac53de21.html"
  Delete "$INSTDIR\doc\html\dir_a693e0399f729a4c656e47adac53de21.js"
  Delete "$INSTDIR\doc\html\dir_a8056c6c1f7c0ece9dad68b7dd289f70.html"
  Delete "$INSTDIR\doc\html\dir_a8056c6c1f7c0ece9dad68b7dd289f70.js"
  Delete "$INSTDIR\doc\html\dir_a8e04a0aec16c24c77026d19e386080d.html"
  Delete "$INSTDIR\doc\html\dir_a8e04a0aec16c24c77026d19e386080d.js"
  Delete "$INSTDIR\doc\html\dir_aaafa20671a50e39df67208137461614.html"
  Delete "$INSTDIR\doc\html\dir_aaafa20671a50e39df67208137461614.js"
  Delete "$INSTDIR\doc\html\dir_ab446da4b10677d231516a215c2593b0.html"
  Delete "$INSTDIR\doc\html\dir_ab446da4b10677d231516a215c2593b0.js"
  Delete "$INSTDIR\doc\html\dir_ac01287d47bed959ac3eb860f37c424a.html"
  Delete "$INSTDIR\doc\html\dir_ac01287d47bed959ac3eb860f37c424a.js"
  Delete "$INSTDIR\doc\html\dir_ac7e44afd0a44393c47248e9b958c834.html"
  Delete "$INSTDIR\doc\html\dir_ac7e44afd0a44393c47248e9b958c834.js"
  Delete "$INSTDIR\doc\html\dir_ace829b89b39dc7471dba565463fab26.html"
  Delete "$INSTDIR\doc\html\dir_ace829b89b39dc7471dba565463fab26.js"
  Delete "$INSTDIR\doc\html\dir_ae9967c8309e42d02eaed7906b4d761a.html"
  Delete "$INSTDIR\doc\html\dir_ae9967c8309e42d02eaed7906b4d761a.js"
  Delete "$INSTDIR\doc\html\dir_c04d921e285ee1e7047dc07228956a5d.html"
  Delete "$INSTDIR\doc\html\dir_c04d921e285ee1e7047dc07228956a5d.js"
  Delete "$INSTDIR\doc\html\dir_c6a8403601c24848571ea2a73ccd640b.html"
  Delete "$INSTDIR\doc\html\dir_c6a8403601c24848571ea2a73ccd640b.js"
  Delete "$INSTDIR\doc\html\dir_d3f050b33e8ab44f7f8481b8e486c613.html"
  Delete "$INSTDIR\doc\html\dir_d3f050b33e8ab44f7f8481b8e486c613.js"
  Delete "$INSTDIR\doc\html\dir_dcb615ec72ec05db5d42ec52739b625b.html"
  Delete "$INSTDIR\doc\html\dir_dcb615ec72ec05db5d42ec52739b625b.js"
  Delete "$INSTDIR\doc\html\dir_de74e5d28a99eeb7db40a22eaf2b6e84.html"
  Delete "$INSTDIR\doc\html\dir_de74e5d28a99eeb7db40a22eaf2b6e84.js"
  Delete "$INSTDIR\doc\html\dir_dee10c86eae1af048e484ab6918c0951.html"
  Delete "$INSTDIR\doc\html\dir_dee10c86eae1af048e484ab6918c0951.js"
  Delete "$INSTDIR\doc\html\dir_e02dec0bef18f00458efcf941bdcd802.html"
  Delete "$INSTDIR\doc\html\dir_e02dec0bef18f00458efcf941bdcd802.js"
  Delete "$INSTDIR\doc\html\dir_e0f72a4a7b4df723ad02ea9cf6df5894.html"
  Delete "$INSTDIR\doc\html\dir_e0f72a4a7b4df723ad02ea9cf6df5894.js"
  Delete "$INSTDIR\doc\html\dir_e3092f42fdaf7ee5568d923f7bdfbfe5.html"
  Delete "$INSTDIR\doc\html\dir_e3092f42fdaf7ee5568d923f7bdfbfe5.js"
  Delete "$INSTDIR\doc\html\dir_f044611984161f2608b01c838efbf751.html"
  Delete "$INSTDIR\doc\html\dir_f044611984161f2608b01c838efbf751.js"
  Delete "$INSTDIR\doc\html\dir_f311637798a211c3495f8027b948e033.html"
  Delete "$INSTDIR\doc\html\dir_f311637798a211c3495f8027b948e033.js"
  Delete "$INSTDIR\doc\html\dir_f742e70b55103b98462bb00a3b94819f.html"
  Delete "$INSTDIR\doc\html\dir_f742e70b55103b98462bb00a3b94819f.js"
  Delete "$INSTDIR\doc\html\dir_fdc2158e5d491370a0e22ceeff6a2471.html"
  Delete "$INSTDIR\doc\html\dir_fdc2158e5d491370a0e22ceeff6a2471.js"
  Delete "$INSTDIR\doc\html\doc.png"
  Delete "$INSTDIR\doc\html\double__buffering__protocol_8hpp.html"
  Delete "$INSTDIR\doc\html\double__buffering__protocol_8hpp_source.html"
  Delete "$INSTDIR\doc\html\doxygen.css"
  Delete "$INSTDIR\doc\html\doxygen.png"
  Delete "$INSTDIR\doc\html\dynsections.js"
  Delete "$INSTDIR\doc\html\efl_8cpp.html"
  Delete "$INSTDIR\doc\html\efl_8cpp.js"
  Delete "$INSTDIR\doc\html\empty__parameter__config_8hpp.html"
  Delete "$INSTDIR\doc\html\empty__parameter__config_8hpp_source.html"
  Delete "$INSTDIR\doc\html\error__codes_8cpp.html"
  Delete "$INSTDIR\doc\html\error__codes_8cpp.js"
  Delete "$INSTDIR\doc\html\error__codes_8hpp.html"
  Delete "$INSTDIR\doc\html\error__codes_8hpp.js"
  Delete "$INSTDIR\doc\html\error__codes_8hpp_source.html"
  Delete "$INSTDIR\doc\html\export__symbol_8hpp.html"
  Delete "$INSTDIR\doc\html\export__symbol_8hpp.js"
  Delete "$INSTDIR\doc\html\export__symbol_8hpp_source.html"
  Delete "$INSTDIR\doc\html\external__base_8cpp.html"
  Delete "$INSTDIR\doc\html\external__base_8hpp.html"
  Delete "$INSTDIR\doc\html\external__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\external__wrapper_8hpp.html"
  Delete "$INSTDIR\doc\html\external__wrapper_8hpp_source.html"
  Delete "$INSTDIR\doc\html\feedthrough_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\feedthrough_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\feedthrough_8cpp.html"
  Delete "$INSTDIR\doc\html\feedthrough_8cpp.js"
  Delete "$INSTDIR\doc\html\ffts__wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\ffts__wrapper_8hpp.html"
  Delete "$INSTDIR\doc\html\ffts__wrapper_8hpp_source.html"
  Delete "$INSTDIR\doc\html\fft__wrapper__base_8hpp.html"
  Delete "$INSTDIR\doc\html\fft__wrapper__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\fft__wrapper__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\fft__wrapper__factory_8cpp.js"
  Delete "$INSTDIR\doc\html\fft__wrapper__factory_8hpp.html"
  Delete "$INSTDIR\doc\html\fft__wrapper__factory_8hpp_source.html"
  Delete "$INSTDIR\doc\html\files.html"
  Delete "$INSTDIR\doc\html\files_dup.js"
  Delete "$INSTDIR\doc\html\filter__routing_8hpp.html"
  Delete "$INSTDIR\doc\html\filter__routing_8hpp_source.html"
  Delete "$INSTDIR\doc\html\filter__routing__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\filter__routing__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\fir_8cpp.html"
  Delete "$INSTDIR\doc\html\fir_8hpp.html"
  Delete "$INSTDIR\doc\html\fir_8hpp.js"
  Delete "$INSTDIR\doc\html\fir_8hpp_source.html"
  Delete "$INSTDIR\doc\html\fir__filter__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\fir__filter__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\fir__filter__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\float__sequence_8cpp.html"
  Delete "$INSTDIR\doc\html\float__sequence_8hpp.html"
  Delete "$INSTDIR\doc\html\float__sequence_8hpp_source.html"
  Delete "$INSTDIR\doc\html\folderclosed.png"
  Delete "$INSTDIR\doc\html\folderopen.png"
  Delete "$INSTDIR\doc\html\fractional__delay__base_8cpp.html"
  Delete "$INSTDIR\doc\html\fractional__delay__base_8hpp.html"
  Delete "$INSTDIR\doc\html\fractional__delay__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\fractional__delay__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\fractional__delay__factory_8hpp.html"
  Delete "$INSTDIR\doc\html\fractional__delay__factory_8hpp_source.html"
  Delete "$INSTDIR\doc\html\fractional__delay__interpolator__base_8hpp.html"
  Delete "$INSTDIR\doc\html\fractional__delay__interpolator__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\functions.html"
  Delete "$INSTDIR\doc\html\functions_0x7e.html"
  Delete "$INSTDIR\doc\html\functions_b.html"
  Delete "$INSTDIR\doc\html\functions_c.html"
  Delete "$INSTDIR\doc\html\functions_d.html"
  Delete "$INSTDIR\doc\html\functions_dup.js"
  Delete "$INSTDIR\doc\html\functions_e.html"
  Delete "$INSTDIR\doc\html\functions_enum.html"
  Delete "$INSTDIR\doc\html\functions_eval.html"
  Delete "$INSTDIR\doc\html\functions_f.html"
  Delete "$INSTDIR\doc\html\functions_func.html"
  Delete "$INSTDIR\doc\html\functions_func.js"
  Delete "$INSTDIR\doc\html\functions_func_0x7e.html"
  Delete "$INSTDIR\doc\html\functions_func_b.html"
  Delete "$INSTDIR\doc\html\functions_func_c.html"
  Delete "$INSTDIR\doc\html\functions_func_d.html"
  Delete "$INSTDIR\doc\html\functions_func_e.html"
  Delete "$INSTDIR\doc\html\functions_func_f.html"
  Delete "$INSTDIR\doc\html\functions_func_g.html"
  Delete "$INSTDIR\doc\html\functions_func_h.html"
  Delete "$INSTDIR\doc\html\functions_func_i.html"
  Delete "$INSTDIR\doc\html\functions_func_j.html"
  Delete "$INSTDIR\doc\html\functions_func_k.html"
  Delete "$INSTDIR\doc\html\functions_func_l.html"
  Delete "$INSTDIR\doc\html\functions_func_m.html"
  Delete "$INSTDIR\doc\html\functions_func_n.html"
  Delete "$INSTDIR\doc\html\functions_func_o.html"
  Delete "$INSTDIR\doc\html\functions_func_p.html"
  Delete "$INSTDIR\doc\html\functions_func_q.html"
  Delete "$INSTDIR\doc\html\functions_func_r.html"
  Delete "$INSTDIR\doc\html\functions_func_s.html"
  Delete "$INSTDIR\doc\html\functions_func_t.html"
  Delete "$INSTDIR\doc\html\functions_func_u.html"
  Delete "$INSTDIR\doc\html\functions_func_v.html"
  Delete "$INSTDIR\doc\html\functions_func_w.html"
  Delete "$INSTDIR\doc\html\functions_func_x.html"
  Delete "$INSTDIR\doc\html\functions_func_y.html"
  Delete "$INSTDIR\doc\html\functions_func_z.html"
  Delete "$INSTDIR\doc\html\functions_g.html"
  Delete "$INSTDIR\doc\html\functions_h.html"
  Delete "$INSTDIR\doc\html\functions_i.html"
  Delete "$INSTDIR\doc\html\functions_j.html"
  Delete "$INSTDIR\doc\html\functions_k.html"
  Delete "$INSTDIR\doc\html\functions_l.html"
  Delete "$INSTDIR\doc\html\functions_m.html"
  Delete "$INSTDIR\doc\html\functions_n.html"
  Delete "$INSTDIR\doc\html\functions_o.html"
  Delete "$INSTDIR\doc\html\functions_p.html"
  Delete "$INSTDIR\doc\html\functions_q.html"
  Delete "$INSTDIR\doc\html\functions_r.html"
  Delete "$INSTDIR\doc\html\functions_rela.html"
  Delete "$INSTDIR\doc\html\functions_s.html"
  Delete "$INSTDIR\doc\html\functions_t.html"
  Delete "$INSTDIR\doc\html\functions_type.html"
  Delete "$INSTDIR\doc\html\functions_u.html"
  Delete "$INSTDIR\doc\html\functions_v.html"
  Delete "$INSTDIR\doc\html\functions_vars.html"
  Delete "$INSTDIR\doc\html\functions_w.html"
  Delete "$INSTDIR\doc\html\functions_x.html"
  Delete "$INSTDIR\doc\html\functions_y.html"
  Delete "$INSTDIR\doc\html\functions_z.html"
  Delete "$INSTDIR\doc\html\gain__fader_8cpp.html"
  Delete "$INSTDIR\doc\html\gain__fader_8hpp.html"
  Delete "$INSTDIR\doc\html\gain__fader_8hpp_source.html"
  Delete "$INSTDIR\doc\html\gain__matrix_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\gain__matrix_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\gain__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\gain__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\gil__ensure__guard_8cpp.html"
  Delete "$INSTDIR\doc\html\gil__ensure__guard_8hpp.html"
  Delete "$INSTDIR\doc\html\gil__ensure__guard_8hpp_source.html"
  Delete "$INSTDIR\doc\html\globals.html"
  Delete "$INSTDIR\doc\html\globals_defs.html"
  Delete "$INSTDIR\doc\html\globals_func.html"
  Delete "$INSTDIR\doc\html\globals_type.html"
  Delete "$INSTDIR\doc\html\hierarchy.html"
  Delete "$INSTDIR\doc\html\hierarchy.js"
  Delete "$INSTDIR\doc\html\hoa__allrad__gain__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\hoa__allrad__gain__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\hoa__source_8hpp.html"
  Delete "$INSTDIR\doc\html\hoa__source_8hpp_source.html"
  Delete "$INSTDIR\doc\html\hoa__source__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\hoa__source__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\hoa__source__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\impl_2audio__connection__descriptor_8cpp.html"
  Delete "$INSTDIR\doc\html\impl_2audio__connection__descriptor_8cpp.js"
  Delete "$INSTDIR\doc\html\index.html"
  Delete "$INSTDIR\doc\html\indexed__value__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\indexed__value__parameter_8hpp.js"
  Delete "$INSTDIR\doc\html\indexed__value__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\index__sequence_8cpp.html"
  Delete "$INSTDIR\doc\html\index__sequence_8hpp.html"
  Delete "$INSTDIR\doc\html\index__sequence_8hpp_source.html"
  Delete "$INSTDIR\doc\html\initialisation__guard_8cpp.html"
  Delete "$INSTDIR\doc\html\initialisation__guard_8hpp.html"
  Delete "$INSTDIR\doc\html\initialisation__guard_8hpp_source.html"
  Delete "$INSTDIR\doc\html\initialise__parameter__library_8cpp.html"
  Delete "$INSTDIR\doc\html\initialise__parameter__library_8cpp.js"
  Delete "$INSTDIR\doc\html\initialise__parameter__library_8hpp.html"
  Delete "$INSTDIR\doc\html\initialise__parameter__library_8hpp.js"
  Delete "$INSTDIR\doc\html\initialise__parameter__library_8hpp_source.html"
  Delete "$INSTDIR\doc\html\installation_8dox.html"
  Delete "$INSTDIR\doc\html\installation_instructions.html"
  Delete "$INSTDIR\doc\html\instantiation_8cpp.html"
  Delete "$INSTDIR\doc\html\instantiation_8cpp.js"
  Delete "$INSTDIR\doc\html\integrity__checking_8hpp.html"
  Delete "$INSTDIR\doc\html\integrity__checking_8hpp.js"
  Delete "$INSTDIR\doc\html\integrity__checking_8hpp_source.html"
  Delete "$INSTDIR\doc\html\interpolating__convolver_8cpp.html"
  Delete "$INSTDIR\doc\html\interpolating__convolver_8cpp.js"
  Delete "$INSTDIR\doc\html\interpolating__convolver__uniform_8hpp.html"
  Delete "$INSTDIR\doc\html\interpolating__convolver__uniform_8hpp_source.html"
  Delete "$INSTDIR\doc\html\interpolating__fir__filter__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\interpolating__fir__filter__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\interpolating__fir__filter__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\ipp__fft__wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\ipp__fft__wrapper_8hpp.html"
  Delete "$INSTDIR\doc\html\ipp__fft__wrapper_8hpp_source.html"
  Delete "$INSTDIR\doc\html\jackconf_8dox.html"
  Delete "$INSTDIR\doc\html\jackexample.jpeg"
  Delete "$INSTDIR\doc\html\jacksimpleexample.jpeg"
  Delete "$INSTDIR\doc\html\jack_audio_interface_configuration.html"
  Delete "$INSTDIR\doc\html\jack__interface_8hpp.html"
  Delete "$INSTDIR\doc\html\jack__interface_8hpp_source.html"
  Delete "$INSTDIR\doc\html\jquery.js"
  Delete "$INSTDIR\doc\html\kiss__fft__wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\kiss__fft__wrapper_8cpp.js"
  Delete "$INSTDIR\doc\html\kiss__fft__wrapper_8hpp.html"
  Delete "$INSTDIR\doc\html\kiss__fft__wrapper_8hpp_source.html"
  Delete "$INSTDIR\doc\html\kiss__fft__wrapper__double_8cpp.html"
  Delete "$INSTDIR\doc\html\kiss__fft__wrapper__float_8cpp.html"
  Delete "$INSTDIR\doc\html\lagrange__coefficient__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\lagrange__coefficient__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\lagrange__coefficient__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\lagrange__interpolator_8hpp.html"
  Delete "$INSTDIR\doc\html\lagrange__interpolator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\late__reverb__filter__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\late__reverb__filter__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\late__reverb__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\late__reverb__parameter_8hpp.js"
  Delete "$INSTDIR\doc\html\late__reverb__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libapputilities_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libapputilities_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libapputilities_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\libapputilities_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\libapputilities_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libaudiointerfaces_2audio__interface_8cpp.html"
  Delete "$INSTDIR\doc\html\libaudiointerfaces_2audio__interface__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\libaudiointerfaces_2audio__interface__factory_8cpp.js"
  Delete "$INSTDIR\doc\html\libaudiointerfaces_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libaudiointerfaces_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libaudiointerfaces_2jack__interface_8cpp.html"
  Delete "$INSTDIR\doc\html\libaudiointerfaces_2portaudio__interface_8cpp.html"
  Delete "$INSTDIR\doc\html\libefl_2basic__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\libefl_2basic__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\libefl_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libefl_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libefl_2test_2lagrange__interpolator_8cpp.html"
  Delete "$INSTDIR\doc\html\libefl_2test_2lagrange__interpolator_8cpp.js"
  Delete "$INSTDIR\doc\html\libefl_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\libefl_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\libobjectmodel_2channel__object_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2diffuse__source_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2hoa__source_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2object_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2object__type_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2object__type_8cpp.js"
  Delete "$INSTDIR\doc\html\libobjectmodel_2object__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2object__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2object__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2plane__wave_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2point__source_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2point__source__extent_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2point__source__with__diffuseness_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2point__source__with__reverb_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2test_2point__source__with__reverb_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2test_2point__source__with__reverb_8cpp.js"
  Delete "$INSTDIR\doc\html\libobjectmodel_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\libobjectmodel_8dox.html"
  Delete "$INSTDIR\doc\html\libobjectmodel_mainpage.html"
  Delete "$INSTDIR\doc\html\libpanning_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libpanning_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libpanning_2test_2loudspeaker__array_8cpp.html"
  Delete "$INSTDIR\doc\html\libpanning_2test_2loudspeaker__array_8cpp.js"
  Delete "$INSTDIR\doc\html\libpanning_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\libpanning_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\libpanning_2test_2vbap_8cpp.html"
  Delete "$INSTDIR\doc\html\libpanning_2test_2vbap_8cpp.js"
  Delete "$INSTDIR\doc\html\libpanning_2vbap_8cpp.html"
  Delete "$INSTDIR\doc\html\libpanning_2_x_y_z_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2double__buffering__protocol_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2empty__parameter__config_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libpml_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libpml_2filter__routing__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2indexed__value__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2interpolation__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2interpolation__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\libpml_2interpolation__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libpml_2listener__position_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2listener__position_8cpp.js"
  Delete "$INSTDIR\doc\html\libpml_2matrix__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2message__queue__protocol_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2object__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2object__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\libpml_2object__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libpml_2scalar__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2shared__data__protocol_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2signal__routing__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2sparse__gain__routing__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2string__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2test_2filter__routing__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2test_2filter__routing__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\libpml_2test_2matrix__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2test_2matrix__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\libpml_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\libpml_2test_2vector__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpml_2test_2vector__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\libpml_2vector__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libpythoncomponents_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libpythoncomponents_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libpythoncomponents_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\libpythoncomponents_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\libpythonsupport_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libpythonsupport_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librbbl_2biquad__coefficient_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2core__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2crossfading__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librbbl_2filter__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2gain__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2gain__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librbbl_2interpolating__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2interpolation__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2interpolation__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2interpolation__parameter_8hpp.js"
  Delete "$INSTDIR\doc\html\librbbl_2interpolation__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librbbl_2lagrange__interpolator_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2multichannel__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2object__channel__allocator_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2sparse__gain__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2test_2biquad__coefficient_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2test_2biquad__coefficient_8cpp.js"
  Delete "$INSTDIR\doc\html\librbbl_2test_2multichannel__convolver_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2test_2multichannel__convolver_8cpp.js"
  Delete "$INSTDIR\doc\html\librbbl_2test_2object__channel__allocator_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2test_2object__channel__allocator_8cpp.js"
  Delete "$INSTDIR\doc\html\librbbl_2test_2sparse__gain__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2test_2sparse__gain__routing_8cpp.js"
  Delete "$INSTDIR\doc\html\librbbl_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\librbbl_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2add_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2biquad__iir__filter_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2cap__gain__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2channel__object__routing__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2crossfading__fir__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2crossfading__fir__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2delay__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2delay__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2delay__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\librcl_2delay__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\librcl_2delay__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librcl_2delay__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2delay__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2delay__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\librcl_2delay__vector_8hpp.js"
  Delete "$INSTDIR\doc\html\librcl_2delay__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librcl_2diffusion__gain__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\librcl_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librcl_2fir__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2fir__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2gain__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\librcl_2gain__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librcl_2gain__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2hoa__allrad__gain__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2interpolating__fir__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2interpolating__fir__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2listener__compensation_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2listener__compensation_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2null__source_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2object__gain__eq__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2panning__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2panning__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2position__decoder_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2scalar__osc__decoder_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2scene__decoder_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2scene__encoder_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2signal__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2sparse__gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2sparse__gain__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2test_2biquad__iir__filter_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2test_2biquad__iir__filter_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2test_2hoa__allrad__gain__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2test_2hoa__allrad__gain__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2test_2scene__decoder_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2test_2scene__decoder_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2test_2signal__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2test_2signal__routing_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\librcl_2time__frequency__inverse__transform_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2time__frequency__transform_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2udp__receiver_8cpp.html"
  Delete "$INSTDIR\doc\html\librcl_2udp__sender_8cpp.html"
  Delete "$INSTDIR\doc\html\libreverbobject_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libreverbobject_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libreverbobject_2late__reverb__filter__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\libreverbobject_2late__reverb__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\libreverbobject_2reverb__object__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\libreverbobject_2reverb__parameter__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\libreverbobject_2reverb__parameter__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\librrl_2audio__signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\librrl_2audio__signal__flow_8cpp.js"
  Delete "$INSTDIR\doc\html\librrl_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\librrl_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\librrl_2integrity__checking_8cpp.html"
  Delete "$INSTDIR\doc\html\librrl_2integrity__checking_8cpp.js"
  Delete "$INSTDIR\doc\html\librrl_2test_2parameter__connection_8cpp.html"
  Delete "$INSTDIR\doc\html\librrl_2test_2parameter__connection_8cpp.js"
  Delete "$INSTDIR\doc\html\librrl_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\librrl_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\libsignalflowspython_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libsignalflowspython_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libsignalflowspython_2visr__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflowspython_2visr__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\libsignalflowspython_2visr__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2baseline__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2baseline__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2baseline__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2core__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2core__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2core__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2delay__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2delay__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2delay__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2gain__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2gain__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2test_2baseline__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2test_2baseline__renderer_8cpp.js"
  Delete "$INSTDIR\doc\html\libsignalflows_2test_2delay__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2test_2delay__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\libsignalflows_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\libsignalflows_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\libsignalflows_2time__frequency__feedthrough_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2atomic__component_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2audio__sample__type_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2audio__sample__type_8cpp.js"
  Delete "$INSTDIR\doc\html\libvisr_2channel__list_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2channel__list_8cpp.js"
  Delete "$INSTDIR\doc\html\libvisr_2communication__protocol__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2component_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2composite__component_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\libvisr_2parameter__base_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2parameter__config__base_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2parameter__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2signal__flow__context_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2test_2audio__sample__type_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2test_2audio__sample__type_8cpp.js"
  Delete "$INSTDIR\doc\html\libvisr_2test_2parameter__connection_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2test_2parameter__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2test_2test__main_8cpp.html"
  Delete "$INSTDIR\doc\html\libvisr_2test_2test__main_8cpp.js"
  Delete "$INSTDIR\doc\html\listener__compensation_8hpp.html"
  Delete "$INSTDIR\doc\html\listener__compensation_8hpp_source.html"
  Delete "$INSTDIR\doc\html\listener__position_8hpp.html"
  Delete "$INSTDIR\doc\html\listener__position_8hpp.js"
  Delete "$INSTDIR\doc\html\listener__position_8hpp_source.html"
  Delete "$INSTDIR\doc\html\load__module_8cpp.html"
  Delete "$INSTDIR\doc\html\load__module_8cpp.js"
  Delete "$INSTDIR\doc\html\load__module_8hpp.html"
  Delete "$INSTDIR\doc\html\load__module_8hpp.js"
  Delete "$INSTDIR\doc\html\load__module_8hpp_source.html"
  Delete "$INSTDIR\doc\html\mainpage_8dox.html"
  Delete "$INSTDIR\doc\html\matrix__convolver_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\matrix__convolver_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\matrix__convolver_8cpp.html"
  Delete "$INSTDIR\doc\html\matrix__convolver_8cpp.js"
  Delete "$INSTDIR\doc\html\matrix__convolver_8hpp.html"
  Delete "$INSTDIR\doc\html\matrix__convolver_8hpp_source.html"
  Delete "$INSTDIR\doc\html\matrix__from__ndarray_8hpp.html"
  Delete "$INSTDIR\doc\html\matrix__from__ndarray_8hpp.js"
  Delete "$INSTDIR\doc\html\matrix__from__ndarray_8hpp_source.html"
  Delete "$INSTDIR\doc\html\matrix__functions_8cpp.html"
  Delete "$INSTDIR\doc\html\matrix__functions_8cpp.js"
  Delete "$INSTDIR\doc\html\matrix__functions_8hpp.html"
  Delete "$INSTDIR\doc\html\matrix__functions_8hpp.js"
  Delete "$INSTDIR\doc\html\matrix__functions_8hpp_source.html"
  Delete "$INSTDIR\doc\html\matrix__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\matrix__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\matrix__parameter__config_8cpp.html"
  Delete "$INSTDIR\doc\html\matrix__parameter__config_8hpp.html"
  Delete "$INSTDIR\doc\html\matrix__parameter__config_8hpp_source.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2delay__vector_2delay__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2delay__vector_2delay__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\maxmspexternals_2delay__vector_2delay__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2delay__vector_2delay__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2gain__matrix_2gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2gain__matrix_2gain__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\maxmspexternals_2gain__matrix_2gain__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2gain__matrix_2gain__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2libmaxsupport_2export__symbols_8hpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2libmaxsupport_2export__symbols_8hpp.js"
  Delete "$INSTDIR\doc\html\maxmspexternals_2libmaxsupport_2export__symbols_8hpp_source.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2init__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2init__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2init__filter__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2init__filter__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2init__filter__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2matrix__convolver_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2visr__renderer_2options_8cpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2visr__renderer_2options_8hpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2visr__renderer_2options_8hpp_source.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2visr__renderer_2visr__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2visr__renderer_2visr__renderer_8cpp.js"
  Delete "$INSTDIR\doc\html\maxmspexternals_2visr__renderer_2visr__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\maxmspexternals_2visr__renderer_2visr__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\max__msp__externals_8dox.html"
  Delete "$INSTDIR\doc\html\menu.js"
  Delete "$INSTDIR\doc\html\menudata.js"
  Delete "$INSTDIR\doc\html\message__queue__protocol_8hpp.html"
  Delete "$INSTDIR\doc\html\message__queue__protocol_8hpp_source.html"
  Delete "$INSTDIR\doc\html\mex_2biquad__iir__filter_2biquad__iir__filter_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2biquad__iir__filter_2biquad__iir__filter_8cpp.js"
  Delete "$INSTDIR\doc\html\mex_2biquad__iir__filter_2signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2biquad__iir__filter_2signal__flow_8cpp.js"
  Delete "$INSTDIR\doc\html\mex_2biquad__iir__filter_2signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\mex_2biquad__iir__filter_2signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\mex_2delay__vector_2delay__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2delay__vector_2delay__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\mex_2feedthrough_2signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2feedthrough_2signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\mex_2feedthrough_2signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\mex_2gain__matrix_2gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2gain__matrix_2gain__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\mex_2gain__matrix_2signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2gain__matrix_2signal__flow_8cpp.js"
  Delete "$INSTDIR\doc\html\mex_2gain__matrix_2signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\mex_2gain__matrix_2signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\mex_2late__reverb__filter__calculator_2late__reverb__filter__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2late__reverb__filter__calculator_2late__reverb__filter__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\mex_2late__reverb__filter__calculator_2signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2late__reverb__filter__calculator_2signal__flow_8hpp.html"
  Delete "$INSTDIR\doc\html\mex_2late__reverb__filter__calculator_2signal__flow_8hpp_source.html"
  Delete "$INSTDIR\doc\html\mex_2multichannel__convolver_2multichannel__convolver_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2multichannel__convolver_2multichannel__convolver_8cpp.js"
  Delete "$INSTDIR\doc\html\mex_2time__frequency__feedthrough_2time__frequency__feedthrough_8cpp.html"
  Delete "$INSTDIR\doc\html\mex_2time__frequency__feedthrough_2time__frequency__feedthrough_8cpp.js"
  Delete "$INSTDIR\doc\html\mex__wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\mex__wrapper_8hpp.html"
  Delete "$INSTDIR\doc\html\mex__wrapper_8hpp_source.html"
  Delete "$INSTDIR\doc\html\multichannel__convolver__uniform_8hpp.html"
  Delete "$INSTDIR\doc\html\multichannel__convolver__uniform_8hpp_source.html"
  Delete "$INSTDIR\doc\html\multichannel__delay__line_8cpp.html"
  Delete "$INSTDIR\doc\html\multichannel__delay__line_8hpp.html"
  Delete "$INSTDIR\doc\html\multichannel__delay__line_8hpp_source.html"
  Delete "$INSTDIR\doc\html\namespacemembers.html"
  Delete "$INSTDIR\doc\html\namespacemembers_b.html"
  Delete "$INSTDIR\doc\html\namespacemembers_c.html"
  Delete "$INSTDIR\doc\html\namespacemembers_d.html"
  Delete "$INSTDIR\doc\html\namespacemembers_dup.js"
  Delete "$INSTDIR\doc\html\namespacemembers_e.html"
  Delete "$INSTDIR\doc\html\namespacemembers_enum.html"
  Delete "$INSTDIR\doc\html\namespacemembers_eval.html"
  Delete "$INSTDIR\doc\html\namespacemembers_f.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func.js"
  Delete "$INSTDIR\doc\html\namespacemembers_func_b.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_c.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_d.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_e.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_f.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_g.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_h.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_i.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_l.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_m.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_n.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_o.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_p.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_q.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_r.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_s.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_t.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_v.html"
  Delete "$INSTDIR\doc\html\namespacemembers_func_w.html"
  Delete "$INSTDIR\doc\html\namespacemembers_g.html"
  Delete "$INSTDIR\doc\html\namespacemembers_h.html"
  Delete "$INSTDIR\doc\html\namespacemembers_i.html"
  Delete "$INSTDIR\doc\html\namespacemembers_l.html"
  Delete "$INSTDIR\doc\html\namespacemembers_m.html"
  Delete "$INSTDIR\doc\html\namespacemembers_n.html"
  Delete "$INSTDIR\doc\html\namespacemembers_o.html"
  Delete "$INSTDIR\doc\html\namespacemembers_p.html"
  Delete "$INSTDIR\doc\html\namespacemembers_q.html"
  Delete "$INSTDIR\doc\html\namespacemembers_r.html"
  Delete "$INSTDIR\doc\html\namespacemembers_s.html"
  Delete "$INSTDIR\doc\html\namespacemembers_t.html"
  Delete "$INSTDIR\doc\html\namespacemembers_type.html"
  Delete "$INSTDIR\doc\html\namespacemembers_u.html"
  Delete "$INSTDIR\doc\html\namespacemembers_v.html"
  Delete "$INSTDIR\doc\html\namespacemembers_vars.html"
  Delete "$INSTDIR\doc\html\namespacemembers_w.html"
  Delete "$INSTDIR\doc\html\namespaceoscpkt.html"
  Delete "$INSTDIR\doc\html\namespacepybind11.html"
  Delete "$INSTDIR\doc\html\namespaces.html"
  Delete "$INSTDIR\doc\html\namespaces_8dox.html"
  Delete "$INSTDIR\doc\html\namespaces_dup.js"
  Delete "$INSTDIR\doc\html\namespacevisr.html"
  Delete "$INSTDIR\doc\html\namespacevisr.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1audio__network__streamer.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1audio__network__streamer.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1baseline__renderer.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1baseline__renderer.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1feedthrough.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1feedthrough.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1gain__matrix.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1gain__matrix.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1matrix__convolver.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1matrix__convolver.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1pythonflowrunner.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1pythonflowrunner.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1scene__decoder.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apps_1_1scene__decoder.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apputilities.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1apputilities.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1audiointerfaces.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1audiointerfaces.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1audiointerfaces_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1audiointerfaces_1_1test.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1detail.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1efl.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1efl.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1efl_1_1python.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1efl_1_1reference.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1efl_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1impl.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1impl.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1impl_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1impl_1_1test.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1maxmsp.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1maxmsp.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1maxmsp_1_1matrix__convolver.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1maxmsp_1_1matrix__convolver.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1maxmsp_1_1visr__renderer.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1maxmsp_1_1visr__renderer.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mexsupport.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mexsupport.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex_1_1feedthrough.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex_1_1feedthrough.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex_1_1gain__matrix.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex_1_1gain__matrix.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex_1_1late__reverb__filter__calculator.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex_1_1late__reverb__filter__calculator.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1mex_1_1multichannel__convolver.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1objectmodel.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1objectmodel.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1objectmodel_1_1python.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1objectmodel_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1panning.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1panning.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1panning_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pml.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pml.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pml_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pythoncomponents.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pythoncomponents.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pythoncomponents_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pythonsupport.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1pythonsupport.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1audiointerfaces.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1bindinghelpers.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1panning.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1pml.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1rcl.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1reverbobject.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1rrl.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1signalflows.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1visr.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1python_1_1visr.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rbbl.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rbbl.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rbbl_1_1python.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rbbl_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rbbl_1_1_parametric_iir_coefficient_calculator.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rcl.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rcl.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rcl_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1reverbobject.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1reverbobject.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1ril.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1ril.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1ril_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rrl.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rrl.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1rrl_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1signalflows.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1signalflows.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1signalflowspython.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1signalflowspython.js"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1signalflows_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1test.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1version.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1_audio_sample_type.html"
  Delete "$INSTDIR\doc\html\namespacevisr_1_1_audio_sample_type.js"
  Delete "$INSTDIR\doc\html\navtree.css"
  Delete "$INSTDIR\doc\html\navtree.js"
  Delete "$INSTDIR\doc\html\navtreedata.js"
  Delete "$INSTDIR\doc\html\navtreeindex0.js"
  Delete "$INSTDIR\doc\html\navtreeindex1.js"
  Delete "$INSTDIR\doc\html\navtreeindex10.js"
  Delete "$INSTDIR\doc\html\navtreeindex11.js"
  Delete "$INSTDIR\doc\html\navtreeindex12.js"
  Delete "$INSTDIR\doc\html\navtreeindex13.js"
  Delete "$INSTDIR\doc\html\navtreeindex14.js"
  Delete "$INSTDIR\doc\html\navtreeindex15.js"
  Delete "$INSTDIR\doc\html\navtreeindex16.js"
  Delete "$INSTDIR\doc\html\navtreeindex17.js"
  Delete "$INSTDIR\doc\html\navtreeindex18.js"
  Delete "$INSTDIR\doc\html\navtreeindex2.js"
  Delete "$INSTDIR\doc\html\navtreeindex3.js"
  Delete "$INSTDIR\doc\html\navtreeindex4.js"
  Delete "$INSTDIR\doc\html\navtreeindex5.js"
  Delete "$INSTDIR\doc\html\navtreeindex6.js"
  Delete "$INSTDIR\doc\html\navtreeindex7.js"
  Delete "$INSTDIR\doc\html\navtreeindex8.js"
  Delete "$INSTDIR\doc\html\navtreeindex9.js"
  Delete "$INSTDIR\doc\html\nav_f.png"
  Delete "$INSTDIR\doc\html\nav_g.png"
  Delete "$INSTDIR\doc\html\nav_h.png"
  Delete "$INSTDIR\doc\html\network__audio__streamer_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\network__audio__streamer_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\null__source_8hpp.html"
  Delete "$INSTDIR\doc\html\null__source_8hpp_source.html"
  Delete "$INSTDIR\doc\html\objectmodel_8cpp.html"
  Delete "$INSTDIR\doc\html\objectmodel_8cpp.js"
  Delete "$INSTDIR\doc\html\object_8hpp.html"
  Delete "$INSTDIR\doc\html\object_8hpp.js"
  Delete "$INSTDIR\doc\html\object_8hpp_source.html"
  Delete "$INSTDIR\doc\html\object__channel__allocator_8hpp.html"
  Delete "$INSTDIR\doc\html\object__channel__allocator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\object__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\object__factory_8cpp.js"
  Delete "$INSTDIR\doc\html\object__factory_8hpp.html"
  Delete "$INSTDIR\doc\html\object__factory_8hpp_source.html"
  Delete "$INSTDIR\doc\html\object__gain__eq__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\object__gain__eq__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\object__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\object__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\object__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\object__type_8hpp.html"
  Delete "$INSTDIR\doc\html\object__type_8hpp.js"
  Delete "$INSTDIR\doc\html\object__type_8hpp_source.html"
  Delete "$INSTDIR\doc\html\object__vector__assign_8cpp.html"
  Delete "$INSTDIR\doc\html\object__vector__assign_8cpp.js"
  Delete "$INSTDIR\doc\html\object__vector__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\object__vector__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\object__vector__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\open.png"
  Delete "$INSTDIR\doc\html\pages.html"
  Delete "$INSTDIR\doc\html\panning_8cpp.html"
  Delete "$INSTDIR\doc\html\panning_8cpp.js"
  Delete "$INSTDIR\doc\html\panning__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\panning__calculator_8hpp.js"
  Delete "$INSTDIR\doc\html\panning__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__base_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__config__base_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__config__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__connection__descriptor_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__connection__descriptor_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__connection__descriptor_8hpp.js"
  Delete "$INSTDIR\doc\html\parameter__connection__descriptor_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__connection__graph_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__connection__graph_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__connection__graph_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__connection__map_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__connection__map_8cpp.js"
  Delete "$INSTDIR\doc\html\parameter__connection__map_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__connection__map_8hpp.js"
  Delete "$INSTDIR\doc\html\parameter__connection__map_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__factory_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__factory_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__input_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__input_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__input_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__instantiation_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__instantiation_8cpp.js"
  Delete "$INSTDIR\doc\html\parameter__output_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__output_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__output_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__port_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__port_8cpp.js"
  Delete "$INSTDIR\doc\html\parameter__port__base_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__port__base_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__port__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__port__base__implementation_8cpp.html"
  Delete "$INSTDIR\doc\html\parameter__port__base__implementation_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__port__base__implementation_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parameter__type_8hpp.html"
  Delete "$INSTDIR\doc\html\parameter__type_8hpp.js"
  Delete "$INSTDIR\doc\html\parameter__type_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient_8cpp.html"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient_8hpp.html"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient_8hpp_source.html"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient__calculator_8hpp.js"
  Delete "$INSTDIR\doc\html\parametric__iir__coefficient__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\plane__wave_8hpp.html"
  Delete "$INSTDIR\doc\html\plane__wave_8hpp_source.html"
  Delete "$INSTDIR\doc\html\plane__wave__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\plane__wave__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\plane__wave__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\pml_8cpp.html"
  Delete "$INSTDIR\doc\html\pml_8cpp.js"
  Delete "$INSTDIR\doc\html\point__source_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source_8hpp_source.html"
  Delete "$INSTDIR\doc\html\point__source__extent_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source__extent_8hpp_source.html"
  Delete "$INSTDIR\doc\html\point__source__extent__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\point__source__extent__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source__extent__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\point__source__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\point__source__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\point__source__with__diffuseness_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source__with__diffuseness_8hpp_source.html"
  Delete "$INSTDIR\doc\html\point__source__with__diffuseness__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\point__source__with__diffuseness__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source__with__diffuseness__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\point__source__with__reverb_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source__with__reverb_8hpp_source.html"
  Delete "$INSTDIR\doc\html\point__source__with__reverb__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\point__source__with__reverb__parser_8hpp.html"
  Delete "$INSTDIR\doc\html\point__source__with__reverb__parser_8hpp_source.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__input_8cpp.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__input_8hpp.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__input_8hpp_source.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__output_8cpp.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__output_8hpp.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__output_8hpp_source.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__port_8cpp.html"
  Delete "$INSTDIR\doc\html\polymorphic__parameter__port_8cpp.js"
  Delete "$INSTDIR\doc\html\portaudioconf_8dox.html"
  Delete "$INSTDIR\doc\html\portaudio_audio_interface_configuration.html"
  Delete "$INSTDIR\doc\html\portaudio__interface_8hpp.html"
  Delete "$INSTDIR\doc\html\portaudio__interface_8hpp_source.html"
  Delete "$INSTDIR\doc\html\port_8cpp.html"
  Delete "$INSTDIR\doc\html\port_8cpp.js"
  Delete "$INSTDIR\doc\html\port__base_8cpp.html"
  Delete "$INSTDIR\doc\html\port__base_8cpp.js"
  Delete "$INSTDIR\doc\html\port__base_8hpp.html"
  Delete "$INSTDIR\doc\html\port__base_8hpp.js"
  Delete "$INSTDIR\doc\html\port__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\port__base__implementation_8cpp.html"
  Delete "$INSTDIR\doc\html\port__base__implementation_8hpp.html"
  Delete "$INSTDIR\doc\html\port__base__implementation_8hpp_source.html"
  Delete "$INSTDIR\doc\html\port__utilities_8cpp.html"
  Delete "$INSTDIR\doc\html\port__utilities_8cpp.js"
  Delete "$INSTDIR\doc\html\port__utilities_8hpp.html"
  Delete "$INSTDIR\doc\html\port__utilities_8hpp.js"
  Delete "$INSTDIR\doc\html\port__utilities_8hpp_source.html"
  Delete "$INSTDIR\doc\html\position__decoder_8hpp.html"
  Delete "$INSTDIR\doc\html\position__decoder_8hpp_source.html"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2audio__interface_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2audio__interface_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2audio__interface__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2audio__interface__factory_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2jack__interface_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2jack__interface_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2portaudio__interface_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2audiointerfaces_2portaudio__interface_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2efl_2basic__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2efl_2basic__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2efl_2basic__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2efl_2basic__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2channel__object_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2channel__object_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2diffuse__source_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2diffuse__source_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2hoa__source_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2hoa__source_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2object_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2object_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2object__type_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2object__type_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2object__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2object__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2plane__wave_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2plane__wave_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source__extent_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source__extent_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source__with__diffuseness_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source__with__diffuseness_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source__with__reverb_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2objectmodel_2point__source__with__reverb_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2panning_2loudspeaker__array_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2panning_2loudspeaker__array_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2panning_2vbap_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2panning_2vbap_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2panning_2_x_y_z_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2panning_2_x_y_z_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2double__buffering__protocol_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2double__buffering__protocol_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2empty__parameter__config_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2empty__parameter__config_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2filter__routing__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2filter__routing__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2indexed__value__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2indexed__value__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2interpolation__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2interpolation__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2listener__position_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2listener__position_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2matrix__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2matrix__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2message__queue__protocol_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2message__queue__protocol_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2object__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2object__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2scalar__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2scalar__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2shared__data__protocol_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2shared__data__protocol_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2signal__routing__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2signal__routing__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2sparse__gain__routing__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2sparse__gain__routing__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2string__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2string__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2pml_2vector__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2pml_2vector__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2biquad__coefficient_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2biquad__coefficient_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2core__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2core__convolver__uniform_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2crossfading__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2crossfading__convolver__uniform_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2filter__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2filter__routing_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2interpolating__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2interpolating__convolver__uniform_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2interpolation__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2interpolation__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2multichannel__convolver__uniform_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2multichannel__convolver__uniform_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2object__channel__allocator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2object__channel__allocator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rbbl_2sparse__gain__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rbbl_2sparse__gain__routing_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2add_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2add_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2biquad__iir__filter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2biquad__iir__filter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2cap__gain__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2cap__gain__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2channel__object__routing__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2channel__object__routing__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2crossfading__fir__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2crossfading__fir__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2delay__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2delay__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2delay__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2delay__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2diffusion__gain__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2diffusion__gain__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2fir__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2fir__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2gain__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2gain__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2gain__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2hoa__allrad__gain__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2hoa__allrad__gain__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2interpolating__fir__filter__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2interpolating__fir__filter__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2listener__compensation_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2listener__compensation_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2null__source_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2null__source_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2object__gain__eq__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2object__gain__eq__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2panning__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2panning__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2position__decoder_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2position__decoder_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2scalar__osc__decoder_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2scalar__osc__decoder_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2scene__decoder_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2scene__decoder_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2scene__encoder_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2scene__encoder_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2signal__routing_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2signal__routing_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2sparse__gain__matrix_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2sparse__gain__matrix_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2time__frequency__inverse__transform_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2time__frequency__inverse__transform_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2time__frequency__transform_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2time__frequency__transform_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2udp__receiver_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2udp__receiver_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rcl_2udp__sender_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rcl_2udp__sender_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2late__reverb__filter__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2late__reverb__filter__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2late__reverb__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2late__reverb__parameter_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2reverb__object__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2reverb__object__renderer_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2reverb__parameter__calculator_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2reverbobject_2reverb__parameter__calculator_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rrl_2audio__signal__flow_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rrl_2audio__signal__flow_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2rrl_2integrity__checking_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2rrl_2integrity__checking_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2baseline__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2baseline__renderer_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2baseline__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2baseline__renderer_8hpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2baseline__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2core__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2core__renderer_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2core__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2core__renderer_8hpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2core__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__vector_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__vector_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__vector_8hpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__vector_8hpp.js"
  Delete "$INSTDIR\doc\html\python_2signalflows_2delay__vector_8hpp_source.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2visr__renderer_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2signalflows_2visr__renderer_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2atomic__component_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2atomic__component_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2audio__sample__type_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2audio__sample__type_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2channel__list_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2channel__list_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2communication__protocol__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2communication__protocol__factory_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2component_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2component_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2composite__component_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2composite__component_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__base_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__base_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__config__base_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__config__base_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__connection_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__connection_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__factory_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2parameter__factory_8cpp.js"
  Delete "$INSTDIR\doc\html\python_2visr_2signal__flow__context_8cpp.html"
  Delete "$INSTDIR\doc\html\python_2visr_2signal__flow__context_8cpp.js"
  Delete "$INSTDIR\doc\html\python_support.html"
  Delete "$INSTDIR\doc\html\python__runner_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\python__runner_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\python__support_8dox.html"
  Delete "$INSTDIR\doc\html\rbbl_8cpp.html"
  Delete "$INSTDIR\doc\html\rbbl_8cpp.js"
  Delete "$INSTDIR\doc\html\rcl_8cpp.html"
  Delete "$INSTDIR\doc\html\rcl_8cpp.js"
  Delete "$INSTDIR\doc\html\resize.js"
  Delete "$INSTDIR\doc\html\reverbobject_8cpp.html"
  Delete "$INSTDIR\doc\html\reverbobject_8cpp.js"
  Delete "$INSTDIR\doc\html\reverb__object__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\reverb__object__parser_8cpp.js"
  Delete "$INSTDIR\doc\html\reverb__object__parser_8h.html"
  Delete "$INSTDIR\doc\html\reverb__object__parser_8h_source.html"
  Delete "$INSTDIR\doc\html\reverb__object__renderer_8hpp.html"
  Delete "$INSTDIR\doc\html\reverb__object__renderer_8hpp_source.html"
  Delete "$INSTDIR\doc\html\reverb__parameter__calculator_8hpp.html"
  Delete "$INSTDIR\doc\html\reverb__parameter__calculator_8hpp_source.html"
  Delete "$INSTDIR\doc\html\rrl_8cpp.html"
  Delete "$INSTDIR\doc\html\rrl_8cpp.js"
  Delete "$INSTDIR\doc\html\s3a_logo.png"
  Delete "$INSTDIR\doc\html\scalar__osc__decoder_8hpp.html"
  Delete "$INSTDIR\doc\html\scalar__osc__decoder_8hpp_source.html"
  Delete "$INSTDIR\doc\html\scalar__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\scalar__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\scene__decoder_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\scene__decoder_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\scene__decoder_8hpp.html"
  Delete "$INSTDIR\doc\html\scene__decoder_8hpp_source.html"
  Delete "$INSTDIR\doc\html\scene__encoder_8hpp.html"
  Delete "$INSTDIR\doc\html\scene__encoder_8hpp_source.html"
  Delete "$INSTDIR\doc\html\scheduling__graph_8cpp.html"
  Delete "$INSTDIR\doc\html\scheduling__graph_8hpp.html"
  Delete "$INSTDIR\doc\html\scheduling__graph_8hpp_source.html"
  Delete "$INSTDIR\doc\html\search"
  Delete "$INSTDIR\doc\html\search\all_0.html"
  Delete "$INSTDIR\doc\html\search\all_0.js"
  Delete "$INSTDIR\doc\html\search\all_1.html"
  Delete "$INSTDIR\doc\html\search\all_1.js"
  Delete "$INSTDIR\doc\html\search\all_10.html"
  Delete "$INSTDIR\doc\html\search\all_10.js"
  Delete "$INSTDIR\doc\html\search\all_11.html"
  Delete "$INSTDIR\doc\html\search\all_11.js"
  Delete "$INSTDIR\doc\html\search\all_12.html"
  Delete "$INSTDIR\doc\html\search\all_12.js"
  Delete "$INSTDIR\doc\html\search\all_13.html"
  Delete "$INSTDIR\doc\html\search\all_13.js"
  Delete "$INSTDIR\doc\html\search\all_14.html"
  Delete "$INSTDIR\doc\html\search\all_14.js"
  Delete "$INSTDIR\doc\html\search\all_15.html"
  Delete "$INSTDIR\doc\html\search\all_15.js"
  Delete "$INSTDIR\doc\html\search\all_16.html"
  Delete "$INSTDIR\doc\html\search\all_16.js"
  Delete "$INSTDIR\doc\html\search\all_17.html"
  Delete "$INSTDIR\doc\html\search\all_17.js"
  Delete "$INSTDIR\doc\html\search\all_18.html"
  Delete "$INSTDIR\doc\html\search\all_18.js"
  Delete "$INSTDIR\doc\html\search\all_19.html"
  Delete "$INSTDIR\doc\html\search\all_19.js"
  Delete "$INSTDIR\doc\html\search\all_1a.html"
  Delete "$INSTDIR\doc\html\search\all_1a.js"
  Delete "$INSTDIR\doc\html\search\all_2.html"
  Delete "$INSTDIR\doc\html\search\all_2.js"
  Delete "$INSTDIR\doc\html\search\all_3.html"
  Delete "$INSTDIR\doc\html\search\all_3.js"
  Delete "$INSTDIR\doc\html\search\all_4.html"
  Delete "$INSTDIR\doc\html\search\all_4.js"
  Delete "$INSTDIR\doc\html\search\all_5.html"
  Delete "$INSTDIR\doc\html\search\all_5.js"
  Delete "$INSTDIR\doc\html\search\all_6.html"
  Delete "$INSTDIR\doc\html\search\all_6.js"
  Delete "$INSTDIR\doc\html\search\all_7.html"
  Delete "$INSTDIR\doc\html\search\all_7.js"
  Delete "$INSTDIR\doc\html\search\all_8.html"
  Delete "$INSTDIR\doc\html\search\all_8.js"
  Delete "$INSTDIR\doc\html\search\all_9.html"
  Delete "$INSTDIR\doc\html\search\all_9.js"
  Delete "$INSTDIR\doc\html\search\all_a.html"
  Delete "$INSTDIR\doc\html\search\all_a.js"
  Delete "$INSTDIR\doc\html\search\all_b.html"
  Delete "$INSTDIR\doc\html\search\all_b.js"
  Delete "$INSTDIR\doc\html\search\all_c.html"
  Delete "$INSTDIR\doc\html\search\all_c.js"
  Delete "$INSTDIR\doc\html\search\all_d.html"
  Delete "$INSTDIR\doc\html\search\all_d.js"
  Delete "$INSTDIR\doc\html\search\all_e.html"
  Delete "$INSTDIR\doc\html\search\all_e.js"
  Delete "$INSTDIR\doc\html\search\all_f.html"
  Delete "$INSTDIR\doc\html\search\all_f.js"
  Delete "$INSTDIR\doc\html\search\classes_0.html"
  Delete "$INSTDIR\doc\html\search\classes_0.js"
  Delete "$INSTDIR\doc\html\search\classes_1.html"
  Delete "$INSTDIR\doc\html\search\classes_1.js"
  Delete "$INSTDIR\doc\html\search\classes_10.html"
  Delete "$INSTDIR\doc\html\search\classes_10.js"
  Delete "$INSTDIR\doc\html\search\classes_11.html"
  Delete "$INSTDIR\doc\html\search\classes_11.js"
  Delete "$INSTDIR\doc\html\search\classes_12.html"
  Delete "$INSTDIR\doc\html\search\classes_12.js"
  Delete "$INSTDIR\doc\html\search\classes_13.html"
  Delete "$INSTDIR\doc\html\search\classes_13.js"
  Delete "$INSTDIR\doc\html\search\classes_14.html"
  Delete "$INSTDIR\doc\html\search\classes_14.js"
  Delete "$INSTDIR\doc\html\search\classes_15.html"
  Delete "$INSTDIR\doc\html\search\classes_15.js"
  Delete "$INSTDIR\doc\html\search\classes_16.html"
  Delete "$INSTDIR\doc\html\search\classes_16.js"
  Delete "$INSTDIR\doc\html\search\classes_2.html"
  Delete "$INSTDIR\doc\html\search\classes_2.js"
  Delete "$INSTDIR\doc\html\search\classes_3.html"
  Delete "$INSTDIR\doc\html\search\classes_3.js"
  Delete "$INSTDIR\doc\html\search\classes_4.html"
  Delete "$INSTDIR\doc\html\search\classes_4.js"
  Delete "$INSTDIR\doc\html\search\classes_5.html"
  Delete "$INSTDIR\doc\html\search\classes_5.js"
  Delete "$INSTDIR\doc\html\search\classes_6.html"
  Delete "$INSTDIR\doc\html\search\classes_6.js"
  Delete "$INSTDIR\doc\html\search\classes_7.html"
  Delete "$INSTDIR\doc\html\search\classes_7.js"
  Delete "$INSTDIR\doc\html\search\classes_8.html"
  Delete "$INSTDIR\doc\html\search\classes_8.js"
  Delete "$INSTDIR\doc\html\search\classes_9.html"
  Delete "$INSTDIR\doc\html\search\classes_9.js"
  Delete "$INSTDIR\doc\html\search\classes_a.html"
  Delete "$INSTDIR\doc\html\search\classes_a.js"
  Delete "$INSTDIR\doc\html\search\classes_b.html"
  Delete "$INSTDIR\doc\html\search\classes_b.js"
  Delete "$INSTDIR\doc\html\search\classes_c.html"
  Delete "$INSTDIR\doc\html\search\classes_c.js"
  Delete "$INSTDIR\doc\html\search\classes_d.html"
  Delete "$INSTDIR\doc\html\search\classes_d.js"
  Delete "$INSTDIR\doc\html\search\classes_e.html"
  Delete "$INSTDIR\doc\html\search\classes_e.js"
  Delete "$INSTDIR\doc\html\search\classes_f.html"
  Delete "$INSTDIR\doc\html\search\classes_f.js"
  Delete "$INSTDIR\doc\html\search\close.png"
  Delete "$INSTDIR\doc\html\search\defines_0.html"
  Delete "$INSTDIR\doc\html\search\defines_0.js"
  Delete "$INSTDIR\doc\html\search\defines_1.html"
  Delete "$INSTDIR\doc\html\search\defines_1.js"
  Delete "$INSTDIR\doc\html\search\defines_2.html"
  Delete "$INSTDIR\doc\html\search\defines_2.js"
  Delete "$INSTDIR\doc\html\search\defines_3.html"
  Delete "$INSTDIR\doc\html\search\defines_3.js"
  Delete "$INSTDIR\doc\html\search\defines_4.html"
  Delete "$INSTDIR\doc\html\search\defines_4.js"
  Delete "$INSTDIR\doc\html\search\defines_5.html"
  Delete "$INSTDIR\doc\html\search\defines_5.js"
  Delete "$INSTDIR\doc\html\search\defines_6.html"
  Delete "$INSTDIR\doc\html\search\defines_6.js"
  Delete "$INSTDIR\doc\html\search\defines_7.html"
  Delete "$INSTDIR\doc\html\search\defines_7.js"
  Delete "$INSTDIR\doc\html\search\defines_8.html"
  Delete "$INSTDIR\doc\html\search\defines_8.js"
  Delete "$INSTDIR\doc\html\search\defines_9.html"
  Delete "$INSTDIR\doc\html\search\defines_9.js"
  Delete "$INSTDIR\doc\html\search\enums_0.html"
  Delete "$INSTDIR\doc\html\search\enums_0.js"
  Delete "$INSTDIR\doc\html\search\enums_1.html"
  Delete "$INSTDIR\doc\html\search\enums_1.js"
  Delete "$INSTDIR\doc\html\search\enums_2.html"
  Delete "$INSTDIR\doc\html\search\enums_2.js"
  Delete "$INSTDIR\doc\html\search\enums_3.html"
  Delete "$INSTDIR\doc\html\search\enums_3.js"
  Delete "$INSTDIR\doc\html\search\enums_4.html"
  Delete "$INSTDIR\doc\html\search\enums_4.js"
  Delete "$INSTDIR\doc\html\search\enums_5.html"
  Delete "$INSTDIR\doc\html\search\enums_5.js"
  Delete "$INSTDIR\doc\html\search\enums_6.html"
  Delete "$INSTDIR\doc\html\search\enums_6.js"
  Delete "$INSTDIR\doc\html\search\enums_7.html"
  Delete "$INSTDIR\doc\html\search\enums_7.js"
  Delete "$INSTDIR\doc\html\search\enums_8.html"
  Delete "$INSTDIR\doc\html\search\enums_8.js"
  Delete "$INSTDIR\doc\html\search\enums_9.html"
  Delete "$INSTDIR\doc\html\search\enums_9.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_0.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_0.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_1.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_1.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_10.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_10.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_11.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_11.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_2.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_2.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_3.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_3.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_4.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_4.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_5.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_5.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_6.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_6.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_7.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_7.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_8.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_8.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_9.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_9.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_a.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_a.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_b.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_b.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_c.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_c.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_d.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_d.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_e.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_e.js"
  Delete "$INSTDIR\doc\html\search\enumvalues_f.html"
  Delete "$INSTDIR\doc\html\search\enumvalues_f.js"
  Delete "$INSTDIR\doc\html\search\files_0.html"
  Delete "$INSTDIR\doc\html\search\files_0.js"
  Delete "$INSTDIR\doc\html\search\files_1.html"
  Delete "$INSTDIR\doc\html\search\files_1.js"
  Delete "$INSTDIR\doc\html\search\files_10.html"
  Delete "$INSTDIR\doc\html\search\files_10.js"
  Delete "$INSTDIR\doc\html\search\files_11.html"
  Delete "$INSTDIR\doc\html\search\files_11.js"
  Delete "$INSTDIR\doc\html\search\files_12.html"
  Delete "$INSTDIR\doc\html\search\files_12.js"
  Delete "$INSTDIR\doc\html\search\files_13.html"
  Delete "$INSTDIR\doc\html\search\files_13.js"
  Delete "$INSTDIR\doc\html\search\files_14.html"
  Delete "$INSTDIR\doc\html\search\files_14.js"
  Delete "$INSTDIR\doc\html\search\files_15.html"
  Delete "$INSTDIR\doc\html\search\files_15.js"
  Delete "$INSTDIR\doc\html\search\files_16.html"
  Delete "$INSTDIR\doc\html\search\files_16.js"
  Delete "$INSTDIR\doc\html\search\files_2.html"
  Delete "$INSTDIR\doc\html\search\files_2.js"
  Delete "$INSTDIR\doc\html\search\files_3.html"
  Delete "$INSTDIR\doc\html\search\files_3.js"
  Delete "$INSTDIR\doc\html\search\files_4.html"
  Delete "$INSTDIR\doc\html\search\files_4.js"
  Delete "$INSTDIR\doc\html\search\files_5.html"
  Delete "$INSTDIR\doc\html\search\files_5.js"
  Delete "$INSTDIR\doc\html\search\files_6.html"
  Delete "$INSTDIR\doc\html\search\files_6.js"
  Delete "$INSTDIR\doc\html\search\files_7.html"
  Delete "$INSTDIR\doc\html\search\files_7.js"
  Delete "$INSTDIR\doc\html\search\files_8.html"
  Delete "$INSTDIR\doc\html\search\files_8.js"
  Delete "$INSTDIR\doc\html\search\files_9.html"
  Delete "$INSTDIR\doc\html\search\files_9.js"
  Delete "$INSTDIR\doc\html\search\files_a.html"
  Delete "$INSTDIR\doc\html\search\files_a.js"
  Delete "$INSTDIR\doc\html\search\files_b.html"
  Delete "$INSTDIR\doc\html\search\files_b.js"
  Delete "$INSTDIR\doc\html\search\files_c.html"
  Delete "$INSTDIR\doc\html\search\files_c.js"
  Delete "$INSTDIR\doc\html\search\files_d.html"
  Delete "$INSTDIR\doc\html\search\files_d.js"
  Delete "$INSTDIR\doc\html\search\files_e.html"
  Delete "$INSTDIR\doc\html\search\files_e.js"
  Delete "$INSTDIR\doc\html\search\files_f.html"
  Delete "$INSTDIR\doc\html\search\files_f.js"
  Delete "$INSTDIR\doc\html\search\functions_0.html"
  Delete "$INSTDIR\doc\html\search\functions_0.js"
  Delete "$INSTDIR\doc\html\search\functions_1.html"
  Delete "$INSTDIR\doc\html\search\functions_1.js"
  Delete "$INSTDIR\doc\html\search\functions_10.html"
  Delete "$INSTDIR\doc\html\search\functions_10.js"
  Delete "$INSTDIR\doc\html\search\functions_11.html"
  Delete "$INSTDIR\doc\html\search\functions_11.js"
  Delete "$INSTDIR\doc\html\search\functions_12.html"
  Delete "$INSTDIR\doc\html\search\functions_12.js"
  Delete "$INSTDIR\doc\html\search\functions_13.html"
  Delete "$INSTDIR\doc\html\search\functions_13.js"
  Delete "$INSTDIR\doc\html\search\functions_14.html"
  Delete "$INSTDIR\doc\html\search\functions_14.js"
  Delete "$INSTDIR\doc\html\search\functions_15.html"
  Delete "$INSTDIR\doc\html\search\functions_15.js"
  Delete "$INSTDIR\doc\html\search\functions_16.html"
  Delete "$INSTDIR\doc\html\search\functions_16.js"
  Delete "$INSTDIR\doc\html\search\functions_17.html"
  Delete "$INSTDIR\doc\html\search\functions_17.js"
  Delete "$INSTDIR\doc\html\search\functions_18.html"
  Delete "$INSTDIR\doc\html\search\functions_18.js"
  Delete "$INSTDIR\doc\html\search\functions_19.html"
  Delete "$INSTDIR\doc\html\search\functions_19.js"
  Delete "$INSTDIR\doc\html\search\functions_1a.html"
  Delete "$INSTDIR\doc\html\search\functions_1a.js"
  Delete "$INSTDIR\doc\html\search\functions_2.html"
  Delete "$INSTDIR\doc\html\search\functions_2.js"
  Delete "$INSTDIR\doc\html\search\functions_3.html"
  Delete "$INSTDIR\doc\html\search\functions_3.js"
  Delete "$INSTDIR\doc\html\search\functions_4.html"
  Delete "$INSTDIR\doc\html\search\functions_4.js"
  Delete "$INSTDIR\doc\html\search\functions_5.html"
  Delete "$INSTDIR\doc\html\search\functions_5.js"
  Delete "$INSTDIR\doc\html\search\functions_6.html"
  Delete "$INSTDIR\doc\html\search\functions_6.js"
  Delete "$INSTDIR\doc\html\search\functions_7.html"
  Delete "$INSTDIR\doc\html\search\functions_7.js"
  Delete "$INSTDIR\doc\html\search\functions_8.html"
  Delete "$INSTDIR\doc\html\search\functions_8.js"
  Delete "$INSTDIR\doc\html\search\functions_9.html"
  Delete "$INSTDIR\doc\html\search\functions_9.js"
  Delete "$INSTDIR\doc\html\search\functions_a.html"
  Delete "$INSTDIR\doc\html\search\functions_a.js"
  Delete "$INSTDIR\doc\html\search\functions_b.html"
  Delete "$INSTDIR\doc\html\search\functions_b.js"
  Delete "$INSTDIR\doc\html\search\functions_c.html"
  Delete "$INSTDIR\doc\html\search\functions_c.js"
  Delete "$INSTDIR\doc\html\search\functions_d.html"
  Delete "$INSTDIR\doc\html\search\functions_d.js"
  Delete "$INSTDIR\doc\html\search\functions_e.html"
  Delete "$INSTDIR\doc\html\search\functions_e.js"
  Delete "$INSTDIR\doc\html\search\functions_f.html"
  Delete "$INSTDIR\doc\html\search\functions_f.js"
  Delete "$INSTDIR\doc\html\search\mag_sel.png"
  Delete "$INSTDIR\doc\html\search\namespaces_0.html"
  Delete "$INSTDIR\doc\html\search\namespaces_0.js"
  Delete "$INSTDIR\doc\html\search\namespaces_1.html"
  Delete "$INSTDIR\doc\html\search\namespaces_1.js"
  Delete "$INSTDIR\doc\html\search\namespaces_2.html"
  Delete "$INSTDIR\doc\html\search\namespaces_2.js"
  Delete "$INSTDIR\doc\html\search\nomatches.html"
  Delete "$INSTDIR\doc\html\search\pages_0.html"
  Delete "$INSTDIR\doc\html\search\pages_0.js"
  Delete "$INSTDIR\doc\html\search\pages_1.html"
  Delete "$INSTDIR\doc\html\search\pages_1.js"
  Delete "$INSTDIR\doc\html\search\pages_2.html"
  Delete "$INSTDIR\doc\html\search\pages_2.js"
  Delete "$INSTDIR\doc\html\search\pages_3.html"
  Delete "$INSTDIR\doc\html\search\pages_3.js"
  Delete "$INSTDIR\doc\html\search\pages_4.html"
  Delete "$INSTDIR\doc\html\search\pages_4.js"
  Delete "$INSTDIR\doc\html\search\pages_5.html"
  Delete "$INSTDIR\doc\html\search\pages_5.js"
  Delete "$INSTDIR\doc\html\search\pages_6.html"
  Delete "$INSTDIR\doc\html\search\pages_6.js"
  Delete "$INSTDIR\doc\html\search\pages_7.html"
  Delete "$INSTDIR\doc\html\search\pages_7.js"
  Delete "$INSTDIR\doc\html\search\related_0.html"
  Delete "$INSTDIR\doc\html\search\related_0.js"
  Delete "$INSTDIR\doc\html\search\related_1.html"
  Delete "$INSTDIR\doc\html\search\related_1.js"
  Delete "$INSTDIR\doc\html\search\related_2.html"
  Delete "$INSTDIR\doc\html\search\related_2.js"
  Delete "$INSTDIR\doc\html\search\related_3.html"
  Delete "$INSTDIR\doc\html\search\related_3.js"
  Delete "$INSTDIR\doc\html\search\related_4.html"
  Delete "$INSTDIR\doc\html\search\related_4.js"
  Delete "$INSTDIR\doc\html\search\related_5.html"
  Delete "$INSTDIR\doc\html\search\related_5.js"
  Delete "$INSTDIR\doc\html\search\related_6.html"
  Delete "$INSTDIR\doc\html\search\related_6.js"
  Delete "$INSTDIR\doc\html\search\related_7.html"
  Delete "$INSTDIR\doc\html\search\related_7.js"
  Delete "$INSTDIR\doc\html\search\search.css"
  Delete "$INSTDIR\doc\html\search\search.js"
  Delete "$INSTDIR\doc\html\search\searchdata.js"
  Delete "$INSTDIR\doc\html\search\search_l.png"
  Delete "$INSTDIR\doc\html\search\search_m.png"
  Delete "$INSTDIR\doc\html\search\search_r.png"
  Delete "$INSTDIR\doc\html\search\typedefs_0.html"
  Delete "$INSTDIR\doc\html\search\typedefs_0.js"
  Delete "$INSTDIR\doc\html\search\typedefs_1.html"
  Delete "$INSTDIR\doc\html\search\typedefs_1.js"
  Delete "$INSTDIR\doc\html\search\typedefs_10.html"
  Delete "$INSTDIR\doc\html\search\typedefs_10.js"
  Delete "$INSTDIR\doc\html\search\typedefs_2.html"
  Delete "$INSTDIR\doc\html\search\typedefs_2.js"
  Delete "$INSTDIR\doc\html\search\typedefs_3.html"
  Delete "$INSTDIR\doc\html\search\typedefs_3.js"
  Delete "$INSTDIR\doc\html\search\typedefs_4.html"
  Delete "$INSTDIR\doc\html\search\typedefs_4.js"
  Delete "$INSTDIR\doc\html\search\typedefs_5.html"
  Delete "$INSTDIR\doc\html\search\typedefs_5.js"
  Delete "$INSTDIR\doc\html\search\typedefs_6.html"
  Delete "$INSTDIR\doc\html\search\typedefs_6.js"
  Delete "$INSTDIR\doc\html\search\typedefs_7.html"
  Delete "$INSTDIR\doc\html\search\typedefs_7.js"
  Delete "$INSTDIR\doc\html\search\typedefs_8.html"
  Delete "$INSTDIR\doc\html\search\typedefs_8.js"
  Delete "$INSTDIR\doc\html\search\typedefs_9.html"
  Delete "$INSTDIR\doc\html\search\typedefs_9.js"
  Delete "$INSTDIR\doc\html\search\typedefs_a.html"
  Delete "$INSTDIR\doc\html\search\typedefs_a.js"
  Delete "$INSTDIR\doc\html\search\typedefs_b.html"
  Delete "$INSTDIR\doc\html\search\typedefs_b.js"
  Delete "$INSTDIR\doc\html\search\typedefs_c.html"
  Delete "$INSTDIR\doc\html\search\typedefs_c.js"
  Delete "$INSTDIR\doc\html\search\typedefs_d.html"
  Delete "$INSTDIR\doc\html\search\typedefs_d.js"
  Delete "$INSTDIR\doc\html\search\typedefs_e.html"
  Delete "$INSTDIR\doc\html\search\typedefs_e.js"
  Delete "$INSTDIR\doc\html\search\typedefs_f.html"
  Delete "$INSTDIR\doc\html\search\typedefs_f.js"
  Delete "$INSTDIR\doc\html\search\variables_0.html"
  Delete "$INSTDIR\doc\html\search\variables_0.js"
  Delete "$INSTDIR\doc\html\search\variables_1.html"
  Delete "$INSTDIR\doc\html\search\variables_1.js"
  Delete "$INSTDIR\doc\html\search\variables_2.html"
  Delete "$INSTDIR\doc\html\search\variables_2.js"
  Delete "$INSTDIR\doc\html\search\variables_3.html"
  Delete "$INSTDIR\doc\html\search\variables_3.js"
  Delete "$INSTDIR\doc\html\search\variables_4.html"
  Delete "$INSTDIR\doc\html\search\variables_4.js"
  Delete "$INSTDIR\doc\html\search\variables_5.html"
  Delete "$INSTDIR\doc\html\search\variables_5.js"
  Delete "$INSTDIR\doc\html\search\variables_6.html"
  Delete "$INSTDIR\doc\html\search\variables_6.js"
  Delete "$INSTDIR\doc\html\search\variables_7.html"
  Delete "$INSTDIR\doc\html\search\variables_7.js"
  Delete "$INSTDIR\doc\html\search\variables_8.html"
  Delete "$INSTDIR\doc\html\search\variables_8.js"
  Delete "$INSTDIR\doc\html\search\variables_9.html"
  Delete "$INSTDIR\doc\html\search\variables_9.js"
  Delete "$INSTDIR\doc\html\search\variables_a.html"
  Delete "$INSTDIR\doc\html\search\variables_a.js"
  Delete "$INSTDIR\doc\html\search\variables_b.html"
  Delete "$INSTDIR\doc\html\search\variables_b.js"
  Delete "$INSTDIR\doc\html\search\variables_c.html"
  Delete "$INSTDIR\doc\html\search\variables_c.js"
  Delete "$INSTDIR\doc\html\search\variables_d.html"
  Delete "$INSTDIR\doc\html\search\variables_d.js"
  Delete "$INSTDIR\doc\html\search\variables_e.html"
  Delete "$INSTDIR\doc\html\search\variables_e.js"
  Delete "$INSTDIR\doc\html\shared__data__protocol_8hpp.html"
  Delete "$INSTDIR\doc\html\shared__data__protocol_8hpp_source.html"
  Delete "$INSTDIR\doc\html\signalflows_8cpp.html"
  Delete "$INSTDIR\doc\html\signalflows_8cpp.js"
  Delete "$INSTDIR\doc\html\signal__flow__context_8hpp.html"
  Delete "$INSTDIR\doc\html\signal__flow__context_8hpp_source.html"
  Delete "$INSTDIR\doc\html\signal__flow__wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\signal__flow__wrapper_8hpp.html"
  Delete "$INSTDIR\doc\html\signal__flow__wrapper_8hpp_source.html"
  Delete "$INSTDIR\doc\html\signal__routing_8hpp.html"
  Delete "$INSTDIR\doc\html\signal__routing_8hpp_source.html"
  Delete "$INSTDIR\doc\html\signal__routing__internal_8cpp.html"
  Delete "$INSTDIR\doc\html\signal__routing__internal_8cpp.js"
  Delete "$INSTDIR\doc\html\signal__routing__internal_8hpp.html"
  Delete "$INSTDIR\doc\html\signal__routing__internal_8hpp.js"
  Delete "$INSTDIR\doc\html\signal__routing__internal_8hpp_source.html"
  Delete "$INSTDIR\doc\html\signal__routing__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\signal__routing__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\sparse__gain__matrix_8hpp.html"
  Delete "$INSTDIR\doc\html\sparse__gain__matrix_8hpp.js"
  Delete "$INSTDIR\doc\html\sparse__gain__matrix_8hpp_source.html"
  Delete "$INSTDIR\doc\html\sparse__gain__routing_8hpp.html"
  Delete "$INSTDIR\doc\html\sparse__gain__routing_8hpp_source.html"
  Delete "$INSTDIR\doc\html\sparse__gain__routing__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\sparse__gain__routing__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\splitbar.png"
  Delete "$INSTDIR\doc\html\status__message_8hpp.html"
  Delete "$INSTDIR\doc\html\status__message_8hpp_source.html"
  Delete "$INSTDIR\doc\html\string__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\string__parameter_8hpp.js"
  Delete "$INSTDIR\doc\html\string__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_instantiate_audio_interface_factory-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_instantiate_audio_interface_factory.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_instantiate_audio_interface_factory.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_jack_interface_1_1_config-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_jack_interface_1_1_config.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_jack_interface_1_1_config.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_config-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_config.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1audiointerfaces_1_1_portaudio_interface_1_1_config.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1efl_1_1_denormalised_numbers-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1efl_1_1_denormalised_numbers.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1efl_1_1_denormalised_numbers.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1efl_1_1_denormalised_numbers_1_1_state-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1efl_1_1_denormalised_numbers_1_1_state.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1efl_1_1_denormalised_numbers_1_1_state.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_audio_connection-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_audio_connection.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_audio_connection.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_compare_ports-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_compare_ports.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_compare_ports.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_parameter_connection-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_parameter_connection.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1impl_1_1_parameter_connection.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1objectmodel_1_1_instantiate_object_factory-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1objectmodel_1_1_instantiate_object_factory.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1objectmodel_1_1_instantiate_object_factory.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_array_configuration_1_1_compare_speakers-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_array_configuration_1_1_compare_speakers.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_array_configuration_1_1_compare_speakers.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_array_configuration_1_1_speaker-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_array_configuration_1_1_speaker.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_array_configuration_1_1_speaker.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_filter_routing_parameter-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_filter_routing_parameter.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_filter_routing_parameter.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_filter_routing_parameter.png"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_signal_routing_parameter_1_1_entry-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_signal_routing_parameter_1_1_entry.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1pml_1_1_signal_routing_parameter_1_1_entry.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_filter_routing-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_filter_routing.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_filter_routing.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_filter_routing.png"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_instantiate_fft_wrapper_factory-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_instantiate_fft_wrapper_factory.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_instantiate_fft_wrapper_factory.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_sparse_gain_routing-members.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_sparse_gain_routing.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_sparse_gain_routing.js"
  Delete "$INSTDIR\doc\html\structvisr_1_1rbbl_1_1_sparse_gain_routing.png"
  Delete "$INSTDIR\doc\html\structvisr_1_1_audio_sample_type_1_1_id_to_type.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1_audio_sample_type_1_1_type_to_id.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1_communication_protocol_to_id.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1_id_to_communication_protocol.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1_id_to_parameter.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1_parameter_to_config_type.html"
  Delete "$INSTDIR\doc\html\structvisr_1_1_parameter_to_id.html"
  Delete "$INSTDIR\doc\html\sync_off.png"
  Delete "$INSTDIR\doc\html\sync_on.png"
  Delete "$INSTDIR\doc\html\t-design__t8___p40_8xml_8cpp.html"
  Delete "$INSTDIR\doc\html\tabs.css"
  Delete "$INSTDIR\doc\html\tab_a.png"
  Delete "$INSTDIR\doc\html\tab_b.png"
  Delete "$INSTDIR\doc\html\tab_h.png"
  Delete "$INSTDIR\doc\html\tab_s.png"
  Delete "$INSTDIR\doc\html\test_2audio__connection__descriptor_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2audio__connection__descriptor_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2channel__object__parser_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2channel__object__parser_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2circular__buffer_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2circular__buffer_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2ffts__wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2ffts__wrapper_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2float__sequence_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2float__sequence_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2index__sequence_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2index__sequence_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2ipp__fft__wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2ipp__fft__wrapper_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2parametric__iir__coefficient_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2parametric__iir__coefficient_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2wrapper_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2_all_r_a_d_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2_all_r_a_d_8cpp.js"
  Delete "$INSTDIR\doc\html\test_2_c_a_p_8cpp.html"
  Delete "$INSTDIR\doc\html\test_2_c_a_p_8cpp.js"
  Delete "$INSTDIR\doc\html\test__kinect__receiver_8cpp.html"
  Delete "$INSTDIR\doc\html\test__listener__compensation_8cpp.html"
  Delete "$INSTDIR\doc\html\test__listener__compensation_8cpp.js"
  Delete "$INSTDIR\doc\html\test___f_i_r_8cpp.html"
  Delete "$INSTDIR\doc\html\test___f_i_r_8cpp.js"
  Delete "$INSTDIR\doc\html\time__frequency__feedthrough_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__feedthrough_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\time__frequency__feedthrough_8hpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__feedthrough_8hpp_source.html"
  Delete "$INSTDIR\doc\html\time__frequency__inverse__transform_8hpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__inverse__transform_8hpp_source.html"
  Delete "$INSTDIR\doc\html\time__frequency__parameter_8cpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\time__frequency__parameter__config_8cpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__parameter__config_8hpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__parameter__config_8hpp_source.html"
  Delete "$INSTDIR\doc\html\time__frequency__transform_8hpp.html"
  Delete "$INSTDIR\doc\html\time__frequency__transform_8hpp_source.html"
  Delete "$INSTDIR\doc\html\todo.html"
  Delete "$INSTDIR\doc\html\typed__parameter__base_8hpp.html"
  Delete "$INSTDIR\doc\html\typed__parameter__base_8hpp_source.html"
  Delete "$INSTDIR\doc\html\udp__receiver_8hpp.html"
  Delete "$INSTDIR\doc\html\udp__receiver_8hpp_source.html"
  Delete "$INSTDIR\doc\html\udp__sender_8hpp.html"
  Delete "$INSTDIR\doc\html\udp__sender_8hpp_source.html"
  Delete "$INSTDIR\doc\html\utility__functions_8cpp.html"
  Delete "$INSTDIR\doc\html\utility__functions_8cpp.js"
  Delete "$INSTDIR\doc\html\utility__functions_8hpp.html"
  Delete "$INSTDIR\doc\html\utility__functions_8hpp.js"
  Delete "$INSTDIR\doc\html\utility__functions_8hpp_source.html"
  Delete "$INSTDIR\doc\html\vector__conversions_8cpp.html"
  Delete "$INSTDIR\doc\html\vector__conversions_8cpp.js"
  Delete "$INSTDIR\doc\html\vector__conversions_8hpp.html"
  Delete "$INSTDIR\doc\html\vector__conversions_8hpp.js"
  Delete "$INSTDIR\doc\html\vector__conversions_8hpp_source.html"
  Delete "$INSTDIR\doc\html\vector__from__ndarray_8hpp.html"
  Delete "$INSTDIR\doc\html\vector__from__ndarray_8hpp.js"
  Delete "$INSTDIR\doc\html\vector__from__ndarray_8hpp_source.html"
  Delete "$INSTDIR\doc\html\vector__functions_8cpp.html"
  Delete "$INSTDIR\doc\html\vector__functions_8cpp.js"
  Delete "$INSTDIR\doc\html\vector__functions_8hpp.html"
  Delete "$INSTDIR\doc\html\vector__functions_8hpp.js"
  Delete "$INSTDIR\doc\html\vector__functions_8hpp_source.html"
  Delete "$INSTDIR\doc\html\vector__functions__ipp_8cpp.html"
  Delete "$INSTDIR\doc\html\vector__functions__ipp_8cpp.js"
  Delete "$INSTDIR\doc\html\vector__functions__reference_8cpp.html"
  Delete "$INSTDIR\doc\html\vector__functions__reference_8cpp.js"
  Delete "$INSTDIR\doc\html\vector__functions__reference_8hpp.html"
  Delete "$INSTDIR\doc\html\vector__functions__reference_8hpp.js"
  Delete "$INSTDIR\doc\html\vector__functions__reference_8hpp_source.html"
  Delete "$INSTDIR\doc\html\vector__functions__reference__impl_8hpp.html"
  Delete "$INSTDIR\doc\html\vector__functions__reference__impl_8hpp.js"
  Delete "$INSTDIR\doc\html\vector__functions__reference__impl_8hpp_source.html"
  Delete "$INSTDIR\doc\html\vector__parameter_8hpp.html"
  Delete "$INSTDIR\doc\html\vector__parameter_8hpp_source.html"
  Delete "$INSTDIR\doc\html\vector__parameter__config_8cpp.html"
  Delete "$INSTDIR\doc\html\vector__parameter__config_8hpp.html"
  Delete "$INSTDIR\doc\html\vector__parameter__config_8hpp_source.html"
  Delete "$INSTDIR\doc\html\version_8cpp.html"
  Delete "$INSTDIR\doc\html\version_8cpp.js"
  Delete "$INSTDIR\doc\html\version_8hpp.html"
  Delete "$INSTDIR\doc\html\version_8hpp.js"
  Delete "$INSTDIR\doc\html\version_8hpp_source.html"
  Delete "$INSTDIR\doc\html\visr_8cpp.html"
  Delete "$INSTDIR\doc\html\visr_8cpp.js"
  Delete "$INSTDIR\doc\html\visr_renderer_usage.html"
  Delete "$INSTDIR\doc\html\visr__renderer_2main_8cpp.html"
  Delete "$INSTDIR\doc\html\visr__renderer_2main_8cpp.js"
  Delete "$INSTDIR\doc\html\visr__renderer__usage_8dox.html"
  Delete "$INSTDIR\doc\html\wrapper_8cpp.html"
  Delete "$INSTDIR\doc\html\wrapper_8hpp.html"
  Delete "$INSTDIR\doc\html\wrapper_8hpp_source.html"
  Delete "$INSTDIR\doc\html\_all_r_a_d_8cpp.html"
  Delete "$INSTDIR\doc\html\_all_r_a_d_8h.html"
  Delete "$INSTDIR\doc\html\_all_r_a_d_8h_source.html"
  Delete "$INSTDIR\doc\html\_c_a_p_8cpp.html"
  Delete "$INSTDIR\doc\html\_c_a_p_8h.html"
  Delete "$INSTDIR\doc\html\_c_a_p_8h_source.html"
  Delete "$INSTDIR\doc\html\_loudspeaker_array_8cpp.html"
  Delete "$INSTDIR\doc\html\_loudspeaker_array_8h.html"
  Delete "$INSTDIR\doc\html\_loudspeaker_array_8h_source.html"
  Delete "$INSTDIR\doc\html\_v_b_a_p_8h.html"
  Delete "$INSTDIR\doc\html\_v_b_a_p_8h_source.html"
  Delete "$INSTDIR\doc\html\_x_y_z_8h.html"
  Delete "$INSTDIR\doc\html\_x_y_z_8h_source.html"
  Delete "$INSTDIR\loudspeaker_configs"
  Delete "$INSTDIR\config"
  Delete "$INSTDIR\config\bbc"
  Delete "$INSTDIR\config\bbc\bbc-listening-room-full.xml"
  Delete "$INSTDIR\config\bbc\bs2051-0+2+0.xml"
  Delete "$INSTDIR\config\bbc\bs2051-0+5+0.xml"
  Delete "$INSTDIR\config\bbc\bs2051-2+5+0-speakers-s3a.txt"
  Delete "$INSTDIR\config\bbc\bs2051-2+5+0.xml"
  Delete "$INSTDIR\config\bbc\bs2051-3+7+0-speakers-s3a.txt"
  Delete "$INSTDIR\config\bbc\bs2051-4+5+0-speakers-s3a.txt"
  Delete "$INSTDIR\config\bbc\bs2051-4+5+0.xml"
  Delete "$INSTDIR\config\bbc\bs2051-4+5+1-speakers-s3a.txt"
  Delete "$INSTDIR\config\bbc\bs2051-4+9+0-speakers-s3a.txt"
  Delete "$INSTDIR\config\bbc\bs2051-4+9+0.xml"
  Delete "$INSTDIR\config\bbc\bs2051-9+10+3-speakers-s3a.txt"
  Delete "$INSTDIR\config\bbc\bs2051-9+10+3.xml"
  Delete "$INSTDIR\config\bbc\listeningroomspeakers-s3a.txt"
  Delete "$INSTDIR\config\bbc\scripts"
  Delete "$INSTDIR\config\bbc\scripts\createBBCConfigs.m"
  Delete "$INSTDIR\config\bbc\speaker_config_original"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-0+2+0-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-0+5+0-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-2+5+0-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-3+7+0-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-4+5+0-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-4+5+1-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-4+9+0-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\bs2051-9+10+3-speakers.txt"
  Delete "$INSTDIR\config\bbc\speaker_config_original\listeningroomspeakers.txt"
  Delete "$INSTDIR\config\filters"
  Delete "$INSTDIR\config\filters\random_phase_allpass_64ch_1024taps.wav"
  Delete "$INSTDIR\config\filters\random_phase_allpass_64ch_128taps.wav"
  Delete "$INSTDIR\config\filters\random_phase_allpass_64ch_256taps.wav"
  Delete "$INSTDIR\config\filters\random_phase_allpass_64ch_512taps.wav"
  Delete "$INSTDIR\config\filters\random_phase_allpass_64ch_512taps_16bit.wav"
  Delete "$INSTDIR\config\filters\random_phase_allpass_64ch_512taps_32bit.wav"
  Delete "$INSTDIR\config\generic"
  Delete "$INSTDIR\config\generic\bs2051-0+2+0.xml"
  Delete "$INSTDIR\config\generic\bs2051-0+5+0.xml"
  Delete "$INSTDIR\config\generic\bs2051-4+5+0.xml"
  Delete "$INSTDIR\config\generic\bs2051-9+10+3.xml"
  Delete "$INSTDIR\config\generic\bs2051-9+10+3_linear.xml"
  Delete "$INSTDIR\config\generic\decode_N8_P40_t-design_t8_P40.txt"
  Delete "$INSTDIR\config\generic\downmix_matrices"
  Delete "$INSTDIR\config\generic\downmix_matrices\22_2_to_2_0.dat"
  Delete "$INSTDIR\config\generic\downmix_matrices\downmix_coefs.mat"
  Delete "$INSTDIR\config\generic\octahedron.xml"
  Delete "$INSTDIR\config\generic\salfordBRIRs.xml"
  Delete "$INSTDIR\config\generic\scripts"
  Delete "$INSTDIR\config\generic\scripts\createBS2051Configs.m"
  Delete "$INSTDIR\config\generic\stereo.xml"
  Delete "$INSTDIR\config\generic\t-design_t8_P40.txt"
  Delete "$INSTDIR\config\generic\Transaural_39.txt"
  Delete "$INSTDIR\config\generic\Transaural_42.txt"
  Delete "$INSTDIR\config\generic\Transaural_42_cart.txt"
  Delete "$INSTDIR\config\isvr"
  Delete "$INSTDIR\config\isvr\audioIfc"
  Delete "$INSTDIR\config\isvr\audioIfc\jackComplexConf.json"
  Delete "$INSTDIR\config\isvr\audioIfc\jackDefConf.json"
  Delete "$INSTDIR\config\isvr\audioIfc\jackPortConf.json"
  Delete "$INSTDIR\config\isvr\audioIfc\jackSimpleConf.json"
  Delete "$INSTDIR\config\isvr\audioIfc\portAudioDefConf.json"
  Delete "$INSTDIR\config\isvr\audiolab_22speakers.xml"
  Delete "$INSTDIR\config\isvr\audiolab_22speakers_1subwoofer.xml"
  Delete "$INSTDIR\config\isvr\audiolab_39speakers.xml"
  Delete "$INSTDIR\config\isvr\audiolab_39speakers_1subwoofer.xml"
  Delete "$INSTDIR\config\isvr\audiolab_5.1_1subwoofer.xml"
  Delete "$INSTDIR\config\isvr\audiolab_9.1_1subwoofer.xml"
  Delete "$INSTDIR\config\isvr\audiolab_cube_1subwoofer.xml"
  Delete "$INSTDIR\config\isvr\audiolab_stereo_1subwoofer.xml"
  Delete "$INSTDIR\config\isvr\audiolab_stereo_1sub_with_rerouting.xml"
  Delete "$INSTDIR\config\isvr\bak"
  Delete "$INSTDIR\config\isvr\bak\22.1_audiolab.txt"
  Delete "$INSTDIR\config\isvr\bak\5.1_audiolab.txt"
  Delete "$INSTDIR\config\isvr\bak\9.1_audiolab.txt"
  Delete "$INSTDIR\config\isvr\bak\cube_audiolab.txt"
  Delete "$INSTDIR\config\isvr\bak\full_39_loudspeaker_layout_audiolab.txt"
  Delete "$INSTDIR\config\isvr\bak\stereo_audiolab.txt"
  Delete "$INSTDIR\config\isvr\data"
  Delete "$INSTDIR\config\isvr\data\audiolab_gain_delay.xls"
  Delete "$INSTDIR\config\isvr\data\gains_distances.mat"
  Delete "$INSTDIR\config\isvr\scripts"
  Delete "$INSTDIR\config\isvr\scripts\createIsvrConfigs.m"
  Delete "$INSTDIR\config\isvr\scripts\createSphereConfig.m"
  Delete "$INSTDIR\config\isvr\scripts\sparSpeakerPos.m"
  Delete "$INSTDIR\config\isvr\scripts\writeArrayConfigXml.m"
  Delete "$INSTDIR\config\isvr\sphere_30channels_1subwoofer.xml"
  Delete "$INSTDIR\config\isvr\sphere_30channels_1subwoofer_crossover.xml"
  Delete "$INSTDIR\config\surrey"
  Delete "$INSTDIR\config\surrey\2.0_surrey.txt"
  Delete "$INSTDIR\config\surrey\22.0_surrey.txt"
  Delete "$INSTDIR\config\surrey\5.0_surrey.txt"
  Delete "$INSTDIR\config\surrey\9.0_surrey.txt"
  Delete "$INSTDIR\config\surrey\obsolete_configurations"
  Delete "$INSTDIR\config\surrey\obsolete_configurations\22.0_surrey_mod_DM.txt"
  Delete "$INSTDIR\config\surrey\obsolete_configurations\22.1_surrey.txt"
  Delete "$INSTDIR\config\surrey\obsolete_configurations\5.1_surrey.txt"
  Delete "$INSTDIR\config\surrey\obsolete_configurations\9.0_surrey_mod_DM.txt"
  Delete "$INSTDIR\config\surrey\obsolete_configurations\9.1_surrey.txt"
  Delete "$INSTDIR\config\surrey\speaker_configuration_surrey_sphere.xml"
  Delete "$INSTDIR\python_externals"
  Delete "$INSTDIR\python"
  Delete "$INSTDIR\python\audiointerfaces.pyd"
  Delete "$INSTDIR\python\efl.pyd"
  Delete "$INSTDIR\python\objectmodel.pyd"
  Delete "$INSTDIR\python\panning.pyd"
  Delete "$INSTDIR\python\pml.pyd"
  Delete "$INSTDIR\python\rbbl.pyd"
  Delete "$INSTDIR\python\rcl.pyd"
  Delete "$INSTDIR\python\reverbobject.pyd"
  Delete "$INSTDIR\python\rrl.pyd"
  Delete "$INSTDIR\python\signalflows.pyd"
  Delete "$INSTDIR\python\visr.pyd"
  Delete "$INSTDIR\python_package_bst"
  Delete "$INSTDIR\python"
  Delete "$INSTDIR\python\visr_bst"
  Delete "$INSTDIR\python\visr_bst\Changelog"
  Delete "$INSTDIR\python\visr_bst\controllers"
  Delete "$INSTDIR\python\visr_bst\controllers\dynamic_hrir_controller.py"
  Delete "$INSTDIR\python\visr_bst\controllers\virtual_loudspeaker_controller.py"
  Delete "$INSTDIR\python\visr_bst\data"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_all_speakers.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemA.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemB.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemC.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemD.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemE.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemF.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemG.xml"
  Delete "$INSTDIR\python\visr_bst\data\loudspeaker_configs\bbcrdlr_systemH.xml"
  Delete "$INSTDIR\python\visr_bst\data\sofa"
  Delete "$INSTDIR\python\visr_bst\data\sofa\brir"
  Delete "$INSTDIR\python\visr_bst\data\sofa\brir\.gitignore"
  Delete "$INSTDIR\python\visr_bst\data\sofa\hoa2binaural"
  Delete "$INSTDIR\python\visr_bst\data\sofa\hoa2binaural\.gitignore"
  Delete "$INSTDIR\python\visr_bst\data\sofa\hrir"
  Delete "$INSTDIR\python\visr_bst\data\sofa\hrir\.gitignore"
  Delete "$INSTDIR\python\visr_bst\examples"
  Delete "$INSTDIR\python\visr_bst\examples\.gitignore"
  Delete "$INSTDIR\python\visr_bst\examples\run_realtime_dynamic_hrir_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\run_realtime_hoa_binaural_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\run_realtime_hoa_object_to_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\run_realtime_virtual_loudspeaker_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\simulate_dynamic_hrir_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\simulate_hoa_binaural_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\simulate_hoa_object_to_binaural_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\simulate_object_to_virtual_loudspeaker_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\simulate_virtual_loudspeaker_renderer.py"
  Delete "$INSTDIR\python\visr_bst\examples\simulate_virtual_loudspeaker_renderer_late_static.py"
  Delete "$INSTDIR\python\visr_bst\examples\test_dynamic_hrir_controller.py"
  Delete "$INSTDIR\python\visr_bst\examples\test_virtual_loudspeaker_controller.py"
  Delete "$INSTDIR\python\visr_bst\hoa_components"
  Delete "$INSTDIR\python\visr_bst\hoa_components\hoa_coefficient_rotation.py"
  Delete "$INSTDIR\python\visr_bst\hoa_components\hoa_object_encoder.py"
  Delete "$INSTDIR\python\visr_bst\hoa_components\hoa_rotation_matrix_calculator.py"
  Delete "$INSTDIR\python\visr_bst\hoa_components\__init__.py"
  Delete "$INSTDIR\python\visr_bst\LICENSE.md"
  Delete "$INSTDIR\python\visr_bst\README.md"
  Delete "$INSTDIR\python\visr_bst\realtime_renderers"
  Delete "$INSTDIR\python\visr_bst\realtime_renderers\realtime_dynamic_hrir_renderer.py"
  Delete "$INSTDIR\python\visr_bst\realtime_renderers\realtime_hoa_binaural_renderer.py"
  Delete "$INSTDIR\python\visr_bst\realtime_renderers\realtime_hoa_object_to_binaural_renderer.py"
  Delete "$INSTDIR\python\visr_bst\realtime_renderers\realtime_virtual_loudspeaker_renderer.py"
  Delete "$INSTDIR\python\visr_bst\renderers"
  Delete "$INSTDIR\python\visr_bst\renderers\dynamic_hrir_renderer.py"
  Delete "$INSTDIR\python\visr_bst\renderers\hoa_binaural_renderer.py"
  Delete "$INSTDIR\python\visr_bst\renderers\hoa_object_to_binaural_renderer.py"
  Delete "$INSTDIR\python\visr_bst\renderers\object_to_virtual_loudspeaker_renderer.py"
  Delete "$INSTDIR\python\visr_bst\renderers\virtual_loudspeaker_renderer.py"
  Delete "$INSTDIR\python\visr_bst\tracker"
  Delete "$INSTDIR\python\visr_bst\tracker\razor_ahrs.py"
  Delete "$INSTDIR\python\visr_bst\tracker\razor_ahrs_with_udp_calibration_trigger.py"
  Delete "$INSTDIR\python\visr_bst\tracker\__init__.py"
  Delete "$INSTDIR\python\visr_bst\util"
  Delete "$INSTDIR\python\visr_bst\util\read_sofa_file.py"
  Delete "$INSTDIR\python\visr_bst\util\real_spherical_harmonics.py"
  Delete "$INSTDIR\python\visr_bst\util\rotation_functions.py"
  Delete "$INSTDIR\python\visr_bst\util\sofa_extract_delay.py"
  Delete "$INSTDIR\python\visr_bst\util\spherical_harmonics_rotation.py"
  Delete "$INSTDIR\python\visr_bst\util\__init__.py"
  Delete "$INSTDIR\python\visr_bst\__init__.py"
  Delete "$INSTDIR\python_package_metadapter"
  Delete "$INSTDIR\python"
  Delete "$INSTDIR\python\metadapter"
  Delete "$INSTDIR\python\metadapter\.gitignore"
  Delete "$INSTDIR\python\metadapter\condition_processors"
  Delete "$INSTDIR\python\metadapter\condition_processors\condition_processor_factory.py"
  Delete "$INSTDIR\python\metadapter\condition_processors\select_group_condition.py"
  Delete "$INSTDIR\python\metadapter\condition_processors\__init__.py"
  Delete "$INSTDIR\python\metadapter\config"
  Delete "$INSTDIR\python\metadapter\config\C4_demo1_init.xml"
  Delete "$INSTDIR\python\metadapter\config\channel_object.xml"
  Delete "$INSTDIR\python\metadapter\config\conditional_empty_branch.xml"
  Delete "$INSTDIR\python\metadapter\config\conditional_no_default.xml"
  Delete "$INSTDIR\python\metadapter\config\conditional_reordered_branches.xml"
  Delete "$INSTDIR\python\metadapter\config\conditional_single_branch.xml"
  Delete "$INSTDIR\python\metadapter\config\conditional_single_branch_no_merge.xml"
  Delete "$INSTDIR\python\metadapter\config\conditional_test1.xml"
  Delete "$INSTDIR\python\metadapter\config\diffuse_source_test.xml"
  Delete "$INSTDIR\python\metadapter\config\envelopment.xml"
  Delete "$INSTDIR\python\metadapter\config\example_processing_chain_1.xml"
  Delete "$INSTDIR\python\metadapter\config\example_processing_chain_1_quot.xml"
  Delete "$INSTDIR\python\metadapter\config\mdo.xml"
  Delete "$INSTDIR\python\metadapter\config\mdoAzimuth.xml"
  Delete "$INSTDIR\python\metadapter\config\mdoPhones.xml"
  Delete "$INSTDIR\python\metadapter\config\mdo_production.xml"
  Delete "$INSTDIR\python\metadapter\config\narrative_importance.xml"
  Delete "$INSTDIR\python\metadapter\config\nested_sequences.xml"
  Delete "$INSTDIR\python\metadapter\config\processing_chain_no_sequence_layer.xml"
  Delete "$INSTDIR\python\metadapter\config\radio_drama_adaptation_demo.xml"
  Delete "$INSTDIR\python\metadapter\config\radio_drama_adaptation_FOW.xml"
  Delete "$INSTDIR\python\metadapter\config\reverb_config_notes.txt"
  Delete "$INSTDIR\python\metadapter\config\room_acoustic_adaptation_test.xml"
  Delete "$INSTDIR\python\metadapter\config\semantic_stereo.xml"
  Delete "$INSTDIR\python\metadapter\config\simple_mdo.xml"
  Delete "$INSTDIR\python\metadapter\config\smpte_demo.xml"
  Delete "$INSTDIR\python\metadapter\config\source_merge_1.xml"
  Delete "$INSTDIR\python\metadapter\config\speech_steering.xml"
  Delete "$INSTDIR\python\metadapter\config\split_sink_1.xml"
  Delete "$INSTDIR\python\metadapter\config\static_point_source_extent.xml"
  Delete "$INSTDIR\python\metadapter\config\test_add_advanced_metadata.xml"
  Delete "$INSTDIR\python\metadapter\config\test_add_mdo_metadata.xml"
  Delete "$INSTDIR\python\metadapter\config\test_add_reverb.xml"
  Delete "$INSTDIR\python\metadapter\config\test_add_reverb_hoa.xml"
  Delete "$INSTDIR\python\metadapter\config\test_edit_distance.xml"
  Delete "$INSTDIR\python\metadapter\config\test_edit_reverb.xml"
  Delete "$INSTDIR\python\metadapter\config\test_edit_reverb_frame.xml"
  Delete "$INSTDIR\python\metadapter\config\test_edit_reverb_scene.xml"
  Delete "$INSTDIR\python\metadapter\config\TMMExperiment2config.xml"
  Delete "$INSTDIR\python\metadapter\config\TMMExperiment3config.xml"
  Delete "$INSTDIR\python\metadapter\core"
  Delete "$INSTDIR\python\metadapter\core\base_processor_interface.py"
  Delete "$INSTDIR\python\metadapter\core\composite_conditional.py"
  Delete "$INSTDIR\python\metadapter\core\composite_sequence.py"
  Delete "$INSTDIR\python\metadapter\core\composite_source_merge.py"
  Delete "$INSTDIR\python\metadapter\core\composite_split_sink.py"
  Delete "$INSTDIR\python\metadapter\core\condition_processor_interface.py"
  Delete "$INSTDIR\python\metadapter\core\instantiate_subflow.py"
  Delete "$INSTDIR\python\metadapter\core\merge_processor_interface.py"
  Delete "$INSTDIR\python\metadapter\core\metadapter_engine.py"
  Delete "$INSTDIR\python\metadapter\core\metadata_processor_factory.py"
  Delete "$INSTDIR\python\metadapter\core\sequence_processor_interface.py"
  Delete "$INSTDIR\python\metadapter\core\sink_processor_interface.py"
  Delete "$INSTDIR\python\metadapter\core\source_processor_interface.py"
  Delete "$INSTDIR\python\metadapter\examples"
  Delete "$INSTDIR\python\metadapter\examples\test_metadapter.py"
  Delete "$INSTDIR\python\metadapter\max"
  Delete "$INSTDIR\python\metadapter\max\AddReverbOSCController.maxpat"
  Delete "$INSTDIR\python\metadapter\max\ReverbControllerv0.maxpat"
  Delete "$INSTDIR\python\metadapter\merge_processors"
  Delete "$INSTDIR\python\metadapter\merge_processors\branch_order_merge_processor.py"
  Delete "$INSTDIR\python\metadapter\merge_processors\merge_processor_factory.py"
  Delete "$INSTDIR\python\metadapter\merge_processors\__init__.py"
  Delete "$INSTDIR\python\metadapter\metadapter.py"
  Delete "$INSTDIR\python\metadapter\networked"
  Delete "$INSTDIR\python\metadapter\networked\engine.py"
  Delete "$INSTDIR\python\metadapter\processors"
  Delete "$INSTDIR\python\metadapter\processors\adapt_diffuseness_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\adapt_position_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\addadvancedmetadata_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\add_mdo_metadata_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\add_objects_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\add_reverb_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\attenuate_action_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\az_remap.py"
  Delete "$INSTDIR\python\metadapter\processors\change_volume_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\compress_above_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\compress_below_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\compress_height_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\edit_distance_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\edit_reverb_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\envelopmentopt_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\envelopment_functions.py"
  Delete "$INSTDIR\python\metadapter\processors\envelopment_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\fakegroups_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\globaleq_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\group_adapt_radius_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\group_volume_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\mdoAzimuth_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\mdoPhones_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\mdoProduction_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\mdo_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\narrativeimportance_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\Nuendo_height_fix_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\print_advanced_metadata_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\print_object_coordinate.py"
  Delete "$INSTDIR\python\metadapter\processors\reverb"
  Delete "$INSTDIR\python\metadapter\processors\reverb\AES142Library.json"
  Delete "$INSTDIR\python\metadapter\processors\reverb\AES142Library1.json"
  Delete "$INSTDIR\python\metadapter\processors\reverb\BFormatParamTest.json"
  Delete "$INSTDIR\python\metadapter\processors\reverb\JAESLibrary.json"
  Delete "$INSTDIR\python\metadapter\processors\room"
  Delete "$INSTDIR\python\metadapter\processors\room\ArrayConfig.xml"
  Delete "$INSTDIR\python\metadapter\processors\room\RoomArrayResponse.json"
  Delete "$INSTDIR\python\metadapter\processors\room_acoustic_adaptation_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\routetosoundbar_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\semantic_stereo_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\solo_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\spat_message_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\spread_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\TMMExperiment2_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\TMMExperiment3_MUSHRA_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\TMMExperiment3_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\TMMGeneric_processor.py"
  Delete "$INSTDIR\python\metadapter\processors\utilities.py"
  Delete "$INSTDIR\python\metadapter\processors\__init__.py"
  Delete "$INSTDIR\python\metadapter\python_examples"
  Delete "$INSTDIR\python\metadapter\python_examples\reactor_udp_tutorial.py"
  Delete "$INSTDIR\python\metadapter\python_examples\README.txt"
  Delete "$INSTDIR\python\metadapter\python_examples\udpreactor.py"
  Delete "$INSTDIR\python\metadapter\sink_processors"
  Delete "$INSTDIR\python\metadapter\sink_processors\print_scene_processor.py"
  Delete "$INSTDIR\python\metadapter\sink_processors\sink_processor_factory.py"
  Delete "$INSTDIR\python\metadapter\sink_processors\__init__.py"
  Delete "$INSTDIR\python\metadapter\source_processors"
  Delete "$INSTDIR\python\metadapter\source_processors\source_processor_factory.py"
  Delete "$INSTDIR\python\metadapter\source_processors\static_scene_source.py"
  Delete "$INSTDIR\python\metadapter\source_processors\__init__.py"
  Delete "$INSTDIR\python\metadapter\templates"
  Delete "$INSTDIR\python\metadapter\templates\add_objects.py"
  Delete "$INSTDIR\python\metadapter\templates\modify_all_objects.py"
  Delete "$INSTDIR\python\metadapter\thirdparty"
  Delete "$INSTDIR\python\metadapter\thirdparty\OSC.py"
  Delete "$INSTDIR\python\metadapter\thirdparty\thirdparty"
  Delete "$INSTDIR\python\metadapter\thirdparty\thirdparty\OSC.py"
  Delete "$INSTDIR\python\metadapter\thirdparty\thirdparty\__init__.py"
  Delete "$INSTDIR\python\metadapter\thirdparty\__init__.py"
  Delete "$INSTDIR\python\metadapter\visr"
  Delete "$INSTDIR\python\metadapter\visr\component.py"
  Delete "$INSTDIR\python\metadapter\__init__.py"
  Delete "$INSTDIR\python_templates"
  Delete "$INSTDIR\python"
  Delete "$INSTDIR\python\templates"
  Delete "$INSTDIR\python\templates\audio_meter.py"
  Delete "$INSTDIR\python\templates\audio_meter_evaluation.py"
  Delete "$INSTDIR\python\templates\audio_meter_gui.py"
  Delete "$INSTDIR\shared_libraries"
  Delete "$INSTDIR\lib"
  Delete "$INSTDIR\lib\apputilities.dll"
  Delete "$INSTDIR\lib\audiointerfaces.dll"
  Delete "$INSTDIR\lib\efl.dll"
  Delete "$INSTDIR\lib\objectmodel.dll"
  Delete "$INSTDIR\lib\panning.dll"
  Delete "$INSTDIR\lib\pml.dll"
  Delete "$INSTDIR\lib\pythoncomponents.dll"
  Delete "$INSTDIR\lib\pythonsupport.dll"
  Delete "$INSTDIR\lib\rbbl.dll"
  Delete "$INSTDIR\lib\rcl.dll"
  Delete "$INSTDIR\lib\reverbobject.dll"
  Delete "$INSTDIR\lib\rrl.dll"
  Delete "$INSTDIR\lib\signalflows.dll"
  Delete "$INSTDIR\lib\signalflowspython.dll"
  Delete "$INSTDIR\lib\visr.dll"
  Delete "$INSTDIR\standalone_applications"
  Delete "$INSTDIR\bin"
  Delete "$INSTDIR\bin\baseline_renderer.exe"
  Delete "$INSTDIR\bin\matrix_convolver.exe"
  Delete "$INSTDIR\bin\network_audio_streamer.exe"
  Delete "$INSTDIR\bin\python_runner.exe"
  Delete "$INSTDIR\bin\visr_renderer.exe"
  Delete "$INSTDIR\thirdparty_libraries"
  Delete "$INSTDIR\3rd"
  Delete "$INSTDIR\3rd\boost_atomic-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_chrono-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_date_time-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_filesystem-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_program_options-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_regex-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_system-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_thread-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\boost_timer-vc140-mt-x64-1_67.dll"
  Delete "$INSTDIR\3rd\libsndfile-1.dll"
  Delete "$INSTDIR\3rd\portaudio_x64.dll"
  Delete "$INSTDIR\user_documentation_pdf"
  Delete "$INSTDIR\doc"
  Delete "$INSTDIR\doc\userdoc-0.11.1.pdf"

  RMDir "$INSTDIR\doc"
  RMDir "$INSTDIR\api_documentation_pdf"
  RMDir "$INSTDIR\base"
  RMDir "$INSTDIR\cmake"
  RMDir "$INSTDIR\include\visr\libapputilities"
  RMDir "$INSTDIR\include\visr\libaudiointerfaces"
  RMDir "$INSTDIR\include\visr\libefl"
  RMDir "$INSTDIR\include\visr\libobjectmodel"
  RMDir "$INSTDIR\include\visr\libpanning"
  RMDir "$INSTDIR\include\visr\libpml"
  RMDir "$INSTDIR\include\visr\libpythoncomponents"
  RMDir "$INSTDIR\include\visr\libpythonsupport"
  RMDir "$INSTDIR\include\visr\librbbl"
  RMDir "$INSTDIR\include\visr\librcl"
  RMDir "$INSTDIR\include\visr\libreverbobject"
  RMDir "$INSTDIR\include\visr\librrl"
  RMDir "$INSTDIR\include\visr\libsignalflows"
  RMDir "$INSTDIR\include\visr\libvisr\detail"
  RMDir "$INSTDIR\include\visr\libvisr"
  RMDir "$INSTDIR\include\visr"
  RMDir "$INSTDIR\include"
  RMDir "$INSTDIR\lib"
  RMDir "$INSTDIR\development_files"
  RMDir "$INSTDIR\doc\html\search"
  RMDir "$INSTDIR\doc\html"
  RMDir "$INSTDIR\doc"
  RMDir "$INSTDIR\doxygen_documentation_html"
  RMDir "$INSTDIR\config\bbc\scripts"
  RMDir "$INSTDIR\config\bbc\speaker_config_original"
  RMDir "$INSTDIR\config\bbc"
  RMDir "$INSTDIR\config\filters"
  RMDir "$INSTDIR\config\generic\downmix_matrices"
  RMDir "$INSTDIR\config\generic\scripts"
  RMDir "$INSTDIR\config\generic"
  RMDir "$INSTDIR\config\isvr\audioIfc"
  RMDir "$INSTDIR\config\isvr\bak"
  RMDir "$INSTDIR\config\isvr\data"
  RMDir "$INSTDIR\config\isvr\scripts"
  RMDir "$INSTDIR\config\isvr"
  RMDir "$INSTDIR\config\surrey\obsolete_configurations"
  RMDir "$INSTDIR\config\surrey"
  RMDir "$INSTDIR\config"
  RMDir "$INSTDIR\loudspeaker_configs"
  RMDir "$INSTDIR\python"
  RMDir "$INSTDIR\python_externals"
  RMDir "$INSTDIR\python\visr_bst\controllers"
  RMDir "$INSTDIR\python\visr_bst\data\loudspeaker_configs"
  RMDir "$INSTDIR\python\visr_bst\data\sofa\brir"
  RMDir "$INSTDIR\python\visr_bst\data\sofa\hoa2binaural"
  RMDir "$INSTDIR\python\visr_bst\data\sofa\hrir"
  RMDir "$INSTDIR\python\visr_bst\data\sofa"
  RMDir "$INSTDIR\python\visr_bst\data"
  RMDir "$INSTDIR\python\visr_bst\examples"
  RMDir "$INSTDIR\python\visr_bst\hoa_components"
  RMDir "$INSTDIR\python\visr_bst\realtime_renderers"
  RMDir "$INSTDIR\python\visr_bst\renderers"
  RMDir "$INSTDIR\python\visr_bst\tracker"
  RMDir "$INSTDIR\python\visr_bst\util"
  RMDir "$INSTDIR\python\visr_bst"
  RMDir "$INSTDIR\python"
  RMDir "$INSTDIR\python_package_bst"
  RMDir "$INSTDIR\python\metadapter\condition_processors"
  RMDir "$INSTDIR\python\metadapter\config"
  RMDir "$INSTDIR\python\metadapter\core"
  RMDir "$INSTDIR\python\metadapter\examples"
  RMDir "$INSTDIR\python\metadapter\max"
  RMDir "$INSTDIR\python\metadapter\merge_processors"
  RMDir "$INSTDIR\python\metadapter\networked"
  RMDir "$INSTDIR\python\metadapter\processors\reverb"
  RMDir "$INSTDIR\python\metadapter\processors\room"
  RMDir "$INSTDIR\python\metadapter\processors"
  RMDir "$INSTDIR\python\metadapter\python_examples"
  RMDir "$INSTDIR\python\metadapter\sink_processors"
  RMDir "$INSTDIR\python\metadapter\source_processors"
  RMDir "$INSTDIR\python\metadapter\templates"
  RMDir "$INSTDIR\python\metadapter\thirdparty\thirdparty"
  RMDir "$INSTDIR\python\metadapter\thirdparty"
  RMDir "$INSTDIR\python\metadapter\visr"
  RMDir "$INSTDIR\python\metadapter"
  RMDir "$INSTDIR\python"
  RMDir "$INSTDIR\python_package_metadapter"
  RMDir "$INSTDIR\python\templates"
  RMDir "$INSTDIR\python"
  RMDir "$INSTDIR\python_templates"
  RMDir "$INSTDIR\lib"
  RMDir "$INSTDIR\shared_libraries"
  RMDir "$INSTDIR\bin"
  RMDir "$INSTDIR\standalone_applications"
  RMDir "$INSTDIR\3rd"
  RMDir "$INSTDIR\thirdparty_libraries"
  RMDir "$INSTDIR\doc"
  RMDir "$INSTDIR\user_documentation_pdf"


!ifdef CPACK_NSIS_ADD_REMOVE
  ;Remove the add/remove program
  Delete "$INSTDIR\AddRemove.exe"
!endif

  ;Remove the uninstaller itself.
  Delete "$INSTDIR\Uninstall.exe"
  DeleteRegKey SHCTX "Software\Microsoft\Windows\CurrentVersion\Uninstall\VISR-0.11.1"

  ;Remove the installation directory if it is empty.
  RMDir "$INSTDIR"

  ; Remove the registry entries.
  DeleteRegKey SHCTX "Software\ISVR\VISR-0.11.1"

  ; Removes all optional components
  !insertmacro SectionList "RemoveSection_CPack"

  !insertmacro MUI_STARTMENU_GETFOLDER Application $MUI_TEMP

  Delete "$SMPROGRAMS\$MUI_TEMP\Uninstall.lnk"



  ;Delete empty start menu parent diretories
  StrCpy $MUI_TEMP "$SMPROGRAMS\$MUI_TEMP"

  startMenuDeleteLoop:
    ClearErrors
    RMDir $MUI_TEMP
    GetFullPathName $MUI_TEMP "$MUI_TEMP\.."

    IfErrors startMenuDeleteLoopDone

    StrCmp "$MUI_TEMP" "$SMPROGRAMS" startMenuDeleteLoopDone startMenuDeleteLoop
  startMenuDeleteLoopDone:

  ; If the user changed the shortcut, then untinstall may not work. This should
  ; try to fix it.
  StrCpy $MUI_TEMP "$START_MENU"
  Delete "$SMPROGRAMS\$MUI_TEMP\Uninstall.lnk"


  ;Delete empty start menu parent diretories
  StrCpy $MUI_TEMP "$SMPROGRAMS\$MUI_TEMP"

  secondStartMenuDeleteLoop:
    ClearErrors
    RMDir $MUI_TEMP
    GetFullPathName $MUI_TEMP "$MUI_TEMP\.."

    IfErrors secondStartMenuDeleteLoopDone

    StrCmp "$MUI_TEMP" "$SMPROGRAMS" secondStartMenuDeleteLoopDone secondStartMenuDeleteLoop
  secondStartMenuDeleteLoopDone:

  DeleteRegKey /ifempty SHCTX "Software\ISVR\VISR-0.11.1"

  Push $INSTDIR\bin
  StrCmp $DO_NOT_ADD_TO_PATH_ "1" doNotRemoveFromPath 0
    Call un.RemoveFromPath
  doNotRemoveFromPath:
SectionEnd

;--------------------------------
; determine admin versus local install
; Is install for "AllUsers" or "JustMe"?
; Default to "JustMe" - set to "AllUsers" if admin or on Win9x
; This function is used for the very first "custom page" of the installer.
; This custom page does not show up visibly, but it executes prior to the
; first visible page and sets up $INSTDIR properly...
; Choose different default installation folder based on SV_ALLUSERS...
; "Program Files" for AllUsers, "My Documents" for JustMe...

Function .onInit
  StrCmp "" "ON" 0 inst

  ReadRegStr $0 HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\VISR-0.11.1" "UninstallString"
  StrCmp $0 "" inst

  MessageBox MB_YESNOCANCEL|MB_ICONEXCLAMATION \
  "VISR-0.11.1 is already installed. $\n$\nDo you want to uninstall the old version before installing the new one?" \
  /SD IDYES IDYES uninst IDNO inst
  Abort

;Run the uninstaller
uninst:
  ClearErrors
  StrLen $2 "\Uninstall.exe"
  StrCpy $3 $0 -$2 # remove "\Uninstall.exe" from UninstallString to get path
  ExecWait '"$0" /S _?=$3' ;Do not copy the uninstaller to a temp file

  IfErrors uninst_failed inst
uninst_failed:
  MessageBox MB_OK|MB_ICONSTOP "Uninstall failed."
  Abort


inst:
  ; Reads components status for registry
  !insertmacro SectionList "InitSection"

  ; check to see if /D has been used to change
  ; the install directory by comparing it to the
  ; install directory that is expected to be the
  ; default
  StrCpy $IS_DEFAULT_INSTALLDIR 0
  StrCmp "$INSTDIR" "$PROGRAMFILES64\VISR-0.11.1" 0 +2
    StrCpy $IS_DEFAULT_INSTALLDIR 1

  StrCpy $SV_ALLUSERS "JustMe"
  ; if default install dir then change the default
  ; if it is installed for JustMe
  StrCmp "$IS_DEFAULT_INSTALLDIR" "1" 0 +2
    StrCpy $INSTDIR "$DOCUMENTS\VISR-0.11.1"

  ClearErrors
  UserInfo::GetName
  IfErrors noLM
  Pop $0
  UserInfo::GetAccountType
  Pop $1
  StrCmp $1 "Admin" 0 +4
    SetShellVarContext all
    ;MessageBox MB_OK 'User "$0" is in the Admin group'
    StrCpy $SV_ALLUSERS "AllUsers"
    Goto done
  StrCmp $1 "Power" 0 +4
    SetShellVarContext all
    ;MessageBox MB_OK 'User "$0" is in the Power Users group'
    StrCpy $SV_ALLUSERS "AllUsers"
    Goto done

  noLM:
    StrCpy $SV_ALLUSERS "AllUsers"
    ;Get installation folder from registry if available

  done:
  StrCmp $SV_ALLUSERS "AllUsers" 0 +3
    StrCmp "$IS_DEFAULT_INSTALLDIR" "1" 0 +2
      StrCpy $INSTDIR "$PROGRAMFILES64\VISR-0.11.1"

  StrCmp "" "ON" 0 noOptionsPage
    !insertmacro MUI_INSTALLOPTIONS_EXTRACT "NSIS.InstallOptions.ini"

  noOptionsPage:
FunctionEnd
